@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: gmt.h,v 1.203 2011/03/03 21:02:50 guru Exp $
+ *	$Id: gmt.h 9740 2012-03-26 21:43:15Z remko $
  *
- *	Copyright (c) 1991-2011 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2012 by P. Wessel and W. H. F. Smith
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -100,7 +100,6 @@ extern "C" {
  *			GMT CONSTANTS MACRO DEFINITIONS
  *--------------------------------------------------------------------*/
 
-#define BOOLEAN GMT_LONG
 #ifndef TRUE
 #define TRUE 1
 #endif
@@ -664,6 +663,10 @@ EXTERN_MSC GMT_LONG GMT_min_meminc, GMT_max_meminc;
 extern struct MEMORY_TRACKER *GMT_mem_keeper;
 #endif
 
+/* Define GMT_version here, and not in gmt_version.h */
+
+EXTERN_MSC char *GMT_version ();
+
 #endif  /* _GMT_H */
 #ifdef __cplusplus
 }
