@@ -1,12 +1,12 @@
 /*--------------------------------------------------------------------
- *	$Id: gmt_calclock.c,v 1.47 2006/04/12 13:11:26 remko Exp $
+ *	$Id: gmt_calclock.c 9656 2012-01-13 18:29:02Z pwessel $
  *
- *	Copyright (c) 1991-2006 by P. Wessel and W. H. F. Smith
- *	See COPYING file for copying and redistribution conditions.
+ *	Copyright (c) 1991-2012 by P. Wessel and W. H. F. Smith
+ *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
- *	the Free Software Foundation; version 2 of the License.
+ *	the Free Software Foundation; version 2 or any later version.
  *
  *	This program is distributed in the hope that it will be useful,
  *	but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -35,62 +35,51 @@
 #define GMT_WITH_NO_PS
 #include "gmt.h"
 
-BOOLEAN want_iso = FALSE;	/* Temporary to test compilation.  Delete this.  */
+GMT_LONG want_iso = FALSE;	/* Temporary to test compilation.  Delete this.  */
 
 /* Functions to assemble/disassemble a continuous
-variable (GMT_dtime) and a calendar day (GMT_cal_rd)
+variable (double) and a calendar day (GMT_cal_rd)
 and time of day (double secs):  
 */
 
-GMT_dtime GMT_rdc2dt (GMT_cal_rd rd, double secs) {
-#ifndef OLDCAL
+double GMT_rdc2dt (GMT_cal_rd rd, double secs) {
 /*	Given rata die rd and double seconds, return 
 	time in TIME_UNIT relative to chosen TIME_EPOCH  */
 	double f_days;
-	f_days = (rd - GMT_time_system[gmtdefs.time_system].rata_die - GMT_time_system[gmtdefs.time_system].epoch_t0);
-	if (gmtdefs.time_system == 6)  f_days += GMT_GCAL_EPOCH;
-	return ((f_days * GMT_DAY2SEC_F  + secs) * GMT_time_system[gmtdefs.time_system].i_scale);
-#else
-/*	Given rata die rd and double seconds, return 
-	GMT_dtime value of time  */
-	return ((GMT_dtime)(GMT_DAY2SEC_F*(rd-GMT_GCAL_EPOCH) + secs));
-#endif
+	f_days = (rd - gmtdefs.time_system.rata_die - gmtdefs.time_system.epoch_t0);
+	return ((f_days * GMT_DAY2SEC_F  + secs) * gmtdefs.time_system.i_scale);
 }
 
-int	splitinteger(double value, int epsilon, double *doublepart) {
+GMT_LONG	splitinteger(double value, GMT_LONG epsilon, double *doublepart) {
 	/* Split value into integer and and floating part for date usage.
-	   While the integer can be negative, doublpart is always >= 0
+	   While the integer can be negative, doublepart is always >= 0
+	   When doublepart is close to 0 or close to epsilon, doublepart is set
+	   to zero and in the latter case, splitinteger is raised by one.
+	   This makes value "snap" to multiples of epsilon.
 	*/
-	int i;
+	GMT_LONG i;
 
-	i = (int) (value/(double)epsilon);
-	*doublepart = value - ((double) i)*((double)epsilon);
-	if ((*doublepart) < 0.0) {	/* For negative relative time we must shift i by -1 */
-		i--;
-		(*doublepart) += (double)epsilon;
+	i = (GMT_LONG) floor(value/(double)epsilon);
+	*doublepart = value - ((double)i)*((double)epsilon);
+	if ((*doublepart) < GMT_SMALL)
+		*doublepart = 0.0;	/* Snap to the lower integer */
+	else if ((double)epsilon - (*doublepart) < GMT_SMALL) {
+		i++;			/* Snap to the higher integer */
+		*doublepart = 0.0;
 	}
 	return i;
 }
 
-void	GMT_dt2rdc (GMT_dtime t, GMT_cal_rd *rd, double *s) {
+void	GMT_dt2rdc (double t, GMT_cal_rd *rd, double *s) {
 
-	int i;
+	GMT_LONG i;
 
-#ifndef OLDCAL
 /*	Given time in TIME_UNIT relative to TIME_EPOCH, load rata die of this day
 	in rd and the seconds since the start of that day in s.  */
 	double t_sec;
-	t_sec = (t * GMT_time_system[gmtdefs.time_system].scale + GMT_time_system[gmtdefs.time_system].epoch_t0 * GMT_DAY2SEC_F);
-	i = splitinteger(t_sec, 86400, s) + GMT_time_system[gmtdefs.time_system].rata_die;
-	if (gmtdefs.time_system == 6) i -= GMT_GCAL_EPOCH;
+	t_sec = (t * gmtdefs.time_system.scale + gmtdefs.time_system.epoch_t0 * GMT_DAY2SEC_F);
+	i = splitinteger(t_sec, 86400, s) + gmtdefs.time_system.rata_die;
 	*rd = (GMT_cal_rd)(i);
-#else
-/*	Given GMT_dtime value t, load calendar data of that day
-	in rd and the seconds since the start of that day in
-	s.  */
-	i = splitinteger(t, 86400, s);
-	*rd = (GMT_cal_rd)(i+GMT_GCAL_EPOCH);
-#endif
 }
 
 
@@ -105,12 +94,12 @@ void	GMT_dt2rdc (GMT_dtime t, GMT_cal_rd *rd, double *s) {
 	print a domain error and return something anyway.
 */
 
-int	GMT_cal_imod (int x, int y) {
+GMT_LONG	GMT_cal_imod (GMT_LONG x, GMT_LONG y) {
 	if (y == 0) {
 		fprintf (stderr, "GMT_cal_imod:  DOMAIN ERROR.\n");
 		return (x);
 	}
-	return (x - y * ( (int)floor((double)x/(double)y) ) );
+	return (x - y * ( (GMT_LONG)floor((double)x/(double)y) ) );
 }
 
 /* kday functions:
@@ -123,28 +112,28 @@ int	GMT_cal_imod (int x, int y) {
 	through the fact that the related day falls on a given
 	kday of the week.  */
 
-GMT_cal_rd GMT_kday_on_or_before (GMT_cal_rd date, int kday) {
+GMT_cal_rd GMT_kday_on_or_before (GMT_cal_rd date, GMT_LONG kday) {
 /*	Given date and kday, return the date of the nearest kday
 	on or before the given date.
 */
-	return ((GMT_cal_rd)(date - GMT_cal_imod((date-kday), 7)));
+	return ((GMT_cal_rd)(date - GMT_cal_imod((GMT_LONG)(date-kday), 7)));
 }
 
-GMT_cal_rd GMT_kday_after (GMT_cal_rd date, int kday) {
+GMT_cal_rd GMT_kday_after (GMT_cal_rd date, GMT_LONG kday) {
 /*	Given date and kday, return the date of the nearest kday
 	after the given date.
 */
 	return (GMT_kday_on_or_before(date+7, kday));
 }
 
-GMT_cal_rd GMT_kday_before (GMT_cal_rd date, int kday) {
+GMT_cal_rd GMT_kday_before (GMT_cal_rd date, GMT_LONG kday) {
 /*	Given date and kday, return the date of the nearest kday
 	before the given date.
 */
 	return (GMT_kday_on_or_before(date-1, kday));
 }
 
-GMT_cal_rd GMT_nth_kday (int n, int kday, GMT_cal_rd date) {
+GMT_cal_rd GMT_nth_kday (GMT_LONG n, GMT_LONG kday, GMT_cal_rd date) {
 /*	Given date, kday, and n, return the date of the n'th
 	kday before or after the given date, according to the
 	sign of n.
@@ -160,12 +149,12 @@ GMT_cal_rd GMT_nth_kday (int n, int kday, GMT_cal_rd date) {
 
 /* Proleptic Gregorian Calendar operations  */
 
-BOOLEAN	GMT_is_gleap (int gyear) {
+GMT_LONG	GMT_is_gleap (GMT_LONG gyear) {
 /*	Given integer proleptic gregorian calendar year,
 	return TRUE if it is a Gregorian leap year; 
 	else return FALSE.  */
 	
-	int	y400;
+	GMT_LONG	y400;
 	
 	if (GMT_cal_imod (gyear, 4) != 0) return (FALSE);
 	
@@ -178,12 +167,12 @@ BOOLEAN	GMT_is_gleap (int gyear) {
 	return (TRUE);
 }
 
-GMT_cal_rd GMT_rd_from_gymd (int gy, int gm, int gd) {
+GMT_cal_rd GMT_rd_from_gymd (GMT_LONG gy, GMT_LONG gm, GMT_LONG gd) {
 /*	Given gregorian calendar year, month, day of month, 
 	return the rata die integer day number.  */
 	
 	double s;
-	int	day_offset, yearm1, rd;
+	GMT_LONG	day_offset, yearm1, rd;
 	
 	
 	if (gm <= 2) {
@@ -199,36 +188,27 @@ GMT_cal_rd GMT_rd_from_gymd (int gy, int gm, int gd) {
 	}
 	
 	yearm1 = gy - 1;
-#ifndef OLDCAL
-	rd = day_offset + gd;
-#else
-	rd = GMT_GCAL_EPOCH - 1 + day_offset + gd;
-#endif
-	rd += 365 * yearm1;
+	rd = day_offset + gd + 365 * yearm1;
 	s = floor(yearm1/4.0) - floor(yearm1/100.0) + floor(yearm1/400.0);
 	s += floor( (367 * gm - 362)/12.0);
-	rd += (int)s;
+	rd += (GMT_LONG)irint(s);
 	return ((GMT_cal_rd)rd);
 }
 
-int	GMT_gyear_from_rd (GMT_cal_rd date) {
+GMT_LONG	GMT_gyear_from_rd (GMT_cal_rd date) {
 /*	Given rata die integer day number, return proleptic Gregorian year  */
 
-	int d0, d1, d2, d3, d4, n400, n100, n4, n1, year;
+	GMT_LONG d0, d1, d2, d3, n400, n100, n4, n1, year;
 	
-#ifndef OLDCAL
-	d0 = date - 1;
-#else
-	d0 = date - GMT_GCAL_EPOCH;
-#endif
-	n400 = (int) floor (d0 / 146097.0);
+	d0 = (GMT_LONG)date - 1;
+	n400 = (GMT_LONG) floor (d0 / 146097.0);
 	d1 = GMT_cal_imod (d0, 146097);
-	n100 = (int) floor (d1 / 36524.0);
+	n100 = (GMT_LONG) floor (d1 / 36524.0);
 	d2 = GMT_cal_imod (d1, 36524);
-	n4 = (int)floor (d2 / 1461.0);
+	n4 = (GMT_LONG)floor (d2 / 1461.0);
 	d3 = GMT_cal_imod (d2, 1461);
-	n1 = (int)floor (d3 / 365.0);
-	d4 = GMT_cal_imod (d3, 365) + 1;
+	n1 = (GMT_LONG)floor (d3 / 365.0);
+	/* d4 = GMT_cal_imod (d3, 365) + 1; NOT USED (removed) */
 	year = 400*n400 + 100*n100 + 4*n4 + n1;
 	
 	if (n100 != 4 && n1 != 4) year++;
@@ -240,7 +220,7 @@ int	GMT_gyear_from_rd (GMT_cal_rd date) {
 
 /* ISO calendar routine  */
 
-GMT_cal_rd GMT_rd_from_iywd (int iy, int iw, int id) {
+GMT_cal_rd GMT_rd_from_iywd (GMT_LONG iy, GMT_LONG iw, GMT_LONG id) {
 /*	Given ISO calendar year, week, day of week, 
 	return the rata die integer day number.  */
 	
@@ -259,11 +239,11 @@ void GMT_gcal_from_rd (GMT_cal_rd date, struct GMT_gcal *gcal) {
 	with proleptic Gregorian and ISO calendar values.  */
 	
 	GMT_cal_rd	prior_days, corexn, tempdate;
-	int	tempyear;
+	GMT_LONG	tempyear;
 	
 	/* Day of the week in 0 thru 6:  */
 	
-	gcal->day_w = GMT_cal_imod (date, 7);
+	gcal->day_w = GMT_cal_imod ((GMT_LONG)date, 7);
 	
 	/* proleptic Gregorian operations:  */
 
@@ -271,7 +251,7 @@ void GMT_gcal_from_rd (GMT_cal_rd date, struct GMT_gcal *gcal) {
 	
 	prior_days = date - GMT_rd_from_gymd (gcal->year, 1, 1);
 
-	gcal->day_y = prior_days + 1;
+	gcal->day_y = (GMT_LONG)prior_days + 1;
 	
 	tempdate = GMT_rd_from_gymd (gcal->year, 3, 1);
 	
@@ -287,55 +267,22 @@ void GMT_gcal_from_rd (GMT_cal_rd date, struct GMT_gcal *gcal) {
 		}
 	}
 	
-	gcal->month = (int) floor( (12*(prior_days + corexn) + 373)/367.0);
+	gcal->month = (GMT_LONG) floor( (12*(prior_days + corexn) + 373)/367.0);
 	
 	tempdate = GMT_rd_from_gymd (gcal->year, gcal->month, 1);
 	
-	gcal->day_m = date - tempdate + 1;
+	gcal->day_m = (GMT_LONG)(date - tempdate) + 1;
 	
 	/* ISO operations:  */
 	
 	tempyear = (prior_days >= 3) ? gcal->year : gcal->year - 1;
 	tempdate = GMT_rd_from_iywd (tempyear+1, 1, 1);
 	gcal->iso_y = (date >= tempdate) ? tempyear + 1 : tempyear;
-	gcal->iso_w = 1 + (int)floor((date - GMT_rd_from_iywd (gcal->iso_y, 1, 1))/7.0);
+	gcal->iso_w = 1 + (GMT_LONG)floor((date - GMT_rd_from_iywd (gcal->iso_y, 1, 1))/7.0);
 	gcal->iso_d = (gcal->day_w) ? gcal->day_w : 7;
 }
 
-double	GMT_usert_from_dt (GMT_dtime t) {
-#ifndef OLDCAL
-	return (t);
-#else
-	double x;
-
-	x = t - GMT_time_system[gmtdefs.time_system].epoch_t0;
-	if (GMT_time_system[gmtdefs.time_system].i_scale != 1.0) x *= GMT_time_system[gmtdefs.time_system].i_scale;
-	return (x);
-#endif
-}
-
-GMT_dtime	GMT_dt_from_usert (double x) {
-
-	/* If we are going to allow the user to have irregular times
-		(months, years) then we have to do something complicated.
-		
-		For now, just scale and offset ?
-	*/
-	
-#ifndef OLDCAL
-	return (x);
-#else
-	if (GMT_time_system[gmtdefs.time_system].scale == 1.0) {
-		return ( (GMT_dtime) (x + GMT_time_system[gmtdefs.time_system].epoch_t0) );
-	}
-	else {
-		return ( (GMT_dtime) (x*GMT_time_system[gmtdefs.time_system].scale 
-			+ GMT_time_system[gmtdefs.time_system].epoch_t0) );
-	}
-#endif
-}
-
-int	GMT_y2_to_y4_yearfix (int y2) {
+GMT_LONG	GMT_y2_to_y4_yearfix (GMT_LONG y2) {
 
 	/* Convert 2-digit year to 4-digit year, using 
 		gmtdefs.Y2K_offset_year.
@@ -367,7 +314,7 @@ int	GMT_y2_to_y4_yearfix (int y2) {
 	return (y2 + ((y2 >= GMT_Y2K_fix.y2_cutoff) ? GMT_Y2K_fix.y100 : GMT_Y2K_fix.y200));
 }
 
-BOOLEAN	GMT_g_ymd_is_bad (int y, int m, int d) {
+GMT_LONG	GMT_g_ymd_is_bad (GMT_LONG y, GMT_LONG m, GMT_LONG d) {
 
 	/* Check year, month, day values to see if they
 		are an appropriate date in the proleptic
@@ -376,19 +323,20 @@ BOOLEAN	GMT_g_ymd_is_bad (int y, int m, int d) {
 		values.  Returns FALSE if this looks like
 		a valid calendar date.  */
 	
-	int	k;
+	GMT_LONG	k;
+	
 	
 	if (m < 1 || m > 12 || d < 1) return (TRUE);
 	
 	k = GMT_gmonth_length (y, m);	/* Number of day in the specified month */
-	
+
 	if (d > k) return (TRUE);	/* More days than we've got in this month */
 	
 	return (FALSE);
 }
 
 
-BOOLEAN	GMT_iso_ywd_is_bad (int y, int w, int d) {
+GMT_LONG	GMT_iso_ywd_is_bad (GMT_LONG y, GMT_LONG w, GMT_LONG d) {
 
 	/* Check ISO_year, ISO_week_of_year, ISO_day_of_week
 		values to see if they form a probably
@@ -408,7 +356,7 @@ BOOLEAN	GMT_iso_ywd_is_bad (int y, int w, int d) {
 	return (FALSE);
 }
 
-BOOLEAN	GMT_hms_is_bad (int h, int m, double s) {
+GMT_LONG	GMT_hms_is_bad (GMT_LONG h, GMT_LONG m, double s) {
 
 	/* Check range of hours, min, and seconds.
 		Returns TRUE if it appears something is out of range.
@@ -420,7 +368,7 @@ BOOLEAN	GMT_hms_is_bad (int h, int m, double s) {
 	return (FALSE);
 }
 
-void	GMT_gcal_from_dt (GMT_dtime t, struct GMT_gcal *cal) {
+void	GMT_gcal_from_dt (double t, struct GMT_gcal *cal) {
 
 	/* Given time in internal units, load cal and clock info
 		in cal.
@@ -430,7 +378,7 @@ void	GMT_gcal_from_dt (GMT_dtime t, struct GMT_gcal *cal) {
 	
 	GMT_cal_rd rd;
 	double	x;
-	int i;
+	GMT_LONG i;
 
 	GMT_dt2rdc (t, &rd, &x);
 	GMT_gcal_from_rd (rd, cal);
@@ -443,11 +391,11 @@ void	GMT_gcal_from_dt (GMT_dtime t, struct GMT_gcal *cal) {
 	
 	
 
-int	GMT_verify_time_step (int step, char unit) {
+GMT_LONG	GMT_verify_time_step (GMT_LONG step, char unit) {
 
 	/* Return -1 if time step and/or unit look bad, 0 if OK.  */
 	
-	int	retval = 0;
+	GMT_LONG	retval = 0;
 	
 	if (step < 0) {
 		fprintf (stderr, "GMT SYNTAX ERROR:  time steps must be positive.\n");
@@ -476,7 +424,7 @@ int	GMT_verify_time_step (int step, char unit) {
 				retval = -1;
 			}
 			break;
-		case 'R':	/* Special Gregorian days: Anotate from start of each week and not first day of month */
+		case 'R':	/* Special Gregorian days: Annotate from start of each week and not first day of month */
 			/* We are leveraging the machinery for 'K' and 'k' to step along but reset to start of week */
 		case 'd':
 		case 'D':
@@ -538,7 +486,7 @@ int	GMT_verify_time_step (int step, char unit) {
 }
 
 
-void	GMT_moment_interval (struct GMT_MOMENT_INTERVAL *p, double dt_in, BOOLEAN init) {
+void	GMT_moment_interval (struct GMT_MOMENT_INTERVAL *p, double dt_in, GMT_LONG init) {
 
 	/*   
 	
@@ -574,7 +522,7 @@ void	GMT_moment_interval (struct GMT_MOMENT_INTERVAL *p, double dt_in, BOOLEAN i
 		
 	*/
 	
-	int	k, kws, kml, kyd;
+	GMT_LONG	k, kws, kml, kyd;
 
 
 	if (init) {
@@ -714,7 +662,7 @@ void	GMT_moment_interval (struct GMT_MOMENT_INTERVAL *p, double dt_in, BOOLEAN i
 			k = (p->cc[0].day_w - kws + 7) % 7;		/* k will be in the 0- 6 range where 0 is the kws day (could be Wednesday or whatever) */
 			k += p->step;					/* Step forward the required number of days */
 			if (k > 6) {					/* Stepped into next week, must reset stride to start at week start */
-				int n_weeks;
+				GMT_LONG n_weeks;
 				n_weeks = p->step / 7;			/* Because the k % 7 would not count weeks */
 				k -= p->step;
 				k = (7 - k) % 7;			/* Number of days left in the week */
@@ -853,7 +801,7 @@ void	GMT_moment_interval (struct GMT_MOMENT_INTERVAL *p, double dt_in, BOOLEAN i
 
 
 
-void	GMT_small_moment_interval (struct GMT_MOMENT_INTERVAL *p, int step_secs, BOOLEAN init) {
+void	GMT_small_moment_interval (struct GMT_MOMENT_INTERVAL *p, GMT_LONG step_secs, GMT_LONG init) {
 
 	/* Called by GMT_moment_interval ().  Get here when p->stuff[0] is initialized and
 		0 < step_secs <= GMT_DAY2SEC_I.  If init, stuff[0] may need to be truncated.  */
@@ -863,7 +811,7 @@ void	GMT_small_moment_interval (struct GMT_MOMENT_INTERVAL *p, int step_secs, BO
 	if (step_secs == GMT_DAY2SEC_I) {
 		/* Special case of a 1-day step.  */
 		if (p->sd[0] != 0.0) {	/* Floor it to start of day.  */
-			p->dt[0] -= (p->sd[0] * GMT_time_system[gmtdefs.time_system].i_scale);
+			p->dt[0] -= (p->sd[0] * gmtdefs.time_system.i_scale);
 			p->sd[0] = 0.0;
 		}
 		/* Now we step to next day in rd first, and set dt from there.
@@ -877,7 +825,7 @@ void	GMT_small_moment_interval (struct GMT_MOMENT_INTERVAL *p, int step_secs, BO
 		if (init) {
 			x = step_secs * floor (p->sd[0] / step_secs);
 			if (x != p->sd[0]) {
-				p->dt[0] -= ((p->sd[0] - x) * GMT_time_system[gmtdefs.time_system].i_scale);
+				p->dt[0] -= ((p->sd[0] - x) * gmtdefs.time_system.i_scale);
 				x = p->sd[0];
 			}
 		}
@@ -896,7 +844,7 @@ void	GMT_small_moment_interval (struct GMT_MOMENT_INTERVAL *p, int step_secs, BO
 		}
 		else {
 			p->sd[1] = x;
-			p->dt[1] = p->dt[0] + step_secs * GMT_time_system[gmtdefs.time_system].i_scale;
+			p->dt[1] = p->dt[0] + step_secs * gmtdefs.time_system.i_scale;
 			/* No call here to reset cc[1] struct, as rd hasn't changed.
 				Later, if it is desired to reset struct for clock
 				changes on same day, add a call here.  */
@@ -906,13 +854,13 @@ void	GMT_small_moment_interval (struct GMT_MOMENT_INTERVAL *p, int step_secs, BO
 
 
 		
-int	GMT_gmonth_length (int year,  int month) {
+GMT_LONG	GMT_gmonth_length (GMT_LONG year,  GMT_LONG month) {
 
 	/* Return the number of days in a month,
 		using the gregorian leap year rule.
 	Months are numbered from 1 to 12.  */
 	
-	int	k;
+	GMT_LONG	k;
 	
 	if (month < 1 || month > 12) return 0;
 	
@@ -930,17 +878,17 @@ int	GMT_gmonth_length (int year,  int month) {
 	return (k);
 }
 
-void GMT_format_calendar (char *date, char *clock, struct GMT_DATE_IO *D, struct GMT_CLOCK_IO *C, BOOLEAN upper, int kind, GMT_dtime dt)
+void GMT_format_calendar (char *date, char *clock, struct GMT_DATE_IO *D, struct GMT_CLOCK_IO *C, GMT_LONG upper, GMT_LONG kind, double dt)
 {	/* Given the internal time representation dt and the formatting information
 	 * in the D and C structure, write the calendar representation to strings date and clock,
 	 * but skip either string if it is a NULL pointer */
 	 
 	struct GMT_gcal calendar;
-	int i_sec, m_sec, ap, ival[3];
+	GMT_LONG i_sec, m_sec, ap, ival[3];
 	char text[GMT_CALSTRING_LENGTH];
-	GMT_dtime step;
+	double step;
 
-	step = 0.5 / C->f_sec_to_int / GMT_time_system[gmtdefs.time_system].scale;	/* Precision desired in time units */
+	step = 0.5 / C->f_sec_to_int / gmtdefs.time_system.scale;	/* Precision desired in time units */
 
 	GMT_gcal_from_dt (dt + step, &calendar);			/* Convert dt to a complete calendar structure */
 	
@@ -948,20 +896,20 @@ void GMT_format_calendar (char *date, char *clock, struct GMT_DATE_IO *D, struct
 		/* Now undo Y2K fix to make a 2-digit year here if necessary */
 	
 		if (D->day_of_year) {		/* Using the year and day-of-year as date entries */
-			if (D->item_pos[0] != -1) ival[D->item_pos[0]] = (D->Y2K_year) ? abs(calendar.year) % 100 : calendar.year;
+			if (D->item_pos[0] != -1) ival[D->item_pos[0]] = (D->Y2K_year) ? GMT_abs (calendar.year) % 100 : calendar.year;
 			if (D->item_pos[3] != -1) ival[D->item_pos[3]] = calendar.day_y;
 		}
 		else if (D->iso_calendar) {	/* Using ISO year, week and day-of-week entries. Order is fixed to be y-m-d */
-			ival[0] = (D->Y2K_year) ? abs(calendar.iso_y) % 100 : calendar.iso_y;
+			ival[0] = (D->Y2K_year) ? GMT_abs (calendar.iso_y) % 100 : calendar.iso_y;
 			ival[1] = calendar.iso_w;
 			ival[2] = calendar.iso_d;
 		}
 		else {				/* Gregorian calendar entries */
-			if (D->item_pos[0] != -1) ival[D->item_pos[0]] = (D->Y2K_year) ? abs(calendar.year) % 100 : calendar.year;
+			if (D->item_pos[0] != -1) ival[D->item_pos[0]] = (D->Y2K_year) ? GMT_abs (calendar.year) % 100 : calendar.year;
 			if (D->item_pos[1] != -1) ival[D->item_pos[1]] = calendar.month;
 			if (D->item_pos[2] != -1) ival[D->item_pos[2]] = calendar.day_m;
 		}
-		memset ((void *)date, 0, GMT_CALSTRING_LENGTH);			/* To set all to zero */
+		memset ((void *)date, 0, (size_t)GMT_CALSTRING_LENGTH);			/* To set all to zero */
 		if (D->mw_text)	{						/* Must write month or week name */
 			if (D->iso_calendar)
 				strcpy (text, GMT_time_language.week_name[kind]);
@@ -981,9 +929,9 @@ void GMT_format_calendar (char *date, char *clock, struct GMT_DATE_IO *D, struct
 
 	if (!clock) return;	/* Do not want a formatted clock string - return here */
 	
-	memset ((void *)clock, 0, GMT_CALSTRING_LENGTH);			/* To set all to zero */
-	i_sec = (int) floor (calendar.sec);
-	m_sec = (int) floor (C->f_sec_to_int * (calendar.sec - i_sec));
+	memset ((void *)clock, 0,  (size_t)GMT_CALSTRING_LENGTH);			/* To set all to zero */
+	i_sec = (GMT_LONG) floor (calendar.sec);
+	m_sec = (GMT_LONG) floor (C->f_sec_to_int * (calendar.sec - i_sec));
 	
 	if (C->twelve_hr_clock) {		/* Must deal with am/pm formatting */
 		if (calendar.hour < 12)
@@ -1012,7 +960,7 @@ void GMT_format_calendar (char *date, char *clock, struct GMT_DATE_IO *D, struct
 	return;
 }
 
-void GMT_get_time_label (char *string, struct GMT_PLOT_CALCLOCK *P, struct GMT_PLOT_AXIS_ITEM *T, GMT_dtime t)
+void GMT_get_time_label (char *string, struct GMT_PLOT_CALCLOCK *P, struct GMT_PLOT_AXIS_ITEM *T, double t)
 {	/* Assemble the annotation label given the formatting options presented */
 	struct GMT_gcal calendar;
 
@@ -1020,30 +968,30 @@ void GMT_get_time_label (char *string, struct GMT_PLOT_CALCLOCK *P, struct GMT_P
 
 	switch (T->unit) {
 		case 'Y':	/* 4-digit integer year */
-			(P->date.compact) ? sprintf (string, "%d", calendar.year) : sprintf (string, "%4.4d", calendar.year);
+			(P->date.compact) ? sprintf (string, "%ld", calendar.year) : sprintf (string, "%4.4ld", calendar.year);
 			break;
 		case 'y':	/* 2-digit integer year */
-			/* (P->date.compact) ? sprintf (string, "%d", calendar.year % 100) : sprintf (string, "%2.2d", calendar.year % 100); */
-			sprintf (string, "%2.2d", calendar.year % 100);
+			/* (P->date.compact) ? sprintf (string, "%ld", calendar.year % 100) : sprintf (string, "%2.2ld", calendar.year % 100); */
+			sprintf (string, "%2.2ld", calendar.year % 100);
 			break;
 		case 'O':	/* Plot via date format */
 			GMT_format_calendar (string, CNULL, &P->date, &P->clock, T->upper_case, T->flavor, t);
 			break;
 		case 'o':	/* 2-digit month */
-			(P->date.compact) ? sprintf (string, "%d", calendar.month) : sprintf (string, "%2.2d", calendar.month);
+			(P->date.compact) ? sprintf (string, "%ld", calendar.month) : sprintf (string, "%2.2ld", calendar.month);
 			break;
 		case 'U':	/* ISO year, week, day via date format */
 			GMT_format_calendar (string, CNULL, &P->date, &P->clock, T->upper_case, T->flavor, t);
 			break;
 		case 'u':	/* 2-digit ISO week */		
-			(P->date.compact) ? sprintf (string, "%d", calendar.iso_w) : sprintf (string, "%2.2d", calendar.iso_w);
+			(P->date.compact) ? sprintf (string, "%ld", calendar.iso_w) : sprintf (string, "%2.2ld", calendar.iso_w);
 			break;
 		case 'K':	/*  ISO Weekday name */
 			if (T->upper_case) GMT_str_toupper (GMT_time_language.day_name[T->flavor][calendar.iso_d%7]);
 			sprintf (string, "%s", GMT_time_language.day_name[T->flavor][calendar.iso_d%7]);
 			break;
 		case 'k':	/* Day of the week 1-7 */
-			sprintf (string, "%d", (calendar.day_w - gmtdefs.time_week_start + 7) % 7 + 1);
+			sprintf (string, "%ld", (calendar.day_w - gmtdefs.time_week_start + 7) % 7 + 1);
 			break;
 		case 'D':	/* Day, via date format */
 			GMT_format_calendar (string, CNULL, &P->date, &P->clock, T->upper_case, T->flavor, t);
@@ -1051,31 +999,31 @@ void GMT_get_time_label (char *string, struct GMT_PLOT_CALCLOCK *P, struct GMT_P
 		case 'd':	/* 2-digit day or 3-digit day of year */
 		case 'R':	/* Gregorian month-days are the same thing - only they start at beginning of weeks and not months */
 			if (P->date.day_of_year)
-				(P->date.compact) ? sprintf (string, "%d", calendar.day_y) : sprintf (string, "%3.3d", calendar.day_y);
+				(P->date.compact) ? sprintf (string, "%ld", calendar.day_y) : sprintf (string, "%3.3ld", calendar.day_y);
 			else
-				(P->date.compact) ? sprintf (string, "%d", calendar.day_m) : sprintf (string, "%2.2d", calendar.day_m);
+				(P->date.compact) ? sprintf (string, "%ld", calendar.day_m) : sprintf (string, "%2.2ld", calendar.day_m);
 			break;
 		case 'H':	/* Hours via clock format */
 			GMT_format_calendar (CNULL, string, &P->date, &P->clock, T->upper_case, T->flavor, t);
 			break;
 		case 'h':	/* 2-digit hour */
-			(P->date.compact) ? sprintf (string, "%d", calendar.hour) : sprintf (string, "%2.2d", calendar.hour);
+			(P->date.compact) ? sprintf (string, "%ld", calendar.hour) : sprintf (string, "%2.2ld", calendar.hour);
 			break;
 		case 'M':	/* Minutes via clock format */
 			GMT_format_calendar (CNULL, string, &P->date, &P->clock, T->upper_case, T->flavor, t);
 			break;
 		case 'm':	/* 2-digit minutes */
-			(P->date.compact) ? sprintf (string, "%d", calendar.min) : sprintf (string, "%2.2d", calendar.min);
+			(P->date.compact) ? sprintf (string, "%ld", calendar.min) : sprintf (string, "%2.2ld", calendar.min);
 			break;
 		case 'C':	/* Seconds via clock format */
 			GMT_format_calendar (CNULL, string, &P->date, &P->clock, T->upper_case, T->flavor, t);
 			break;
 		case 'c':	/* 2-digit seconds */
-			(P->date.compact) ? sprintf (string, "%d", (int)calendar.sec) : sprintf (string, "%2.2d", (int)calendar.sec);
+			(P->date.compact) ? sprintf (string, "%ld", (GMT_LONG)irint(calendar.sec)) : sprintf (string, "%2.2ld", (GMT_LONG)irint(calendar.sec));
 			break;
 		default:
-			fprintf (stderr, "ERROR: wrong unit passed to GMT_get_time_label\n");
-			exit (EXIT_FAILURE);
+			fprintf (stderr, "%s: INTERNAL ERROR: wrong unit passed to GMT_get_time_label\n", GMT_program);
+			sprintf (string, "NaN");
 			break;
 	}
 }
@@ -1084,7 +1032,7 @@ void GMT_get_time_label (char *string, struct GMT_PLOT_CALCLOCK *P, struct GMT_P
 #ifdef USE_UNUSED_GMT_FUNCTIONS
 /* String reading functions for parsing ISO 8601 cal and clock  */
 
-int	GMT_read_clock (char *s, double *t) {
+GMT_LONG	GMT_read_clock (char *s, double *t) {
 /* 	Given a clock string s, try to read the form
 	hh[:mm[:ss[.xxx]]] with 0 <= hh <= 24 (yes, 24; 
 	ISO 8601 allows this), 0 <= mm <= 59, and 0 <= 
@@ -1099,8 +1047,8 @@ int	GMT_read_clock (char *s, double *t) {
 */
 
 	double	dsec;
-	int	j, k;
-	char	*cm, *cs;
+	GMT_LONG	j, k;
+	char	*cm = NULL, *cs = NULL;
 	
 	/* Get hours:  */
 	cm = strpbrk (s, ":");
@@ -1110,7 +1058,7 @@ int	GMT_read_clock (char *s, double *t) {
 		s[k-j] = '\0';
 		cm = &cm[1];	/* Move beyond colon  */
 	}
-	if ( (sscanf(s, "%d", &k)) != 1) return (-1);
+	if ( (sscanf(s, "%" GMT_LL "d", &k)) != 1) return (-1);
 	if (k < 0 || k > 24) return (-1);
 	*t = GMT_HR2SEC_I * k;		/* t now has hours (in secs) */
 	
@@ -1123,7 +1071,7 @@ int	GMT_read_clock (char *s, double *t) {
 		cm[k-j] = '\0';
 		cs = &cs[1];
 	}
-	if ( (sscanf(cm, "%d", &k)) != 1) return (-1);
+	if ( (sscanf(cm, "%" GMT_LL "d", &k)) != 1) return (-1);
 	if (k < 0 || k > 59) return (-1);
 	(*t) += GMT_MIN2SEC_I * k;	/* Now add in minutes (in secs) */
 	
@@ -1134,7 +1082,7 @@ int	GMT_read_clock (char *s, double *t) {
 	return (0);
 }
 
-int	GMT_read_cal (char *s, GMT_cal_rd *rd) {
+GMT_LONG	GMT_read_cal (char *s, GMT_cal_rd *rd) {
 /*	Given a string thought to be a calendar string,
 	find the rata die date of the string.  The string
 	could be of the form 
@@ -1147,8 +1095,8 @@ int	GMT_read_cal (char *s, GMT_cal_rd *rd) {
 	present then <year>, <j>, <k> are (proleptic
 	gregorian) year, month, day of month.  */
 
-	int	i, j, k, itemp1, itemp2=1, itemp3=1, is_iso=0;
-	char	*cj, *ck = CNULL;
+	GMT_LONG	i, j, k, itemp1, itemp2=1, itemp3=1, is_iso=0;
+	char	*cj = NULL, *ck = CNULL;
 	
 	/* A period or comma would be wrong for this format:  */
 	if ( (strpbrk (s, ".") ) || (strpbrk (s, ",") ) ) return (-1);
@@ -1168,7 +1116,7 @@ int	GMT_read_cal (char *s, GMT_cal_rd *rd) {
 		s[k-(i+j)] = '\0';
 	}
 	
-	if ( (sscanf (s, "%d", &itemp1) ) != 1) return (-1);
+	if ( (sscanf (s, "%" GMT_LL "d", &itemp1) ) != 1) return (-1);
 	
 	/* A null value of cj is not an error.  We might have
 		had "1985T" passed to a function and it might
@@ -1191,7 +1139,7 @@ int	GMT_read_cal (char *s, GMT_cal_rd *rd) {
 			k = strlen (cj);
 			cj[k-j] = '\0';
 		}
-		if ( (sscanf (cj, "%d", &itemp2) ) != 1) return (-1);
+		if ( (sscanf (cj, "%" GMT_LL "d", &itemp2) ) != 1) return (-1);
 		if (itemp2 < 1) return (-1);
 		if (is_iso) {
 			if (itemp2 > 53) return (-1);
@@ -1201,7 +1149,7 @@ int	GMT_read_cal (char *s, GMT_cal_rd *rd) {
 		}
 		if (ck) {
 			/* Read it, skipping the leading hyphen.  */
-			if ( (sscanf (&ck[1], "%d", &itemp3) ) != 1) return (-1);
+			if ( (sscanf (&ck[1], "%" GMT_LL "d", &itemp3) ) != 1) return (-1);
 			if (itemp3 < 1) return (-1);
 			if (is_iso) {
 				if (itemp3 > 7) return (-1);
@@ -1215,12 +1163,10 @@ int	GMT_read_cal (char *s, GMT_cal_rd *rd) {
 	/* Get here when ready to make a rata die out of 3 itemp values,
 		based on value of is_iso:  */
 	
-	if (is_iso) {
+	if (is_iso)
 		*rd = GMT_rd_from_iywd (itemp1, itemp2, itemp3);
-	}
-	else {
+	else
 		*rd = GMT_rd_from_gymd (itemp1, itemp2, itemp3);
-	}
 	return (0);
 }
 
@@ -1232,14 +1178,14 @@ double	GMT_cal_dmod (double x, double y) {
 	return (x - y * floor(x/y) );
 }
 
-GMT_cal_rd GMT_kday_on_or_after (GMT_cal_rd date, int kday) {
+GMT_cal_rd GMT_kday_on_or_after (GMT_cal_rd date, GMT_LONG kday) {
 /*	Given date and kday, return the date of the nearest kday
 	on or after the given date.
 */
 	return (GMT_kday_on_or_before(date+6, kday));
 }
 
-GMT_cal_rd GMT_kday_nearest (GMT_cal_rd date, int kday) {
+GMT_cal_rd GMT_kday_nearest (GMT_cal_rd date, GMT_LONG kday) {
 /*	Given date and kday, return the date of the nearest kday
 	to the given date.
 */
