@@ -1,12 +1,12 @@
 /*--------------------------------------------------------------------
- *	$Id: gmtconvert.c,v 1.39 2006/10/17 03:01:08 pwessel Exp $
+ *	$Id: gmtconvert.c 9729 2012-03-21 23:19:37Z pwessel $
  *
- *	Copyright (c) 1991-2006 by P. Wessel and W. H. F. Smith
- *	See COPYING file for copying and redistribution conditions.
+ *	Copyright (c) 1991-2012 by P. Wessel and W. H. F. Smith
+ *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
- *	the Free Software Foundation; version 2 of the License.
+ *	the Free Software Foundation; version 2 or any later version.
  *
  *	This program is distributed in the hope that it will be useful,
  *	but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -29,57 +29,59 @@
 #define GMT_WITH_NO_PS
 #include "gmt.h"
 
-#define GMTCONVERT_MAX_COL BUFSIZ
-
 struct GMTCONVERT_CTRL {
 	struct A {	/* -A */
-		BOOLEAN active;
+		GMT_LONG active;
 	} A;
 	struct D {	/* -D[<template>] */
-		BOOLEAN active;
-		char *template;
+		GMT_LONG active;
+		char *name;
 	} D;
 	struct E {	/* -E */
-		BOOLEAN active;
+		GMT_LONG active;
+		GMT_LONG mode;
 	} E;
 	struct F {	/* -F<cols> */
-		BOOLEAN active;
-		short int *col;
+		GMT_LONG active;
+		GMT_LONG *col;
 	} F;
 	struct L {	/* -L */
-		BOOLEAN active;
+		GMT_LONG active;
 	} L;
 	struct I {	/* -I */
-		BOOLEAN active;
+		GMT_LONG active;
 	} I;
 	struct N {	/* -N */
-		BOOLEAN active;
+		GMT_LONG active;
 	} N;
-	struct S {	/* -S\"search string\" */
-		BOOLEAN active;
+	struct S {	/* -S[~]\"search string\" */
+		GMT_LONG active;
+		GMT_LONG inverse;
 		char *pattern;
 	} S;
 };
 
 int main (int argc, char **argv)
 {
-	int i, j, k, pos, start, stop, n_total_read = 0, n_files = 0, n_tot_cols = 0, n_hdr_delay = 0, fno;
-	int n_out = 0, n_rev_alloc = 0, n_rev = 0, error = 0, n_seg = 0, n_in_seg = 0, n_nan, out_seg;
-	BOOLEAN on, ascii_in_bin_out;
+	GMT_LONG pos, start, stop, fno, n_files = 0, n_out = 0, n_tot_cols = 0;
+	GMT_LONG i, j, k, n_total_read = 0, n_hdr_delay = 0, n_out_seg = 0;
+	GMT_LONG n_rev_alloc = 0, n_rev = 0, n_rev_recs = 0, error = 0, n_seg = 0, n_in_seg = 0, n_nan, out_seg;
+	
+	GMT_LONG on, ascii_in_bin_out, match = FALSE;
 
-	double	*in, *out = VNULL, *R_out = (double *)NULL;
+	double	*in = NULL, *out = VNULL, *R_out = (double *)NULL;
 
 	char record_str[BUFSIZ], ptr[BUFSIZ], header[BUFSIZ], filename[BUFSIZ];
 	
-	FILE *fp_out;
+	FILE *fp_out = NULL;
 
-	struct GMTCONVERT_CTRL *Ctrl;
+	struct GMTCONVERT_CTRL *Ctrl = NULL;
 
-	void *New_Gmtconvert_Ctrl (), Free_Gmtconvert_Ctrl (struct GMTCONVERT_CTRL *C);
+	void *New_gmtconvert_Ctrl (), Free_gmtconvert_Ctrl (struct GMTCONVERT_CTRL *C);
 	
-	argc = GMT_begin (argc, argv);
+	argc = (int)GMT_begin (argc, argv);
 
-	Ctrl = (struct GMTCONVERT_CTRL *)New_Gmtconvert_Ctrl ();	/* Allocate and initialize a new control structure */
+	Ctrl = (struct GMTCONVERT_CTRL *)New_gmtconvert_Ctrl ();	/* Allocate and initialize a new control structure */
 	
 	for (i = 1; i < argc; i++) {
 		if (argv[i][0] == '-') {
@@ -89,9 +91,12 @@ int main (int argc, char **argv)
 				/* Common parameters */
 
 				case 'H':
+				case 'M':
 				case 'V':
 				case 'b':
 				case 'f':
+				case 'g':
+				case 'm':
 				case ':':
 				case '\0':
 					error += GMT_parse_common_options (argv[i], 0, 0, 0, 0);
@@ -103,18 +108,24 @@ int main (int argc, char **argv)
 					Ctrl->A.active = TRUE;
 					break;
 				case 'D':               /* Write each segment to a separate output file */
-					if (argv[i][2]) Ctrl->D.template = strdup (&argv[i][2]);
+					if (argv[i][2]) Ctrl->D.name = strdup (&argv[i][2]);
 					Ctrl->D.active = TRUE;
 					break;
 				case 'E':
 					Ctrl->E.active = TRUE;
+					if (argv[i][2] == 'f')		/* Get first point only */
+						Ctrl->E.mode = 1;
+					else if (argv[i][2] == 'l')	/* Get last point only */
+						Ctrl->E.mode = 2;
+					else				/* Get first and point only */
+						Ctrl->E.mode = 3;
 					break;
 				case 'F':
 					pos = 0;
 					Ctrl->F.active = TRUE;
-					while ((GMT_strtok (&argv[i][2], ",", &pos, ptr))) {	/* Proces all tokens */
+					while ((GMT_strtok (&argv[i][2], ",", &pos, ptr))) {	/* Process all tokens */
 						if (strchr (ptr, '-'))	/* Range of columns given. e.g., 7-9 */
-							sscanf (ptr, "%d-%d", &start, &stop);
+							sscanf (ptr, "%" GMT_LL "d-%" GMT_LL "d", &start, &stop);
 						else if (isdigit ((int)ptr[0]))	/* Just a single column, e.g., 13 */
 							start = stop = atoi (ptr);
 						else {				/* Badness! */
@@ -131,15 +142,14 @@ int main (int argc, char **argv)
 				case 'I':
 					Ctrl->I.active = TRUE;
 					break;
-				case 'M':
-					GMT_multisegment (&argv[i][2]);
-					break;
 				case 'N':
 					Ctrl->N.active = TRUE;
 					break;
 				case 'S':
-					Ctrl->S.pattern = strdup (&argv[i][2]);
 					Ctrl->S.active = TRUE;
+					k = (argv[i][2] == '\\' && strlen (argv[i]) > 3 && argv[i][3] == '~') ? 3 : 2;	/* Special escape if pattern starts with ~ */
+					if (argv[i][2] == '~') Ctrl->S.inverse = TRUE;
+					Ctrl->S.pattern = strdup (&argv[i][k+Ctrl->S.inverse]);
 					break;
 				default:
 					error = TRUE;
@@ -153,30 +163,34 @@ int main (int argc, char **argv)
 
 	if (argc == 1 || GMT_give_synopsis_and_exit) {
 		fprintf (stderr, "gmtconvert %s - Convert format, Paste, and/or Extract columns from table data\n\n", GMT_VERSION);
-		fprintf(stderr,"usage:	gmtconvert [files] [-A] [-D[<template>]] [-E] [-F<cols>] [%s] [-I]\n", GMT_H_OPT);
-		fprintf(stderr,"\t[-L] [%s] [-N] [-S\"search string\"] [-V] [%s]\n\t[%s] [%s]\n\n", GMT_M_OPT, GMT_t_OPT, GMT_b_OPT, GMT_f_OPT);
+		fprintf(stderr,"usage:	gmtconvert [files] [-A] [-D[<template>]] [-E[f|l]] [-F<cols>] [%s] [-I]\n", GMT_H_OPT);
+		fprintf(stderr,"\t[-L] [-N] [-S[~]\"search string\"] [-V] [%s]\n\t[%s] [%s] [%s] [%s]\n\n", GMT_t_OPT, GMT_b_OPT, GMT_f_OPT, GMT_g_OPT, GMT_m_OPT);
 
 		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
 
 		fprintf (stderr, "\n\tOPTIONS:\n");
-		fprintf (stderr, "\t-A Input files should be pAsted, not concatenated [Default is concatenated]\n");
-		fprintf (stderr, "\t-D writes individual segments to separate files [Default writes one multisegment file to stdout].\n");
-		fprintf (stderr, "\t   Append file name template which MUST contain a C-format specified for an integer (e.g., %%d).\n");
-		fprintf (stderr, "\t   [Default uses gmtconvert_segment_%%d.d]\n");
-		fprintf (stderr, "\t-E Extract first and last point per segment only [Default outputs all points]\n");
-		fprintf(stderr,"\t-F Give comma-separated list of desired columns or ranges (0 is first column) [Default is all]\n");
+		fprintf (stderr, "\t-A Input files should be pAsted, not concatenated [Default is concatenated].\n");
+		fprintf (stderr, "\t-D Writes individual segments to separate files [Default writes one multisegment file to stdout].\n");
+		fprintf (stderr, "\t   Append file name template which MUST contain a C-format specified for an integer (e.g., %%d)\n");
+		fprintf (stderr, "\t   [Default uses gmtconvert_segment_%%d.d].\n");
+		fprintf (stderr, "\t-E Extract first and last point per segment only [Default outputs all points].\n");
+		fprintf (stderr, "\t   Append f for first only or l for last only\n");
+		fprintf (stderr, "\t-F Give comma-separated list of desired columns or ranges (0 is first column) [Default is all].\n");
 		GMT_explain_option ('H');
-		fprintf (stderr,"\t-I Invert order of rows, i.e., outputting in reverse order\n");
-		fprintf (stderr, "\t-L Output all multisegment headers only, no data records (requires -M and ASCII data)\n");
-		GMT_explain_option ('M');
-		fprintf (stderr, "\t-N Skip output records where all fields == NaN [Default writes all records]\n");
+		fprintf (stderr, "\t-I Invert order of rows, i.e., outputting in reverse order.\n");
+		fprintf (stderr, "\t-L Output all multisegment headers only, no data records (requires -m and ASCII data).\n");
+		fprintf (stderr, "\t-N Skip output records where all fields == NaN [Default writes all records].\n");
 		fprintf (stderr, "\t-S Only output segments whose segment headers contains the pattern \"string\".\n");
-		fprintf (stderr, "\t   (Requires -M and ASCII data) [All segments]\n");
+		fprintf (stderr, "\t   Use -S~\"string\" to output segment that DO NOT contain this pattern.\n");
+		fprintf (stderr, "\t   If your pattern begins with ~, escape it with \\~.\n");
+		fprintf (stderr, "\t   (-S requires -m and ASCII data) [All segments].\n");
 		GMT_explain_option ('V');
 		GMT_explain_option ('i');
 		GMT_explain_option ('n');
 		GMT_explain_option ('o');
 		GMT_explain_option ('f');
+		GMT_explain_option ('g');
+		GMT_explain_option ('m');
 		exit (EXIT_FAILURE);
 	}
 
@@ -192,17 +206,15 @@ int main (int argc, char **argv)
 		fprintf (stderr, "%s: GMT SYNTAX ERROR.  -L or -S requires multisegment files\n", GMT_program);
 		error++;
 	}
-	if (Ctrl->D.active && Ctrl->D.template && !strstr (Ctrl->D.template, "%")) {
+	if (Ctrl->D.active && Ctrl->D.name && !strstr (Ctrl->D.name, "%")) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR:  output template must contain %%d\n", GMT_program);
 		error++;
 	}
 	if (error) exit (EXIT_FAILURE);
 
-	GMT_put_history (argc, argv);	/* Update .gmtcommands */
-
 	if (GMT_io.binary[GMT_IN] && gmtdefs.verbose) {
 		char *type[2] = {"double", "single"};
-		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
+		fprintf (stderr, "%s: Expects %ld-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
 	}
 
 #ifdef SET_IO_MODE
@@ -211,17 +223,23 @@ int main (int argc, char **argv)
 	on = (Ctrl->L.active) ? FALSE : TRUE;
 	ascii_in_bin_out = (!GMT_io.binary[GMT_IN] && GMT_io.binary[GMT_OUT]);
 	fp_out = (Ctrl->D.active) ? NULL : GMT_stdout;
-	if (Ctrl->D.active && !Ctrl->D.template) Ctrl->D.template = strdup ("gmtconvert_segment_%d.d");
+	if (Ctrl->D.active && !Ctrl->D.name) Ctrl->D.name = strdup ("gmtconvert_segment_%d.d");
 	out_seg = -1;
 
+	/* When NAN_RECORDS = pass and no multi-segment output, turn off default NaN handling */
+	if (gmtdefs.nan_is_gap && !GMT_io.multi_segments[GMT_OUT]) {
+		for (i = 0; i < 2; i++) GMT_io.skip_if_NaN[i] = FALSE;
+	}
+
 	if (Ctrl->A.active) {	/* Do paste and probably cut as well */
-		int *n_expected_fields, *n_fields, n_alloc = GMT_SMALL_CHUNK, all_set = 0;
-		double *val;
+		GMT_LONG *n_expected_fields = NULL, *n_fields = NULL, all_set = 0;
+		GMT_LONG n_alloc = GMT_SMALL_CHUNK;
+		double *val = NULL;
 		FILE **fp = NULL;
 
-		fp = (FILE **) GMT_memory (VNULL, n_files, sizeof (FILE *), GMT_program);
-		n_expected_fields = (int *) GMT_memory (VNULL, n_files, sizeof (int), GMT_program);
-		n_fields = (int *) GMT_memory (VNULL, n_files, sizeof (int), GMT_program);
+		fp = (FILE **) GMT_memory (VNULL, (size_t)n_files, sizeof (FILE *), GMT_program);
+		n_expected_fields = (GMT_LONG *) GMT_memory (VNULL, n_files, sizeof (GMT_LONG), GMT_program);
+		n_fields = (GMT_LONG *) GMT_memory (VNULL, n_files, sizeof (GMT_LONG), GMT_program);
 
 		for (fno = 1, k = 0; fno < argc; fno++) {	/* Loop over input files, if any */
 			if (argv[fno][0] == '-') continue;
@@ -258,18 +276,18 @@ int main (int argc, char **argv)
 						strcat (header, &record_str[1]);
 				}
 				n_total_read ++;
-				if (!GMT_io.binary[GMT_OUT] && GMT_io.io_header[GMT_OUT]) fprintf (GMT_stdout, "%s", header);
+				if (!GMT_io.binary[GMT_OUT] && GMT_io.io_header[GMT_OUT]) GMT_fputs (header, GMT_stdout);
 			}
 		}
 
 		for (k = j = 0; k < n_files; k++) {	/* Get first record from each file */
-			n_expected_fields[k] = (GMT_io.ncol[GMT_IN]) ? GMT_io.ncol[GMT_IN] : BUFSIZ; /* BUFSIZ -> Ascii table, must first find the number of fields */
+			n_expected_fields[k] = (GMT_io.ncol[GMT_IN]) ? GMT_io.ncol[GMT_IN] : GMT_MAX_COLUMNS; /* GMT_MAX_COLUMNS -> Ascii table, must first find the number of fields */
 			n_fields[k] = GMT_input (fp[k], &n_expected_fields[k], &in);
 			if (! GMT_io.status & GMT_IO_SEGMENT_HEADER) {	/* Good, a data record, update number of cols etc */
 				all_set = 1;
 				n_tot_cols += n_expected_fields[k];
 				if (n_tot_cols > n_alloc) {
-					n_alloc += GMT_SMALL_CHUNK;
+					n_alloc <<= 1;
 					val = (double *) GMT_memory ((char *)val, (size_t)n_alloc, sizeof (double), GMT_program);
 				}
 				for (i = 0; i < n_fields[k]; i++) val[j++] = in[i];	/* Copy the input values */
@@ -283,7 +301,7 @@ int main (int argc, char **argv)
 			}
 		}
 		n_total_read ++;
-		if (all_set == 1) {	/* Means we know how many total colums and can now allocate memory */
+		if (all_set == 1) {	/* Means we know how many total columns and can now allocate memory */
 			all_set = 2;
 			if (n_out == 0) {	/* No cut action, just paste and return all columns */
 				n_out = n_tot_cols;
@@ -291,19 +309,19 @@ int main (int argc, char **argv)
 			}
 			out = (double *) GMT_memory (VNULL, sizeof (double), (size_t)n_out, GMT_program);
 			if (n_tot_cols < n_out) {
-				fprintf (stderr, "%s: Columns requested (%d) exceed actual (%d) - aborting\n", GMT_program, n_out, n_tot_cols);
+				fprintf (stderr, "%s: Columns requested (%ld) exceed actual (%ld) - aborting\n", GMT_program, n_out, n_tot_cols);
 				exit (EXIT_FAILURE);
 			}
 		}
-		while (! (GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
+		while (!GMT_REC_IS_EOF) {	/* Not yet EOF */
 
-			while (GMT_io.status & GMT_IO_SEGMENT_HEADER && !(GMT_io.status & GMT_IO_EOF)) {	/* As long as we have read a segment header */
+			while (GMT_REC_IS_NEW_SEGMENT && !GMT_REC_IS_EOF) {	/* As long as we have read a segment header */
 				strcpy (GMT_io.segment_header, header);
 				if (ascii_in_bin_out && n_out == 0)
 					n_hdr_delay++;
 				else if (Ctrl->D.active) {
 					out_seg++;
-					sprintf (filename, Ctrl->D.template, out_seg);
+					sprintf (filename, Ctrl->D.name, (int)out_seg);
 					if (fp_out != NULL) GMT_fclose (fp_out);
 					if ((fp_out = GMT_fopen (filename, GMT_io.w_mode)) == NULL ) {
 						fprintf (stderr, "%s: Error creating file %s\n", GMT_program, filename);
@@ -314,12 +332,12 @@ int main (int argc, char **argv)
 					GMT_write_segmentheader (fp_out, n_out);
 				for (k = 0; k < n_files; k++) {	/* Get next record */
 					n_fields[k] = GMT_input (fp[k],  &n_expected_fields[k], &in);
-					if (! GMT_io.status & GMT_IO_SEGMENT_HEADER) {	/* Data record */
+					if (!GMT_REC_IS_NEW_SEGMENT) {	/* Data record */
 						if (! all_set) {	/* Very first data record */
 							all_set = 1;
 							n_tot_cols += n_expected_fields[k];
 							if (n_tot_cols > n_alloc) {
-								n_alloc += GMT_SMALL_CHUNK;
+								n_alloc <<= 1;
 								val = (double *) GMT_memory ((char *)val, (size_t)n_alloc, sizeof (double), GMT_program);
 							}
 						}
@@ -336,7 +354,7 @@ int main (int argc, char **argv)
 				n_total_read ++;
 			}
 
-			if (all_set == 1) {	/* Well, we still havent allocated space etc */
+			if (all_set == 1) {	/* Well, we still have not allocated space etc */
 				all_set = 2;
 				if (n_out == 0) {	/* No cut action, just paste */
 					n_out = n_tot_cols;
@@ -348,14 +366,14 @@ int main (int argc, char **argv)
 				}
 				out = (double *) GMT_memory (VNULL, (size_t)n_out, sizeof (double), GMT_program);
 				if (n_tot_cols < n_out) {
-					fprintf (stderr, "%s: Columns requested (%d) exceed actual (%d) - aborting\n", GMT_program, n_out, n_tot_cols);
+					fprintf (stderr, "%s: Columns requested (%ld) exceed actual (%ld) - aborting\n", GMT_program, n_out, n_tot_cols);
 					exit (EXIT_FAILURE);
 				}
 			}
-			while (! (GMT_io.status & (GMT_IO_SEGMENT_HEADER | GMT_IO_EOF))) {	/* Keep going until FALSE or = 2 segment header */
-				if (GMT_io.status & GMT_IO_MISMATCH) {
-					fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %d\n", GMT_program, n_fields[0], n_expected_fields[0], n_total_read);
-					exit (EXIT_FAILURE);
+			while (!GMT_REC_IS_NEW_SEGMENT && !GMT_REC_IS_EOF) {	/* Keep going until FALSE or = 2 segment header */
+				if (GMT_REC_IS_ERROR) {
+					fprintf (stderr, "%s: Mismatch between actual (%ld) and expected (%ld) fields near line %ld (skipped)\n", GMT_program, n_fields[0], n_expected_fields[0], n_total_read);
+					continue;
 				}
 
 				/* Now shuffle and output the chosen columns */
@@ -366,12 +384,13 @@ int main (int argc, char **argv)
 				}
 
 				if (Ctrl->I.active) {	/* Collect all rows an write in reverse order */
-					if (n_rev == n_rev_alloc) {
-						n_rev_alloc += (n_out * GMT_CHUNK);
-						R_out = (double *) GMT_memory ((void *)R_out, n_rev_alloc, sizeof (double), GMT_program);
+					if (n_rev_recs == n_rev_alloc) {
+						n_rev_alloc = (n_rev_recs == 0) ? GMT_CHUNK : (n_rev_alloc << 1);
+						R_out = (double *) GMT_memory ((void *)R_out, (size_t)(n_rev_alloc * n_out), sizeof (double), GMT_program);
 					}
 					if (!Ctrl->N.active || n_nan < n_out) {
 						for (i = 0; i < n_out; i++) R_out[n_rev++] = out[i];
+						n_rev_recs++;
 					}
 				}
 				else if (!Ctrl->N.active || n_nan < n_out)
@@ -379,7 +398,7 @@ int main (int argc, char **argv)
 
 				for (k = j = 0; k < n_files; k++) {	/* Get next record */
 					n_fields[k] = GMT_input (fp[k], &n_expected_fields[k], &in);
-					if (GMT_io.status & GMT_IO_SEGMENT_HEADER) {	/* ANother segment header */
+					if (GMT_REC_IS_SEG_HEADER) {	/* Another segment header */
 						if (k < (n_files-1)) GMT_io.segment_header[strlen(GMT_io.segment_header)-1] = '\0';
 						if (k == 0)
 							strcpy (header, GMT_io.segment_header);
@@ -394,12 +413,16 @@ int main (int argc, char **argv)
 		}
 
 		for (k = 0; k < n_files; k++) GMT_fclose(fp[k]);
+		GMT_free ((void *)fp);
+		GMT_free ((void *)n_expected_fields);
+		GMT_free ((void *)n_fields);
+		GMT_free ((void *)val);
 
-		if (gmtdefs.verbose) fprintf(stderr, "gmtconvert: %d records passed (input cols = %d; output cols = %d)\n", n_total_read, n_tot_cols, n_out);
+		if (gmtdefs.verbose) fprintf(stderr, "gmtconvert: %ld records passed (input cols = %ld; output cols = %ld)\n", n_total_read, n_tot_cols, n_out);
 	}
 	else {	/* Just cutting out columns */
-		int n_expected_fields, n_fields, n_args;
-		BOOLEAN nofile = TRUE, done = FALSE;
+		GMT_LONG n_expected_fields, n_fields, n_args;
+		GMT_LONG nofile = TRUE, done = FALSE;
 		FILE *fp = NULL;
 
 		if (GMT_io.binary[GMT_IN] && n_out == 0) {
@@ -438,24 +461,27 @@ int main (int argc, char **argv)
 				for (i = 0; i < GMT_io.n_header_recs; i++) {
 					GMT_fgets (record_str, BUFSIZ, fp);
 					n_total_read ++;
-					if (!GMT_io.binary[GMT_OUT] && GMT_io.io_header[GMT_OUT]) fputs (record_str, GMT_stdout);
+					if (!GMT_io.binary[GMT_OUT] && GMT_io.io_header[GMT_OUT]) GMT_fputs (record_str, GMT_stdout);
 				}
 			}
 
-			n_expected_fields = (GMT_io.ncol[GMT_IN]) ? GMT_io.ncol[GMT_IN] : BUFSIZ; /* BUFSIZ -> Ascii table, must first find the number of fields */
+			n_expected_fields = (GMT_io.ncol[GMT_IN]) ? GMT_io.ncol[GMT_IN] : GMT_MAX_COLUMNS; /* GMT_MAX_COLUMNS -> Ascii table, must first find the number of fields */
 			n_fields = GMT_input (fp, &n_expected_fields, &in);
-
+			if (GMT_IO_GAP_CHECKING) GMT_write_segmentheader (GMT_stdout, n_out);
 			n_seg = n_in_seg = 0;
-			while (! (GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
+			while (!GMT_REC_IS_EOF) {	/* Not yet EOF */
 
-				while (GMT_io.status & GMT_IO_SEGMENT_HEADER && !(GMT_io.status & GMT_IO_EOF)) {
-					if (Ctrl->E.active && n_in_seg > 0) fprintf (GMT_stdout, "%s", record_str);
-					if (Ctrl->S.active) on = (strstr (GMT_io.segment_header, Ctrl->S.pattern) != NULL);	/* If match then we turn output ON */
+				while (GMT_REC_IS_NEW_SEGMENT && !GMT_REC_IS_EOF) {
+					if (Ctrl->E.active && (Ctrl->E.mode & 2) && n_in_seg > 0) GMT_fputs (record_str, GMT_stdout);		/* Previous segments last point */
+					if (Ctrl->S.active) {	/* See if this header has text matching our search string */
+						match = (strstr (GMT_io.segment_header, Ctrl->S.pattern) != NULL);	/* TRUE if we matched */
+						on = (Ctrl->S.inverse == !match);	/* If match then we turn output ON */
+					}
 					if (ascii_in_bin_out && n_out == 0)
 						n_hdr_delay++;	/* Unable to write header yet since we dont know the field count... */
 					else if ((Ctrl->L.active || on) && Ctrl->D.active) {
 						out_seg++;
-						sprintf (filename, Ctrl->D.template, out_seg);
+						sprintf (filename, Ctrl->D.name, (int)out_seg);
 						if (fp_out != NULL) GMT_fclose (fp_out);
 						if ((fp_out = GMT_fopen (filename, GMT_io.w_mode)) == NULL ) {
 							fprintf (stderr, "%s: Error creating file %s\n", GMT_program, filename);
@@ -465,14 +491,19 @@ int main (int argc, char **argv)
 					else if (Ctrl->L.active || on)
 						GMT_write_segmentheader (GMT_stdout, n_out);
 					n_seg++;
+					if (on) n_out_seg++;
 					n_in_seg = 0;
 					n_fields = GMT_input (fp,  &n_expected_fields, &in);
 					n_total_read ++;
 				}
+				if (GMT_REC_IS_GAP) {
+					GMT_write_segmentheader (GMT_stdout, n_out);
+					GMT_io.status = 0;	/* Done with gap */
+				}
 
-				while (! (GMT_io.status & (GMT_IO_SEGMENT_HEADER | GMT_IO_EOF))) {	/* Keep going until FALSE or = 2 segment header */
-					if (GMT_io.status & GMT_IO_MISMATCH) {
-						fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %d\n", GMT_program, n_fields, n_expected_fields, n_total_read);
+				while (!GMT_REC_IS_LINE_BREAK && !GMT_REC_IS_EOF) {	/* Keep going until FALSE or = 2 segment header */
+					if (GMT_REC_IS_ERROR) {
+						fprintf (stderr, "%s: Mismatch between actual (%ld) and expected (%ld) fields near line %ld\n", GMT_program, n_fields, n_expected_fields, n_total_read);
 						exit (EXIT_FAILURE);
 					}
 					if (n_out == 0) {
@@ -485,7 +516,7 @@ int main (int argc, char **argv)
 						out = (double *) GMT_memory (VNULL, (size_t)n_out, sizeof (double), GMT_program);
 					}
 					if (n_expected_fields < n_out) {
-						fprintf (stderr, "%s: Columns requested (%d) exceed actual (%d) - aborting\n", GMT_program, n_expected_fields, n_out);
+						fprintf (stderr, "%s: Columns requested (%ld) exceed actual (%ld) - aborting\n", GMT_program, n_expected_fields, n_out);
 						exit (EXIT_FAILURE);
 					}
 
@@ -497,16 +528,17 @@ int main (int argc, char **argv)
 					}
 
 					if (Ctrl->I.active) {	/* Collect all rows an write in reverse order */
-						if (n_rev == n_rev_alloc) {
-							n_rev_alloc += (n_out * GMT_CHUNK);
-							R_out = (double *) GMT_memory ((void *)R_out, n_rev_alloc, sizeof (double), GMT_program);
+						if (n_rev_recs == n_rev_alloc) {
+							n_rev_alloc = (n_rev_recs == 0) ? GMT_CHUNK : (n_rev_alloc << 1);
+							R_out = (double *) GMT_memory ((void *)R_out, (size_t)(n_rev_alloc * n_out), sizeof (double), GMT_program);
 						}
 						if (!Ctrl->N.active || n_nan < n_out) {	/* OK to use this record */
 							for (i = 0; i < n_out; i++) R_out[n_rev++] = out[i];
+							n_rev_recs++;
 						}
 					}
 					else if (Ctrl->E.active) {
-						if (n_in_seg == 0) fprintf (GMT_stdout, "%s", GMT_io.current_record);
+						if (n_in_seg == 0 && (Ctrl->E.mode & 1)) GMT_fputs (GMT_io.current_record, GMT_stdout);
 						strcpy (record_str, GMT_io.current_record);
 						n_in_seg++;
 					}
@@ -518,18 +550,16 @@ int main (int argc, char **argv)
 					n_fields = GMT_input (fp, &n_expected_fields, &in);
 				}
 			}
-			if (Ctrl->E.active && n_in_seg > 1) fprintf (GMT_stdout, "%s", record_str);
+			if (Ctrl->E.active && (Ctrl->E.mode & 2) && n_in_seg > 0) GMT_fputs (record_str, GMT_stdout);
 
 			if (fp != GMT_stdin) GMT_fclose(fp);
 		}
-		if (gmtdefs.verbose) fprintf(stderr, "%s: %d records passed (input cols = %d; output cols = %d)\n", GMT_program, n_total_read, n_expected_fields, n_out);
-		if (gmtdefs.verbose && Ctrl->S.active) fprintf (stderr, "%s: Extracted %d segments\n", argv[0], n_seg);
+		if (gmtdefs.verbose) fprintf(stderr, "%s: %ld records passed (input cols = %ld; output cols = %ld)\n", GMT_program, n_total_read, n_expected_fields, n_out);
+		if (gmtdefs.verbose && Ctrl->S.active) fprintf (stderr, "%s: Extracted %ld from a total of %ld segments\n", argv[0], n_out_seg, n_seg);
 	}
 
 	if (Ctrl->I.active) {	/* Time to write out rows in reverse order */
-		int n_rows;
-		n_rows = n_rev / n_out;
-		for (j = n_rows - 1; j >= 0; j--) GMT_output (fp_out, n_out, &R_out[j*n_out]);
+		for (j = n_rev_recs - 1; j >= 0; j--) GMT_output (fp_out, n_out, &R_out[j*n_out]);
 		GMT_free ((void *)R_out);
 	}
 
@@ -537,27 +567,28 @@ int main (int argc, char **argv)
 	
 	GMT_free ((void *)out);
 
-	Free_Gmtconvert_Ctrl (Ctrl);	/* Deallocate control structure */
+	Free_gmtconvert_Ctrl (Ctrl);	/* Deallocate control structure */
 
 	GMT_end (argc, argv);
 
 	exit (EXIT_SUCCESS);
 }
 
-void *New_Gmtconvert_Ctrl () {	/* Allocate and initialize a new control structure */
+void *New_gmtconvert_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct GMTCONVERT_CTRL *C;
 	
-	C = (struct GMTCONVERT_CTRL *) GMT_memory (VNULL, 1, sizeof (struct GMTCONVERT_CTRL), "New_Gmtconvert_Ctrl");
+	C = (struct GMTCONVERT_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct GMTCONVERT_CTRL), "New_gmtconvert_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	
-	C->F.col = (short int *) GMT_memory (VNULL, GMTCONVERT_MAX_COL, sizeof (short int), GMT_program);
+	C->F.col = (GMT_LONG *) GMT_memory (VNULL, (size_t)GMT_MAX_COLUMNS, sizeof (GMT_LONG), GMT_program);
 	
 	return ((void *)C);
 }
 
-void Free_Gmtconvert_Ctrl (struct GMTCONVERT_CTRL *C) {	/* Deallocate control structure */
+void Free_gmtconvert_Ctrl (struct GMTCONVERT_CTRL *C) {	/* Deallocate control structure */
+	if (C->D.name) free ((void *)C->D.name);	
 	if (C->F.col) GMT_free ((void *)C->F.col);	
-	if (C->S.pattern) GMT_free ((void *)C->S.pattern);	
+	if (C->S.pattern) free ((void *)C->S.pattern);	
 	GMT_free ((void *)C);	
 }
