@@ -1,12 +1,12 @@
 /*--------------------------------------------------------------------
- *    $Id: grdblend.c,v 1.43 2006/10/24 21:03:28 remko Exp $
+ *    $Id: grdblend.c 9729 2012-03-21 23:19:37Z pwessel $
  *
- *	Copyright (c) 1991-2006 by P. Wessel and W. H. F. Smith
- *	See COPYING file for copying and redistribution conditions.
+ *	Copyright (c) 1991-2012 by P. Wessel and W. H. F. Smith
+ *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
- *	the Free Software Foundation; version 2 of the License.
+ *	the Free Software Foundation; version 2 or any later version.
  *
  *	This program is distributed in the hope that it will be useful,
  *	but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -16,7 +16,7 @@
  *	Contact info: gmt.soest.hawaii.edu
  *--------------------------------------------------------------------*/
 /*
- * grdblend reads any number of gridfiles that may partly overlap and
+ * grdblend reads any number of grid files that may partly overlap and
  * creates a blend of all the files given certain criteria.  Each input
  * grid is considered to have an "outer" and "inner" region.  The outer
  * region is the extent of the grid; the inner region is provided as
@@ -42,59 +42,68 @@
 
 struct GRDBLEND_CTRL {
 	struct G {	/* -G<grdfile> */
-		BOOLEAN active;
+		GMT_LONG active;
 		char *file;
 	} G;
 	struct I {	/* -Idx[/dy] */
-		BOOLEAN active;
+		GMT_LONG active;
 		double xinc, yinc;
 	} I;
 	struct N {	/* -N<nodata> */
-		BOOLEAN active;
+		GMT_LONG active;
 		double nodata;
 	} N;
 	struct Q {	/* -Q */
-		BOOLEAN active;
+		GMT_LONG active;
 	} Q;
 	struct Z {	/* -Z<scale> */
-		BOOLEAN active;
+		GMT_LONG active;
 		double scale;
 	} Z;
 	struct W {	/* -W */
-		BOOLEAN active;
+		GMT_LONG active;
 	} W;
 };
 
 struct GRDBLEND_INFO {	/* Structure with info about each input grid file */
 	struct GMT_GRDFILE G;				/* I/O structure for grid files, including grd header */
-	int in_i0, in_i1, out_i0, out_i1;		/* Indices of outer and inner x-coordinates (in output grid coordinates) */
-	int in_j0, in_j1, out_j0, out_j1;		/* Indices of outer and inner y-coordinates (in output grid coordinates) */
-	BOOLEAN outside;				/* TRUE if the current output row is outside the range of this grid */
-	BOOLEAN invert;					/* TRUE if weight was given as negative and we want to taper to zero INSIDE the grid region */
+	GMT_LONG in_i0, in_i1, out_i0, out_i1;		/* Indices of outer and inner x-coordinates (in output grid coordinates) */
+	GMT_LONG in_j0, in_j1, out_j0, out_j1;		/* Indices of outer and inner y-coordinates (in output grid coordinates) */
+	GMT_LONG offset;				/* grid offset when the grid extends beyond north */
+	long skip;					/* Byte offset to skip in native binary files */
+	GMT_LONG outside;				/* TRUE if the current output row is outside the range of this grid */
+	GMT_LONG invert;					/* TRUE if weight was given as negative and we want to taper to zero INSIDE the grid region */
+	GMT_LONG open;					/* TRUE if file is currently open */
+	char file[GMT_LONG_TEXT];			/* Name of grid file */
 	double weight, wt_y, wxr, wxl, wyu, wyd;	/* Various weighting factors used for cosine-taper weights */
 	double w_in, e_in, s_in, n_in;			/* Boundaries of inner region */
 	float *z;					/* Row vector holding the current row from this file */
 };
 
+#define N_NOT_SUPPORTED	6
+
+GMT_LONG found_unsupported_format (struct GRD_HEADER *h, char *text, char *file);
+
 int main (int argc, char **argv)
 {
-	int i, col, row, k, kk, m, n_blend, one_or_zero, error = 0;
-	size_t n_fill, n_tot;
+	GMT_LONG i, col, pcol, row, nx_360 = 0, k, kk, m, n_blend, err, error = 0;
+	GMT_LONG n_fill, n_tot, type;
 	double wt_x, w, wt;
-	float *z, no_data_f;
-	char mode[2] = {'w', 'W'}, type;
+	float *z = NULL, no_data_f;
+	char mode[2] = {'w', 'W'};
+	GMT_LONG wrap_x;
 	FILE *fp = NULL;
 	struct GRDBLEND_INFO *blend = NULL;
 	struct GMT_GRDFILE S;
-	struct GRDBLEND_CTRL *Ctrl;
+	struct GRDBLEND_CTRL *Ctrl = NULL;
 
-	void sync_input_rows (int row, struct GRDBLEND_INFO *blend, int n_blend, double half);
-	int init_blend_job (FILE *fp, struct GRD_HEADER *h, struct GRDBLEND_INFO **blend);
-	void *New_Grdblend_Ctrl (), Free_Grdblend_Ctrl (struct GRDBLEND_CTRL *C);
+	void sync_input_rows (GMT_LONG row, struct GRDBLEND_INFO *blend, GMT_LONG n_blend, double half);
+	GMT_LONG init_blend_job (FILE *fp, struct GRD_HEADER *h, struct GRDBLEND_INFO **blend);
+	void *New_grdblend_Ctrl (), Free_grdblend_Ctrl (struct GRDBLEND_CTRL *C);
 
-	argc = GMT_begin (argc, argv);
+	argc = (int)GMT_begin (argc, argv);
 
-	Ctrl = (struct GRDBLEND_CTRL *)New_Grdblend_Ctrl ();	/* Allocate and initialize a new control structure */
+	Ctrl = (struct GRDBLEND_CTRL *)New_grdblend_Ctrl ();	/* Allocate and initialize a new control structure */
 	
 	GMT_grd_init (&S.header, argc, argv, FALSE);
 
@@ -156,38 +165,43 @@ int main (int argc, char **argv)
 					break;
 			}
 		}
-		else if ((fp = fopen (argv[i], "r")) == NULL) {
+		else if ((fp = GMT_fopen (argv[i], "r")) == NULL) {
 			fprintf (stderr, "%s: Cannot open file %s\n", GMT_program, argv[i]);
 			exit (EXIT_FAILURE);
 		}
 	}
 
 	if (argc == 1 || GMT_give_synopsis_and_exit) {
-		fprintf (stderr,"grdblend %s - Blend several partially over-lapping grdfiles onto one grid\n\n", GMT_VERSION);
+		fprintf (stderr,"grdblend %s - Blend several partially over-lapping grid files onto one grid\n\n", GMT_VERSION);
 		fprintf (stderr, "usage: grdblend [<blendfile>] -G<grdfile> %s\n", GMT_I_OPT);
 		fprintf (stderr, "\t%s [-N<nodata>] [-Q] [-Z<scale>] [-V] [-W]\n\t[%s]\n", GMT_Rgeo_OPT, GMT_f_OPT);
 
 		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
 
-		fprintf (stderr, "\t<blendfile> is an ASCII file (or stdin) with blending parameters for each input grid\n");
-		fprintf (stderr, "\t   Each record has:  grdfilename -Rw/e/s/n weight\n");
-		fprintf (stderr, "\t   Relative weights are <weight> inside the given -R and cosine taper to 0 at actual -R.\n");
+		fprintf (stderr, "\t<blendfile> is an ASCII file (or stdin) with blending parameters for each input grid.\n");
+		fprintf (stderr, "\t   Each record has three items:  filename -Rw/e/s/n weight.\n");
+		fprintf (stderr, "\t   Relative weights are <weight> inside the given -R and cosine taper to 0 at actual grid -R.\n");
+		fprintf (stderr, "\t   Give filename - weight if inner region should equal the actual region.\n");
 		fprintf (stderr, "\t   Give a negative weight to invert the sense of the taper (i.e., |<weight>| outside given R.\n");
-		fprintf (stderr, "\t-G <grdfile> is the name of the final 2-D binary data set\n");
+		fprintf (stderr, "\t   Each grid must be in netCDF or native binary format.\n");
+		fprintf (stderr, "\t-G <grdfile> is the name of the output 2-D binary data set.\n");
+		fprintf (stderr, "\t   Only netCDF or native binary grid formats are supported.\n");
 		GMT_inc_syntax ('I', 0);
 		GMT_explain_option ('R');
 		fprintf (stderr, "\n\tOPTIONS:\n");
-		fprintf (stderr, "\t-N Set value for nodes without constraints [Default is NaN]\n");
-		fprintf (stderr, "\t-Q grdraster-compatible output without leading grd header [Default writes GMT grd file]\n");
+		fprintf (stderr, "\t-N Set value for nodes without constraints [Default is NaN].\n");
+		fprintf (stderr, "\t-Q grdraster-compatible output without leading grd header [Default writes GMT grid file].\n");
 		fprintf (stderr, "\t   Output grid must be in native binary format (i.e., not netCDF).\n");
-		fprintf (stderr, "\t-Z Multiply z-values by this scale before writing to file [1]\n");
+		fprintf (stderr, "\t-Z Multiply z-values by this scale before writing to file [1].\n");
 		GMT_explain_option ('V');
-		fprintf (stderr, "\t-W Write out weights only (only applies to a single input file) [make blend grid]\n");
+		fprintf (stderr, "\t-W Write out weights only (only applies to a single input file) [make blend grid].\n");
 		GMT_explain_option ('f');
 		GMT_explain_option ('.');
 		exit (EXIT_FAILURE);
 	}
 
+	GMT_check_lattice (&Ctrl->I.xinc, &Ctrl->I.yinc, NULL, &Ctrl->I.active);
+
 	if (!project_info.region_supplied) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -R:  Must specify region\n", GMT_program);
 		error++;
@@ -200,10 +214,15 @@ int main (int argc, char **argv)
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -G:  Must specify output file\n", GMT_program);
 		error++;
 	}
-	GMT_grd_get_format (Ctrl->G.file, &S.header, FALSE);
+	if ((err = GMT_grd_get_format (Ctrl->G.file, &S.header, FALSE)) != GMT_NOERROR){
+		fprintf (stderr, "%s: GMT SYNTAX ERROR: %s [%s]\n", GMT_program, GMT_strerror(err), Ctrl->G.file);
+		exit (EXIT_FAILURE);
+	}
+	/* Only allow netcdf (both v3 and new) and native binary output */
+	if (found_unsupported_format (&S.header, "GMT SYNTAX ERROR -G", Ctrl->G.file)) exit (EXIT_FAILURE);
 	type = GMT_grdformats[S.header.type][0];
 	if (Ctrl->Q.active && (type == 'c' || type == 'n')) {
-		fprintf (stderr, "%s: GMT SYNTAX ERROR -Q:  Output grdfile cannot be netCDF format\n", GMT_program);
+		fprintf (stderr, "%s: GMT SYNTAX ERROR -Q:  Only native binary grids can skip writing the header.\n", GMT_program);
 		error++;
 	}
 
@@ -214,8 +233,6 @@ int main (int argc, char **argv)
 	
 	GMT_RI_prepare (&S.header);	/* Ensure -R -I consistency and set nx, ny */
 
-	GMT_put_history (argc, argv);	/* Update .gmtcommands4 */
-
 	if (fp == NULL) {	/* No file given, use standard input */
 		fp = GMT_stdin;
 		if (gmtdefs.verbose) fprintf (stderr, "%s: Reads blend parameters from standard input\n", GMT_program);
@@ -226,38 +243,35 @@ int main (int argc, char **argv)
 	no_data_f = (float)Ctrl->N.nodata;
 	n_blend = init_blend_job (fp, &S.header, &blend);
 
-	if (fp != GMT_stdin) fclose (fp);
+	if (fp != GMT_stdin) GMT_fclose (fp);
 
 	if (Ctrl->W.active && n_blend > 1) {
-		fprintf (stderr, "%s: GMT SYNTAX ERROR -W:  Only applies when there is a single input grdfile\n", GMT_program);
+		fprintf (stderr, "%s: GMT SYNTAX ERROR -W:  Only applies when there is a single input grid file\n", GMT_program);
 		exit (EXIT_FAILURE);
 	}
 
 	/* Initialize header structure for output blend grid */
 
-	one_or_zero = !S.header.node_offset;
-
 	S.header.z_min = S.header.z_max = 0.0;
 
-	GMT_grd_RI_verify (&S.header, 1);
+	GMT_err_fail (GMT_grd_RI_verify (&S.header, 1), Ctrl->G.file);
 
-	n_tot = (size_t)S.header.nx * (size_t)S.header.ny;
+	n_tot = GMT_get_nm (S.header.nx, S.header.ny);
 
 	z = (float *) GMT_memory (VNULL, (size_t)S.header.nx, sizeof (float), GMT_program);	/* Memory for one output row */
 
-	if (!Ctrl->Q.active && GMT_write_grd_info (Ctrl->G.file, &S.header)) {	/* Write the output header structure to file */
-		fprintf (stderr, "%s: Error opening file %s\n", GMT_program, Ctrl->G.file);
-		exit (EXIT_FAILURE);
-	}
-
-	GMT_open_grd (Ctrl->G.file, &S, mode[Ctrl->Q.active]);	/* Open the gridfile for further incremental writing */
+	if (!Ctrl->Q.active) GMT_err_fail (GMT_write_grd_info (Ctrl->G.file, &S.header), Ctrl->G.file);
 
+	GMT_err_fail (GMT_open_grd (Ctrl->G.file, &S, mode[Ctrl->Q.active]), Ctrl->G.file);
+	
 	if (gmtdefs.verbose && Ctrl->Z.active) {
 		fprintf (stderr, "%s: Output data will be scaled by %g\n", GMT_program, Ctrl->Z.scale);
 	}
 
 	S.header.z_min = DBL_MAX;	/* These will be updated in the loop below */
 	S.header.z_max = -DBL_MAX;
+	wrap_x = (GMT_io.out_col_type[GMT_X] == GMT_IS_LON);	/* Periodic geographic grid */
+	if (wrap_x) nx_360 = irint (360.0 / S.header.x_inc);
 
 	for (row = 0; row < S.header.ny; row++) {	/* For every output row */
 
@@ -270,13 +284,21 @@ int main (int argc, char **argv)
 			w = 0.0;
 			for (k = m = 0; k < n_blend; k++) {	/* Loop over every input grid */
 				if (blend[k].outside) continue;					/* This grid is currently outside the s/n range */
-				if (col < blend[k].out_i0 || col > blend[k].out_i1) continue;	/* This grid is currently outside the w/e range */
-				kk = col - blend[k].out_i0;					/* kk is the local column variable for this grid */
+				if (wrap_x) {	/* Special testing for periodic x coordinates */
+					pcol = col + nx_360;
+					while (pcol > blend[k].out_i1) pcol -= nx_360;
+					if (pcol < blend[k].out_i0) continue;	/* This grid is currently outside the w/e range */
+				}
+				else {	/* Not periodic */
+					if (col < blend[k].out_i0 || col > blend[k].out_i1) continue;	/* This grid is currently outside the xmin/xmax range */
+					pcol = col;
+				}
+				kk = pcol - blend[k].out_i0;					/* kk is the local column variable for this grid */
 				if (GMT_is_fnan (blend[k].z[kk])) continue;			/* NaNs do not contribute */
-				if (col <= blend[k].in_i0)					/* Left cosine-taper weight */
-					wt_x = 0.5 * (1.0 - cos ((col - blend[k].out_i0 + S.header.xy_off) * blend[k].wxl));
-				else if (col >= blend[k].in_i1)					/* Right cosine-taper weight */
-					wt_x = 0.5 * (1.0 - cos ((blend[k].out_i1 - col + S.header.xy_off) * blend[k].wxr));
+				if (pcol <= blend[k].in_i0)					/* Left cosine-taper weight */
+					wt_x = 0.5 * (1.0 - cos ((pcol - blend[k].out_i0 + S.header.xy_off) * blend[k].wxl));
+				else if (pcol >= blend[k].in_i1)					/* Right cosine-taper weight */
+					wt_x = 0.5 * (1.0 - cos ((blend[k].out_i1 - pcol + S.header.xy_off) * blend[k].wxr));
 				else								/* Inside inner region, weight = 1 */
 					wt_x = 1.0;
 				wt = wt_x * blend[k].wt_y;					/* Actual weight is 2-D cosine taper */
@@ -300,21 +322,21 @@ int main (int argc, char **argv)
 			else			/* No grids covered this node, defaults to the no_data value */
 				z[col] = no_data_f;
 		}
-		GMT_write_grd_row (&S, 0, z);	/* Write out this weighted average row */
+		GMT_err_fail (GMT_write_grd_row (&S, 0, z), Ctrl->G.file);
 
-		if (gmtdefs.verbose && row%10 == 0)  fprintf (stderr, "%s: Processed row %7d of %d\r", GMT_program, row, S.header.ny);
+		if (gmtdefs.verbose && row%10 == 0)  fprintf (stderr, "%s: Processed row %7ld of %d\r", GMT_program, row, S.header.ny);
 
 	}
-	if (gmtdefs.verbose)  fprintf (stderr, "%s: Processed row %7d\n", GMT_program, row);
+	if (gmtdefs.verbose)  fprintf (stderr, "%s: Processed row %7ld\n", GMT_program, row);
 
 	GMT_close_grd (&S);	/* Close the output gridfile */
-	if (!Ctrl->Q.active && GMT_update_grd_info (Ctrl->G.file, &S.header)) {	/* Update header to reflect the actual extrema */
-		fprintf (stderr, "%s: Error opening file %s\n", GMT_program, Ctrl->G.file);
-		exit (EXIT_FAILURE);
-	}
+	if (!Ctrl->Q.active) GMT_err_fail (GMT_update_grd_info (Ctrl->G.file, &S.header), Ctrl->G.file);
 	GMT_free ((void *)z);
 
-	for (k = 0; k < n_blend; k++) GMT_close_grd (&blend[k].G);	/* Close all the input grd files */
+	for (k = 0; k < n_blend; k++) if (blend[k].open) {
+		GMT_free ((void *)blend[k].z);
+		GMT_close_grd (&blend[k].G);	/* Close all input grd files still open */
+	}
 
 	if (gmtdefs.verbose) {
 		char empty[GMT_TEXT_LEN];
@@ -326,45 +348,61 @@ int main (int argc, char **argv)
 				strcpy (empty, "NaN");
 			else
 				sprintf (empty, "%g", no_data_f);
-			fprintf (stderr, "%s: %d nodes assigned values, %d set to %s\n", GMT_program, (int)n_fill, (int)(n_tot - n_fill), empty);
+			fprintf (stderr, "%s: %ld nodes assigned values, %ld set to %s\n", GMT_program, (GMT_LONG)n_fill, (GMT_LONG)(n_tot - n_fill), empty);
 		}
 	}
 
 	GMT_free ((void *)blend);
 
-	Free_Grdblend_Ctrl (Ctrl);	/* Deallocate control structure */
+	Free_grdblend_Ctrl (Ctrl);	/* Deallocate control structure */
 
 	GMT_end (argc, argv);
 
 	exit (EXIT_SUCCESS);
 }
 
-int init_blend_job (FILE *fp, struct GRD_HEADER *h, struct GRDBLEND_INFO **blend) {
-	int n = 0, nr, one_or_zero = 0;
-	size_t n_alloc = GMT_SMALL_CHUNK;
-	struct GRDBLEND_INFO *B;
-	char line[BUFSIZ], r_in[GMT_LONG_TEXT], file[GMT_LONG_TEXT], type, *sense[2] = {"normal", "inverse"};
+GMT_LONG found_unsupported_format (struct GRD_HEADER *h, char *text, char *file)
+{
+	GMT_LONG i;
+	static char *not_supported[N_NOT_SUPPORTED] = {"rb", "rf", "sf", "sd", "af", "gd"};
+	for (i = 0; i < N_NOT_SUPPORTED; i++) {	/* Only allow netcdf (both v3 and new) and native binary output */
+		if (h->type == GMT_grd_format_decoder (not_supported[i])) {
+			fprintf (stderr, "grdblend: %s:  Grid format type %s for file %s is not supported in grdblend\n", text, not_supported[i], file);
+			return (1);
+		}
+	}
+	return (0);
+}
+GMT_LONG init_blend_job (FILE *fp, struct GRD_HEADER *h, struct GRDBLEND_INFO **blend) {
+	GMT_LONG n = 0, nr, one_or_zero = 0, n_alloc = 0, type;
+	struct GRDBLEND_INFO *B = NULL;
+	char line[BUFSIZ], r_in[GMT_LONG_TEXT], *sense[2] = {"normal", "inverse"};
 	void decode_R (char *string, double *w, double *e, double *s, double *n);
 
-	B = (struct GRDBLEND_INFO *) GMT_memory (VNULL, n_alloc, sizeof (struct GRDBLEND_INFO), GMT_program);
+	GMT_set_meminc (GMT_SMALL_CHUNK);
+	while (GMT_fgets (line, BUFSIZ, fp)) {	/* Read each input file for grid information */
+		GMT_chop (line);
+		if (line[0] == '#' || line[0] == '\0') continue;	/* Skip comment lines or blank lines */
 
-	while (fgets (line, BUFSIZ, fp)) {	/* Read each input file for grid information */
-		if (line[0] == '#' || line[0] == '\n' || line[0] == '\r') continue;	/* Skip comment lines or blank lines */
-
-		nr = sscanf (line, "%s %s %lf", file, r_in, &B[n].weight);
+		if (n == n_alloc) n_alloc = GMT_alloc_memory ((void **)&B, n, n_alloc, sizeof (struct GRDBLEND_INFO), GMT_program);
+		memset ((void *)&B[n], 0, sizeof (struct GRDBLEND_INFO));	/* Initialize memory */
+		nr = sscanf (line, "%s %s %lf", B[n].file, r_in, &B[n].weight);
 		if (nr != 3) {
-			fprintf (stderr, "%s: Read error for blending parameters near row %d\n", GMT_program, n);
+			fprintf (stderr, "%s: Read error for blending parameters near row %ld\n", GMT_program, n);
 			exit (EXIT_FAILURE);
 		}
-		decode_R (&r_in[2], &B[n].w_in, &B[n].e_in, &B[n].s_in, &B[n].n_in);	/* Decode inner region */
-		if (GMT_read_grd_info (file, &B[n].G.header)) {				/* Read header structure */
-			fprintf (stderr, "%s: Error opening file %s\n", GMT_program, file);
-			exit (EXIT_FAILURE);
+		GMT_err_fail (GMT_read_grd_info (B[n].file, &B[n].G.header), B[n].file);	/* Read header structure */
+		if (found_unsupported_format (&B[n].G.header, "Error for input grid", B[n].file)) exit (EXIT_FAILURE);
+		if (!strcmp (r_in, "-")) {	/* Set inner = outer region */
+			B[n].w_in = B[n].G.header.x_min;	B[n].e_in = B[n].G.header.x_max;
+			B[n].s_in = B[n].G.header.y_min;	B[n].n_in = B[n].G.header.y_max;
 		}
+		else	/* Must decode the -R string */
+			decode_R (&r_in[2], &B[n].w_in, &B[n].e_in, &B[n].s_in, &B[n].n_in);	/* Decode inner region */
 
 		/* Skip the file if its outer region does not lie within the final grid region */
 		if (h->x_min > B[n].e_in || h->x_max < B[n].w_in || h->y_min > B[n].n_in || h->y_max < B[n].s_in) {
-			fprintf (stderr, "%s: Warning: File %s entirely outside final grid region (skipped)\n", GMT_program, file);
+			fprintf (stderr, "%s: Warning: File %s entirely outside final grid region (skipped)\n", GMT_program, B[n].file);
 			continue;
 		}
 
@@ -373,17 +411,18 @@ int init_blend_job (FILE *fp, struct GRD_HEADER *h, struct GRDBLEND_INFO **blend
 		if (n == 0) {
 			h->node_offset = B[n].G.header.node_offset;
 			one_or_zero = !h->node_offset;
+			GMT_RI_prepare (h);	/* Ensure -R -I consistency and set nx, ny */
 		}
 		if (h->node_offset != B[n].G.header.node_offset){
-			fprintf (stderr, "%s: File %s has different registration that the first file given\n", GMT_program, file);
+			fprintf (stderr, "%s: File %s has different registration than the first file given\n", GMT_program, B[n].file);
 			exit (EXIT_FAILURE);
 		}
-		if (fabs (B[n].G.header.x_inc - h->x_inc) > GMT_CONV_LIMIT){
-			fprintf (stderr, "%s: File %s has different x-increment (%g) that the chosen output increment (%g)\n", GMT_program, file, B[n].G.header.x_inc, h->x_inc);
+		if (!GMT_IS_ZERO (B[n].G.header.x_inc - h->x_inc)){
+			fprintf (stderr, "%s: File %s has different x-increment (%g) than the chosen output increment (%g)\n", GMT_program, B[n].file, B[n].G.header.x_inc, h->x_inc);
 			exit (EXIT_FAILURE);
 		}
-		if (fabs (B[n].G.header.y_inc - h->y_inc) > GMT_CONV_LIMIT){
-			fprintf (stderr, "%s: File %s has different y-increment (%g) that the chosen output increment (%g)\n", GMT_program, file, B[n].G.header.y_inc, h->y_inc);
+		if (!GMT_IS_ZERO (B[n].G.header.y_inc - h->y_inc)){
+			fprintf (stderr, "%s: File %s has different y-increment (%g) than the chosen output increment (%g)\n", GMT_program, B[n].file, B[n].G.header.y_inc, h->y_inc);
 			exit (EXIT_FAILURE);
 		}
 		if (B[n].weight < 0.0) {	/* Negative weight means invert sense of taper */
@@ -391,7 +430,7 @@ int init_blend_job (FILE *fp, struct GRD_HEADER *h, struct GRDBLEND_INFO **blend
 			B[n].invert = TRUE;
 		}
 
-		GMT_open_grd (file, &B[n].G, 'r');	/* Open the grid for incremental row reading */
+		GMT_err_fail (GMT_open_grd (B[n].file, &B[n].G, 'r'), B[n].file);	/* Open the grid for incremental row reading */
 
 		/* Here, i0, j0 is the very first col, row to read, while i1, j1 is the very last col, row to read .
 		 * Weights at the outside i,j should be 0, and reach 1 at the edge of the inside block */
@@ -416,38 +455,42 @@ int init_blend_job (FILE *fp, struct GRD_HEADER *h, struct GRDBLEND_INFO **blend
 		if (B[n].out_j0 < 0) {	/* Must skip to first row inside the present -R */
 			type = GMT_grdformats[B[n].G.header.type][0];
 			if (type == 'c')	/* Old-style, 1-D netcdf grid */
-				B[n].G.start[0] += B[n].G.header.nx * abs (B[n].out_j0);
+				B[n].offset = B[n].G.header.nx * GMT_abs (B[n].out_j0);
 			else if (type == 'n')	/* New, 2-D netcdf grid */
-				B[n].G.start[0] -= abs (B[n].out_j0);
+				B[n].offset = B[n].out_j0;
 			else
-				fseek (B[n].G.fp, (long)(B[n].G.n_byte * abs (B[n].out_j0)), SEEK_CUR);
+				B[n].skip = (long)(B[n].G.n_byte * GMT_abs (B[n].out_j0));	/* do the fseek when we are ready to read first row */
 		}
 
 		/* Allocate space for one entire row */
 
 		B[n].z = (float *) GMT_memory (VNULL, (size_t)(B[n].G.header.nx), sizeof (float), GMT_program);
-		if (gmtdefs.verbose) fprintf (stderr, "%s: Blend file %s in %g/%g/%g/%g with %s weight %g [%d-%d]\n",
+		if (gmtdefs.verbose) fprintf (stderr, "%s: Blend file %s in %g/%g/%g/%g with %s weight %g [%ld-%ld]\n",
 			GMT_program, B[n].G.header.name, B[n].w_in, B[n].e_in, B[n].s_in, B[n].n_in, sense[B[n].invert], B[n].weight, B[n].out_j0, B[n].out_j1);
 
+		GMT_close_grd (&B[n].G);
+
 		n++;
-		if (n == (int)n_alloc) {
-			n_alloc += GMT_SMALL_CHUNK;
-			B = (struct GRDBLEND_INFO *) GMT_memory ((void *)B, n_alloc, sizeof (struct GRDBLEND_INFO), GMT_program);
-		}
 	}
 
-	B = (struct GRDBLEND_INFO *) GMT_memory ((void *)B, n, sizeof (struct GRDBLEND_INFO), GMT_program);
+	n_alloc = GMT_alloc_memory ((void **)&B, 0, n, sizeof (struct GRDBLEND_INFO), GMT_program);
 	*blend = B;
+	GMT_reset_meminc ();
 
 	return (n);
 }
 
-void sync_input_rows (int row, struct GRDBLEND_INFO *B, int n_blend, double half) {
-	int k;
+void sync_input_rows (GMT_LONG row, struct GRDBLEND_INFO *B, GMT_LONG n_blend, double half) {
+	GMT_LONG k;
 
 	for (k = 0; k < n_blend; k++) {	/* Get every input grid ready for the new row */
 		if (row < B[k].out_j0 || row > B[k].out_j1) {	/* Either done with grid or haven't gotten to this range yet */
 			B[k].outside = TRUE;
+			if (B[k].open) {
+				GMT_close_grd (&B[k].G);	/* Done with this file */
+				B[k].open = FALSE;
+				GMT_free ((void *)B[k].z);
+			}
 			continue;
 		}
 		B[k].outside = FALSE;
@@ -459,13 +502,20 @@ void sync_input_rows (int row, struct GRDBLEND_INFO *B, int n_blend, double half
 			B[k].wt_y = 1.0;
 		B[k].wt_y *= B[k].weight;
 
-		GMT_read_grd_row (&B[k].G, 0, B[k].z);	/* Get one row from this file */
+		if (!B[k].open) {
+			GMT_err_fail (GMT_open_grd (B[k].file, &B[k].G, 'r'), B[k].file);	/* Open the grid for incremental row reading */
+			if (B[k].skip) GMT_fseek (B[k].G.fp, B[k].skip, SEEK_CUR);	/* Position for native binary files */
+			B[k].G.start[0] += B[k].offset;					/* Start position for netCDF files */
+			B[k].open = TRUE;
+		}
+
+		GMT_err_fail (GMT_read_grd_row (&B[k].G, 0, B[k].z), B[k].file);	/* Get one row from this file */
 	}
 }
 
 void decode_R (char *string, double *w, double *e, double *s, double *n) {
 	double *p[4];
-	int i, pos, error = 0;
+	GMT_LONG i, pos, error = 0;
 	char text[BUFSIZ];
 
 	/* Needed to decode the inner region -Rw/e/s/n string */
@@ -482,10 +532,10 @@ void decode_R (char *string, double *w, double *e, double *s, double *n) {
 	}
 }
 
-void *New_Grdblend_Ctrl () {	/* Allocate and initialize a new control structure */
+void *New_grdblend_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct GRDBLEND_CTRL *C;
 	
-	C = (struct GRDBLEND_CTRL *) GMT_memory (VNULL, 1, sizeof (struct GRDBLEND_CTRL), "New_Grdblend_Ctrl");
+	C = (struct GRDBLEND_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct GRDBLEND_CTRL), "New_grdblend_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	
@@ -495,7 +545,7 @@ void *New_Grdblend_Ctrl () {	/* Allocate and initialize a new control structure
 	return ((void *)C);
 }
 
-void Free_Grdblend_Ctrl (struct GRDBLEND_CTRL *C) {	/* Deallocate control structure */
-	if (C->G.file) GMT_free ((void *)C->G.file);	
+void Free_grdblend_Ctrl (struct GRDBLEND_CTRL *C) {	/* Deallocate control structure */
+	if (C->G.file) free ((void *)C->G.file);	
 	GMT_free ((void *)C);	
 }
