@@ -1,12 +1,12 @@
 /*--------------------------------------------------------------------
- *	$Id: grdvolume.c,v 1.29 2006/08/23 17:19:26 remko Exp $
+ *	$Id: grdvolume.c 9729 2012-03-21 23:19:37Z pwessel $
  *
- *	Copyright (c) 1991-2006 by P. Wessel and W. H. F. Smith
- *	See COPYING file for copying and redistribution conditions.
+ *	Copyright (c) 1991-2012 by P. Wessel and W. H. F. Smith
+ *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
- *	the Free Software Foundation; version 2 of the License.
+ *	the Free Software Foundation; version 2 or any later version.
  *
  *	This program is distributed in the hope that it will be useful,
  *	but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -16,7 +16,7 @@
  *	Contact info: gmt.soest.hawaii.edu
  *--------------------------------------------------------------------*/
 /*
- * grdvolume reads a 2d binary gridded grdfile, and calculates the volume
+ * grdvolume reads a 2d binary gridded grid file, and calculates the volume
  * under the surface using exact integration of the bilinear interpolating
  * surface.  As an option, the user may supply a contour value; then the
  * volume is only integrated inside the chosen contour.
@@ -31,62 +31,59 @@
 
 struct GRDVOLUME_CTRL {
 	struct C {	/* -C */
-		BOOLEAN active;
+		GMT_LONG active;
 		double low, high, inc;
 	} C;
 	struct L {	/* -L<base> */
-		BOOLEAN active;
+		GMT_LONG active;
 		double value;
 	} L;
 	struct S {	/* -S */
-		BOOLEAN active;
+		GMT_LONG active;
 		char unit;
 	} S;
 	struct T {	/* -T */
-		BOOLEAN active;
+		GMT_LONG active;
 	} T;
 	struct Z {	/* Z<fact>[/<shift>] */
-		BOOLEAN active;
+		GMT_LONG active;
 		double scale, offset;
 	} Z;
 };
 
-double vol_prism_frac_x (float *z, size_t ij, size_t nx, double x0, double x1, double a, double b, double c, double d);
-double vol_prism_frac_y (float *z, size_t ij, size_t nx, double y0, double y1, double a, double b, double c, double d);
+double vol_prism_frac_x (float *z, GMT_LONG ij, GMT_LONG nx, double x0, double x1, double a, double b, double c, double d);
+double vol_prism_frac_y (float *z, GMT_LONG ij, GMT_LONG nx, double y0, double y1, double a, double b, double c, double d);
 
 int main (int argc, char **argv)
 {
-	BOOLEAN error = FALSE, full = FALSE, bad, cut[4];
+	GMT_LONG error = FALSE, full = FALSE, bad, cut[4];
 
 	char *grdfile = CNULL, format[BUFSIZ];
 
-	int i, j, n = 0, c, k, pos, neg, nc, n_contours, mode = 0, nz = 0; 
-
-	size_t ij, nm, nx, ny;
+	GMT_LONG i, j, n = 0, c, k, pos, neg, nc, n_contours, nx, ny, mode = 0, nz = 0; 
+	GMT_LONG ij, nm, ij_inc[4];
 	
-	long ij_inc[4];
-	
-	float *f;
+	float *f = NULL;
 
-	double take_out, west, east, south, north, dv, da, cval = 0.0, cellsize, fact, m_pr_deg;
-	double *area, *vol, *height, this_base, small;
+	double take_out, west, east, south, north, dv, da, cval = 0.0, cellsize, fact, dist_pr_deg;
+	double *area = NULL, *vol = NULL, *height = NULL, this_base, small;
 
-	struct GRDVOLUME_CTRL *Ctrl;
+	struct GRDVOLUME_CTRL *Ctrl = NULL;
 
 	struct GRD_HEADER grd;
 
-	void SW_triangle (float f[], size_t ij, size_t nx, BOOLEAN triangle, double *dv, double *da);
-	void NE_triangle (float f[], size_t ij, size_t nx, BOOLEAN triangle, double *dv, double *da);
-	void SE_triangle (float f[], size_t ij, size_t nx, BOOLEAN triangle, double *dv, double *da);
-	void NW_triangle (float f[], size_t ij, size_t nx, BOOLEAN triangle, double *dv, double *da);
-	void NS_trapezoid (float f[], size_t ij, size_t nx, BOOLEAN right, double *dv, double *da);
-	void EW_trapezoid (float f[], size_t ij, size_t nx, BOOLEAN top, double *dv, double *da);
-	int ors_find_kink (double y[], int n, int mode);
-	void *New_Grdvolume_Ctrl (), Free_Grdvolume_Ctrl (struct GRDVOLUME_CTRL *C);
+	void SW_triangle (float f[], GMT_LONG ij, GMT_LONG nx, GMT_LONG triangle, double *dv, double *da);
+	void NE_triangle (float f[], GMT_LONG ij, GMT_LONG nx, GMT_LONG triangle, double *dv, double *da);
+	void SE_triangle (float f[], GMT_LONG ij, GMT_LONG nx, GMT_LONG triangle, double *dv, double *da);
+	void NW_triangle (float f[], GMT_LONG ij, GMT_LONG nx, GMT_LONG triangle, double *dv, double *da);
+	void NS_trapezoid (float f[], GMT_LONG ij, GMT_LONG nx, GMT_LONG right, double *dv, double *da);
+	void EW_trapezoid (float f[], GMT_LONG ij, GMT_LONG nx, GMT_LONG top, double *dv, double *da);
+	GMT_LONG ors_find_kink (double y[], GMT_LONG n, GMT_LONG mode);
+	void *New_grdvolume_Ctrl (), Free_grdvolume_Ctrl (struct GRDVOLUME_CTRL *C);
 
-	argc = GMT_begin (argc, argv);
+	argc = (int)GMT_begin (argc, argv);
 
-	Ctrl = (struct GRDVOLUME_CTRL *) New_Grdvolume_Ctrl ();	/* Allocate and initialize a new control structure */
+	Ctrl = (struct GRDVOLUME_CTRL *) New_grdvolume_Ctrl ();	/* Allocate and initialize a new control structure */
 	
 	west = east = south = north = 0.0;
 
@@ -100,6 +97,7 @@ int main (int argc, char **argv)
 					if (argv[i][2] == 'L' || argv[i][2] == 'l') full = TRUE;
 				case 'R':
 				case ':':
+				case 'f':
 				case '\0':
 					error += GMT_parse_common_options (argv[i], &west, &east, &south, &north);
 					break;
@@ -150,28 +148,29 @@ int main (int argc, char **argv)
 
 	if (argc == 1 || GMT_give_synopsis_and_exit) {
 		fprintf (stderr,"grdvolume %s - Calculating volume under a surface within a contour\n\n", GMT_VERSION);
-		fprintf (stderr, "usage: grdvolume <grdfile> [-C<cval> or -C<low/high/delta>] [-L<base>] [-S[k]] [-T]\n\t[%s] [-V] [-Z<fact>[/<shift>]]\n", GMT_Rgeo_OPT);
+		fprintf (stderr, "usage: grdvolume <grdfile> [-C<cval> or -C<low/high/delta>] [-L<base>] [-S[k]] [-T]\n\t[%s] [-V] [-Z<fact>[/<shift>]] [%s]\n", GMT_Rgeo_OPT, GMT_f_OPT);
 
 		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
 
-		fprintf (stderr, "\t<grdfile> is the name of the 2-D binary data set\n");
+		fprintf (stderr, "\t<grdfile> is the name of the 2-D binary data set.\n");
 		fprintf (stderr, "\n\tOPTIONS:\n");
 		fprintf (stderr, "\t-C find area and volume inside the <cval> contour\n");
 		fprintf (stderr, "\t   OR search using all contours from low to high\n");
-		fprintf (stderr, "\t   [Default returns entire area and volume of grid]\n");
-		fprintf (stderr, "\t-L Add volume from <base> up to contour [Default is from contour and up only]\n");
-		fprintf (stderr, "\t-S Convert degrees to m, append k for km [Default is Cartesian]\n");
-		fprintf (stderr, "\t-T Use curvature rather than maximum to find best contour value\n");
+		fprintf (stderr, "\t   [Default returns entire area and volume of grid].\n");
+		fprintf (stderr, "\t-L Add volume from <base> up to contour [Default is from contour and up only].\n");
+		fprintf (stderr, "\t-S Convert degrees to m, append k for km [Default is Cartesian].\n");
+		fprintf (stderr, "\t-T Use curvature rather than maximum to find best contour value.\n");
 		GMT_explain_option ('R');
 		GMT_explain_option ('V');
-		fprintf (stderr, "\t   Append l for listing of all results (when contour search is selected)\n");
-		fprintf (stderr, "\t-Z Subtract <shift> and then multiply data by <fact> before processing [1/0]\n");
+		fprintf (stderr, "\t   Append l for listing of all results (when contour search is selected).\n");
+		fprintf (stderr, "\t-Z Subtract <shift> and then multiply data by <fact> before processing [1/0].\n");
+		GMT_explain_option ('f');
 		GMT_explain_option ('.');
 		exit (EXIT_FAILURE);
 	}
 
 	if (!grdfile) {
-		fprintf (stderr, "%s: GMT SYNTAX ERROR:  Must specify input grd file\n", GMT_program);
+		fprintf (stderr, "%s: GMT SYNTAX ERROR:  Must specify input grid file\n", GMT_program);
 		error++;
 	}
 	if (Ctrl->C.active && !(n == 1 || n == 3)) {
@@ -189,12 +188,8 @@ int main (int argc, char **argv)
 
 	if (error) exit (EXIT_FAILURE);
 
-	GMT_put_history (argc, argv);	/* Update .gmtcommands4 */
-
-	if (GMT_read_grd_info (grdfile, &grd)) {
-		fprintf (stderr, "%s: Error opening file %s\n", GMT_program, grdfile);
-		exit (EXIT_FAILURE);
-	}
+	GMT_grd_init (&grd, argc, argv, FALSE);
+	GMT_err_fail (GMT_read_grd_info (grdfile, &grd), grdfile);
 
 	if (!project_info.region_supplied) {	/* No subset asked for */
 		west = grd.x_min;
@@ -205,31 +200,28 @@ int main (int argc, char **argv)
 	else if (!project_info.region)	/* Got w/s/e/n, make into w/e/s/n */
 		d_swap (south, east);
 
-	nx = (size_t)GMT_get_n (west, east, grd.x_inc, grd.node_offset);
-	ny = (size_t)GMT_get_n (south, north, grd.y_inc, grd.node_offset);
-	nm = nx * ny;
+	nx = GMT_get_n (west, east, grd.x_inc, grd.node_offset);
+	ny = GMT_get_n (south, north, grd.y_inc, grd.node_offset);
+	nm = GMT_get_nm (nx, ny);
 
 	f = (float *) GMT_memory (VNULL, nm, sizeof (float), GMT_program);
 
-	if (GMT_read_grd (grdfile, &grd, f, west, east, south, north, GMT_pad, FALSE)) {
-		fprintf (stderr, "%s: Error reading file %s\n", GMT_program, grdfile);
-		exit (EXIT_FAILURE);
-	}
+	GMT_err_fail (GMT_read_grd (grdfile, &grd, f, west, east, south, north, GMT_pad, FALSE), grdfile);
 
 	ij_inc[0] = 0;	ij_inc[1] = 1;	ij_inc[2] = 1 - nx;	ij_inc[3] = -(long)nx;
 	mode = (Ctrl->T.active) ? 1 : 0;
 	cellsize = grd.x_inc * grd.y_inc;
 	if (Ctrl->S.active) {
-		m_pr_deg = TWO_PI * gmtdefs.ref_ellipsoid[gmtdefs.ellipsoid].eq_radius / 360.0;
-		if (Ctrl->S.unit == 'k') m_pr_deg *= 0.001;	/* Use km instead */
-		cellsize *= m_pr_deg * m_pr_deg;
+		dist_pr_deg = project_info.DIST_M_PR_DEG;
+		if (Ctrl->S.unit == 'k') dist_pr_deg *= 0.001;	/* Use km instead */
+		cellsize *= dist_pr_deg * dist_pr_deg;
 	}
 
 	n_contours = (Ctrl->C.active) ? irint ((Ctrl->C.high - Ctrl->C.low) / Ctrl->C.inc) + 1 : 1;
 
-	height = (double *) GMT_memory (VNULL, (size_t)n_contours, sizeof (double), GMT_program);
-	vol = (double *) GMT_memory (VNULL, (size_t)n_contours, sizeof (double), GMT_program);
-	area = (double *) GMT_memory (VNULL, (size_t)n_contours, sizeof (double), GMT_program);
+	height = (double *) GMT_memory (VNULL, n_contours, sizeof (double), GMT_program);
+	vol = (double *) GMT_memory (VNULL, n_contours, sizeof (double), GMT_program);
+	area = (double *) GMT_memory (VNULL, n_contours, sizeof (double), GMT_program);
 
 	if (!(Ctrl->Z.scale == 1.0 && Ctrl->Z.offset == 0.0)) {
 		if (gmtdefs.verbose) fprintf (stderr, "%s: Subtracting %g and multiplying by %g\n", GMT_program, Ctrl->Z.offset, Ctrl->Z.scale);
@@ -374,19 +366,19 @@ int main (int argc, char **argv)
 
 	/* Print out final estimates */
 
-        sprintf (format, "%s\t%s\t%s\t%s\n", gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format);
+        sprintf (format, "%s%s%s%s%s%s%s\n", gmtdefs.d_format, gmtdefs.field_delimiter, gmtdefs.d_format, gmtdefs.field_delimiter, gmtdefs.d_format, gmtdefs.field_delimiter, gmtdefs.d_format);
 
 	if (full) {
-		sprintf (format, "%s\t%s\t%s\t%s\n", gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format);
-		for (c = 0; c < n_contours; c++) fprintf (GMT_stdout, format, Ctrl->C.low + c * Ctrl->C.inc, area[c], vol[c], height[c]);
+		sprintf (format, "%s%s%s%s%s%s%s\n", gmtdefs.d_format, gmtdefs.field_delimiter, gmtdefs.d_format, gmtdefs.field_delimiter, gmtdefs.d_format, gmtdefs.field_delimiter, gmtdefs.d_format);
+		for (c = 0; c < n_contours; c++) fprintf (stdout, format, Ctrl->C.low + c * Ctrl->C.inc, area[c], vol[c], height[c]);
 	}
 	else if (Ctrl->C.active) {
-        	sprintf (format, "%s\t%s\t%s\t%s\n", gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format);
-		fprintf (GMT_stdout, format, Ctrl->C.low + c * Ctrl->C.inc, area[c], vol[c], height[c]);
+        	sprintf (format, "%s%s%s%s%s%s%s\n", gmtdefs.d_format, gmtdefs.field_delimiter, gmtdefs.d_format, gmtdefs.field_delimiter, gmtdefs.d_format, gmtdefs.field_delimiter, gmtdefs.d_format);
+		fprintf (stdout, format, Ctrl->C.low + c * Ctrl->C.inc, area[c], vol[c], height[c]);
 	}
 	else {
-        	sprintf (format, "%s\t%s\t%s\n", gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format);
-		fprintf (GMT_stdout, format, area[c], vol[c], height[c]);
+        	sprintf (format, "%s%s%s%s%s\n", gmtdefs.d_format, gmtdefs.field_delimiter, gmtdefs.d_format, gmtdefs.field_delimiter, gmtdefs.d_format);
+		fprintf (stdout, format, area[c], vol[c], height[c]);
 	}
 
 	GMT_free ((void *)f);
@@ -394,14 +386,14 @@ int main (int argc, char **argv)
 	GMT_free ((void *)vol);
 	GMT_free ((void *)height);
 
-	GMT_end (argc, argv);
+	Free_grdvolume_Ctrl (Ctrl);	/* Deallocate control structure */
 
-	Free_Grdvolume_Ctrl (Ctrl);	/* Deallocate control structure */
+	GMT_end (argc, argv);
 
 	exit (EXIT_SUCCESS);
 }
 
-void SW_triangle (float f[], size_t ij, size_t nx, BOOLEAN triangle, double *dv, double *da)
+void SW_triangle (float f[], GMT_LONG ij, GMT_LONG nx, GMT_LONG triangle, double *dv, double *da)
 {	/* Calculates area of a SW-corner triangle */
 	/* triangle = TRUE gets triangle, FALSE gives the complementary area */
 	double x1, y0, frac;
@@ -419,7 +411,7 @@ void SW_triangle (float f[], size_t ij, size_t nx, BOOLEAN triangle, double *dv,
 	}
 }
 
-void NE_triangle (float f[], size_t ij, size_t nx, BOOLEAN triangle, double *dv, double *da)
+void NE_triangle (float f[], GMT_LONG ij, GMT_LONG nx, GMT_LONG triangle, double *dv, double *da)
 {	/* Calculates area of a NE-corner triangle */
 	/* triangle = TRUE gets triangle, FALSE gives the complementary area */
 	double x0, y1, a, x0_1, y1_1, frac = 0.0;
@@ -442,7 +434,7 @@ void NE_triangle (float f[], size_t ij, size_t nx, BOOLEAN triangle, double *dv,
 	}
 }
 
-void SE_triangle (float f[], size_t ij, size_t nx, BOOLEAN triangle, double *dv, double *da)
+void SE_triangle (float f[], GMT_LONG ij, GMT_LONG nx, GMT_LONG triangle, double *dv, double *da)
 {	/* Calculates area of a SE-corner triangle */
 	/* triangle = TRUE gets triangle, FALSE gives the complementary area */
 	double x0, y1, c, x0_1, frac = 0.0;
@@ -464,7 +456,7 @@ void SE_triangle (float f[], size_t ij, size_t nx, BOOLEAN triangle, double *dv,
 	}
 }
 
-void NW_triangle (float f[], size_t ij, size_t nx, BOOLEAN triangle, double *dv, double *da)
+void NW_triangle (float f[], GMT_LONG ij, GMT_LONG nx, GMT_LONG triangle, double *dv, double *da)
 {	/* Calculates area of a NW-corner triangle */
 	/* triangle = TRUE gets triangle, FALSE gives the complementary area */
 	double x1, y0, y0_1, frac;
@@ -483,7 +475,7 @@ void NW_triangle (float f[], size_t ij, size_t nx, BOOLEAN triangle, double *dv,
 	}
 }
 
-void NS_trapezoid (float f[], size_t ij, size_t nx, BOOLEAN right, double *dv, double *da)
+void NS_trapezoid (float f[], GMT_LONG ij, GMT_LONG nx, GMT_LONG right, double *dv, double *da)
 {	/* Calculates area of a NS trapezoid */
 	/* right = TRUE gets the right trapezoid, FALSE gets the left */
 	double x0, x1;
@@ -500,7 +492,7 @@ void NS_trapezoid (float f[], size_t ij, size_t nx, BOOLEAN right, double *dv, d
 	}
 }
 
-void EW_trapezoid (float f[], size_t ij, size_t nx, BOOLEAN top, double *dv, double *da)
+void EW_trapezoid (float f[], GMT_LONG ij, GMT_LONG nx, GMT_LONG top, double *dv, double *da)
 {	/* Calculates area of a EW trapezoid */
 	/* top = TRUE gets the top trapezoid, FALSE gets the bottom */
 	double y0, y1;
@@ -521,7 +513,7 @@ void EW_trapezoid (float f[], size_t ij, size_t nx, BOOLEAN top, double *dv, dou
  * lines x0 and x1 and two horizontal lines y0 = ax +b and y1 = cx + d
  */
 
-double vol_prism_frac_x (float *z, size_t ij, size_t nx, double x0, double x1, double a, double b, double c, double d)
+double vol_prism_frac_x (float *z, GMT_LONG ij, GMT_LONG nx, double x0, double x1, double a, double b, double c, double d)
 {
 	double dzdx, dzdy, dzdxy, ca, db, c2a2, d2b2, cdab, v, x02, x12, x03, x04, x13, x14;
 
@@ -549,7 +541,7 @@ double vol_prism_frac_x (float *z, size_t ij, size_t nx, double x0, double x1, d
  * lines y0 and y1 and two vertical lines x0 = ay +b and x1 = cy + d
  */
 
-double vol_prism_frac_y (float *z, size_t ij, size_t nx, double y0, double y1, double a, double b, double c, double d)
+double vol_prism_frac_y (float *z, GMT_LONG ij, GMT_LONG nx, double y0, double y1, double a, double b, double c, double d)
 {
 	double dzdx, dzdy, dzdxy, ca, db, c2a2, d2b2, cdab, v, y02, y03, y04, y12, y13, y14;
 
@@ -573,9 +565,9 @@ double vol_prism_frac_y (float *z, size_t ij, size_t nx, double y0, double y1, d
 	return (v);
 }
 
-int ors_find_kink (double y[], int n, int mode)
+GMT_LONG ors_find_kink (double y[], GMT_LONG n, GMT_LONG mode)
 {	/* mode: 0 = find maximum, 1 = find curvature kink */
-	int i, ic, im;
+	GMT_LONG i, ic, im;
 	double *c, *f;
 	double median3 (double x[]);
 
@@ -587,7 +579,7 @@ int ors_find_kink (double y[], int n, int mode)
 
 	/* Calculate curvatures */
 
-	c = (double *) GMT_memory (VNULL, (size_t)n, sizeof (double), GMT_program);
+	c = (double *) GMT_memory (VNULL, n, sizeof (double), GMT_program);
 
 	for (i = 1; i < (n-1); i++) c[i] = y[i+1] - 2.0 * y[i] + y[i-1];
 	c[0] = c[1];
@@ -595,7 +587,7 @@ int ors_find_kink (double y[], int n, int mode)
 
 	/* Apply 3-point median filter to curvatures  */
 
-	f = (double *) GMT_memory (VNULL, (size_t)n, sizeof (double), GMT_program);
+	f = (double *) GMT_memory (VNULL, n, sizeof (double), GMT_program);
 	for (i = 1; i < (n-1); i++) f[i] = median3 (&c[i-1]);
 
 	/* Find maximum negative filtered curvature */
@@ -623,10 +615,10 @@ double median3 (double x[])
 	}
 }
 
-void *New_Grdvolume_Ctrl () {	/* Allocate and initialize a new control structure */
+void *New_grdvolume_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct GRDVOLUME_CTRL *C;
 	
-	C = (struct GRDVOLUME_CTRL *) GMT_memory (VNULL, 1, sizeof (struct GRDVOLUME_CTRL), "New_Grdvolume_Ctrl");
+	C = (struct GRDVOLUME_CTRL *) GMT_memory (VNULL, 1, sizeof (struct GRDVOLUME_CTRL), "New_grdvolume_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	C->L.value = GMT_d_NaN;
@@ -634,6 +626,6 @@ void *New_Grdvolume_Ctrl () {	/* Allocate and initialize a new control structure
 	return ((void *)C);
 }
 
-void  Free_Grdvolume_Ctrl (struct GRDVOLUME_CTRL *C) {	/* Deallocate control structure */
+void  Free_grdvolume_Ctrl (struct GRDVOLUME_CTRL *C) {	/* Deallocate control structure */
 	GMT_free ((void *)C);	
 }
