@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: gmtlist.c,v 1.5 2004/04/25 08:47:07 pwessel Exp $
+ *	$Id: gmtlist.c 9733 2012-03-22 18:02:01Z pwessel $
  *
- *    Copyright (c) 1991-2001 by P. Wessel and W. H. F. Smith
+ *    Copyright (c) 1991-2012 by P. Wessel and W. H. F. Smith
  *    See README file for copying and redistribution conditions.
  *--------------------------------------------------------------------*/
 /*
@@ -33,7 +33,7 @@
  *
  *
  */
- 
+
 #include "gmt.h"
 #include "gmt_mgg.h"
 #include "x_system.h"
@@ -42,70 +42,67 @@
 #define MAXLEGS 5000
 #define S_PR_DAY 86400
 
-struct CORR **bin;
-
-int binsize = sizeof(struct CORR);
-int nlegs = 0;
-
-int get_id (char *name);
-
-main (int argc, char **argv)
+int main (int argc, char **argv)
 {
-	int leg_year, rec, i, j, n_records, argno, id, no[10], nval = 0;
-	int start_time = 0, stop_time = 2000000000, dlon, last_lon;
-	int mon1, day1 = 0, year1, hour1, min1, time, last_lat, mon2, day2 = 0, year2, hour2, min2;
-	int yy, mm, dd, hh, mi, ss, jd, dt, last_time, n_cruises = 0;
-	
-	int error = FALSE, wantgmt, want_all = FALSE, geodetic = TRUE;
-	int correct = FALSE, tsec = FALSE, calender = FALSE, do_heading = FALSE;
-	int no_g, no_m, no_t, greenwich = FALSE, do_speed = FALSE, binary = FALSE;
-	
-	char gmtfile[BUFSIZ], corrfile[BUFSIZ], agency[10], ndata, g, m, t, *LIBDIR;
-	
+	int leg_year, n_records, rec, i, j, argno, id, no[10], nval = 0;
+	int start_time = 0, stop_time = 2000000000, dlon, last_lon = 0;
+	int mon1, day1 = 0, year1, hour1, min1, time, last_lat = 0, mon2, day2 = 0, year2, hour2, min2;
+	int yy, mm, dd, hh, mi, ss, jd, dt, last_time = 0, n_cruises = 0;
+	size_t not_used = 0;
+
+	GMT_LONG error = FALSE, wantgmt, want_all = FALSE, geodetic = TRUE;
+	GMT_LONG correct = FALSE, tsec = FALSE, calender = FALSE, do_heading = FALSE;
+	GMT_LONG no_g, no_m, no_t, greenwich = FALSE, do_speed = FALSE, binary = FALSE;
+
+	char gmtfile[BUFSIZ], corrfile[BUFSIZ], agency[10], ndata, g, m, t;
+
 	double lat, lon, grv, mag, top, dist, val, start_dist = 0., stop_dist = 1.0E100;
 	double ds, dx, dy, west = 0.0, east = 360.0, south = -90.0, north = 90.0;
 	double heading, speed, weight = 1.0;
-	
-	struct GMTMGG_TIME *gmt;
-	
-	FILE *fp, *fpc;
-	
-	struct GMTMGG_REC record;
-	
-	if ((LIBDIR = getenv ("GMTHOME")) == (char *)NULL) {
-		fprintf (stderr, "binlegs: Environment variable GMTHOME not set!\n");
-		exit (EXIT_FAILURE);
-	}
 
-	g = m = t = no_g = no_m = no_t = FALSE;
-	
+	struct CORR **bin = NULL;
+
+	int binsize = sizeof (struct CORR);
+	int nlegs = 0;
+
+	int get_id (char *name, int nlegs, struct CORR **bin);
+
+	struct GMTMGG_TIME *gmt = NULL;
+
+	FILE *fp = NULL, *fpc = NULL;
+
+	struct GMTMGG_REC record;
+
+	g = m = t = FALSE;
+	no_g = no_m = no_t = FALSE;
+
 	/* Check and interpret the command line arguments */
-	
-	argc = GMT_begin (argc, argv);
+
+	argc = (int)GMT_begin (argc, argv);
 
 	for (i =1; !error && i < argc; i++) {
 		if (argv[i][0] == '-') {
 			switch(argv[i][1]) {
-			
+
 				case 'H':
 				case 'R':
 				case 'V':
 				case '\0':
-					error += GMT_get_common_args (argv[i], &west, &east, &south, &north);
+					error += GMT_parse_common_options (argv[i], &west, &east, &south, &north);
 					break;
-					
+
 				case 'b':	/* Binary output */
 					binary = TRUE;
 					break;
-					
+
 				case 'C':
 					if (argv[i][2] == 0)	/* Use default corrfile */
-						sprintf (corrfile, "%s%cshare%cmgg%cxx_corrections.b\0", LIBDIR, DIR_DELIM, DIR_DELIM, DIR_DELIM);
+						GMT_getsharepath ("mgg", "cxx_corrections", ".b", corrfile);
 					else
 						strcpy (corrfile, &argv[i][2]);
 					correct = TRUE;
 					break;
-					
+
 				case 'D':		/* Assign start/stop times for sub-section */
 					if (argv[i][2] == 'a') {	/* Start date */
 						sscanf(&argv[i][3], "%d/%d/%d/%d:%d",
@@ -177,7 +174,7 @@ main (int argc, char **argv)
 						}
 					}
 					break;
-						
+
 				case 'S':		/* Assign start/stop position for sub-section */
 					if (argv[i][2] == 'a')	/* Start position */
 						start_dist = atof(&argv[i][3]);
@@ -186,14 +183,14 @@ main (int argc, char **argv)
 					else
 						error = TRUE;
 					break;
-					
+
 				case 'G':
 					geodetic = FALSE;
 					break;
 				case 'W':		/* Assign a weight to these data */
 					weight = atof (&argv[i][2]);
 					break;
-					
+
 				default:		/* Options not recognized */
 					error = TRUE;
 					break;
@@ -202,9 +199,9 @@ main (int argc, char **argv)
 		else
 			n_cruises++;
 	}
-	
+
 	/* Check that the options selected are mutually consistent */
-	
+
 	if (nval > 10) error = TRUE;
 	if (start_dist > stop_dist || start_time > stop_time) error = TRUE;
 	if ((day1 > 0 && start_dist > 0.) || (day2 > 0 && stop_dist < 1.0e100)) error = TRUE;
@@ -214,11 +211,11 @@ main (int argc, char **argv)
 
 	if (error || argc == 1) {	/* Display usage */
 		fprintf(stderr,"usage: gmtlist <cruise(s)> [-C<corrfile>] [-Da<startdate>] [-Db<stopdate>] [-F<dataflags>]\n");
-		fprintf(stderr,"	[-G] [-H] [-R<west>/<east>/<south>/<north>] [-Sa<startdist>] [-Sb<stopdist>] [-V] [-W<Weight>] [-b]\n\n");
-         
-		if (GMT_quick) exit (EXIT_FAILURE);
-              
-		fprintf(stderr,"	<cruises> is one or more legnames, e.g. c2104 v3206 etc.\n");
+		fprintf(stderr,"	[-G] [-H] [%s] [-Sa<startdist>] [-Sb<stopdist>] [-V] [-W<Weight>] [-b]\n\n", GMT_Rgeo_OPT);
+
+		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
+
+		fprintf(stderr,"	<cruises> is one or more legnames, e.g., c2104 v3206 etc.\n");
 		fprintf(stderr,"	OPTIONS:\n\n");
 		fprintf(stderr,"	-C<file> applies crossover corrections to the data. If no file name\n");
 		fprintf(stderr,"	   is given, the default correction file is assumed\n");
@@ -247,18 +244,18 @@ main (int argc, char **argv)
 		fprintf(stderr,"	-V verbose, report progress\n");
 		fprintf(stderr,"	-W sets weight for these data\n");
 		fprintf(stderr,"	-b means binary (double) output [ascii]\n");
-		exit(EXIT_FAILURE);
+		exit (EXIT_FAILURE);
 	}
 
 	gmtmggpath_init();
 
 	if ((west < 0.0 && east > 0.0) || (west < 360.0 && east > 360.0)) greenwich = TRUE;
 	if (!geodetic) greenwich = TRUE;
-	
+
 	if (correct) {	/* Read correction table */
 		if ((fpc = fopen(corrfile, "rb")) == NULL) {
 			fprintf(stderr, "Could not read correction file %s\n", corrfile);
-			exit(EXIT_FAILURE);
+			exit (EXIT_FAILURE);
 		}
 		bin = (struct CORR **) GMT_memory (VNULL, (size_t)MAXLEGS, sizeof (struct CORR *), "gmtlist");
 		i = 0;
@@ -270,10 +267,10 @@ main (int argc, char **argv)
 		fclose(fpc);
 		nlegs = i;
 	}
-			
-		
+
+
 	/* Sort the  order in which the parameters appear */
-	
+
 	if (nval == 0) {		/* Nothing selected, default used */
 		g = m = t = TRUE;	/* No data was specified so all data [default] is output */
 		for (i = 0; i < 10; i++) no[i] = i;
@@ -281,8 +278,8 @@ main (int argc, char **argv)
 		want_all = tsec = TRUE;
 		do_heading = do_speed = TRUE;
 	}
-	 
-	if (!binary && gmtdefs.io_header[GMT_OUT]) {
+
+	if (!binary && GMT_io.io_header[GMT_OUT]) {
 		/* Write out header record */
 		for (i = 0; i < nval; i++) {
 			switch(no[i]) {
@@ -327,7 +324,7 @@ main (int argc, char **argv)
 	}
 
 	for (argno = 1; argno < argc; argno++) {	/* Loop over all the files */
-	
+
 		if (argv[argno][0] == '-') continue;
   		if (gmtmggpath_func (gmtfile, argv[argno])) {
    			fprintf (stderr, "gmtlist : Cannot find leg %s\n", argv[argno]);
@@ -339,9 +336,9 @@ main (int argc, char **argv)
 		}
 
 		if (gmtdefs.verbose) fprintf (stderr, "gmtlist: Now processing cruise %s\n",  argv[argno]);
-		
+
 		/* Read first record of file containing start-year, n_records and info */
-		
+
 		if (fread ((void *)(&leg_year), (size_t)4, (size_t)1, fp) != 1) {
 			fprintf (stderr,"gmtlist: Error while reading first year\n");
 			exit (EXIT_FAILURE);
@@ -354,34 +351,34 @@ main (int argc, char **argv)
 			fprintf (stderr,"gmtlist: Error while reading info-header\n");
 			exit (EXIT_FAILURE);
 		}
-	
+
 		gmt = gmtmgg_init (leg_year);	/* Initialize gmt_structure */
 		if (correct)
-			id = get_id(argv[argno]);
+			id = get_id(argv[argno], nlegs, bin);
 		else
 			id = -1;
-	
+
 		/* Decode date to time in sec if needed */
-	
+
 		if (day1 > 0) gmtmgg_time (&start_time, year1, mon1, day1, hour1, min1, 0, gmt);
 		if (day2 > 0) gmtmgg_time (&stop_time, year2, mon2, day2, hour2, min2, 0, gmt);
-	
+
 		dist = 0.0;
 		time = 0;
-	
+
 		wantgmt = (g || m || t) ? TRUE : FALSE;
 		if (want_all) wantgmt = FALSE;
-		
+
 		/* Start reading data from file */
-	
+
 		for (rec = 0; rec < n_records && dist < stop_dist && time < stop_time; rec++) {
 			if (fread ((void *)(&record), (size_t)18, (size_t)1, fp) != 1) {
 				fprintf (stderr,"gmtlist: Error reading data record no %d\n",rec);
 				exit (EXIT_FAILURE);
 			}
-		
+
 			/* Compute accumulated distance along track (Flat Earth) */
-		
+
 			if (rec == 0) {
 				last_lon = record.lon;
 				last_lat = record.lat;
@@ -393,7 +390,7 @@ main (int argc, char **argv)
 			else {
 				dlon = record.lon - last_lon;
 				if (abs (dlon) > 180000000) dlon = irint(copysign ((double) (360000000 - abs (dlon)), (double)dlon));
-				dx = (double) dlon * cos (0.5e-06*D2R*(double)(record.lat+last_lat));
+				dx = (double) dlon * cosd (0.5e-06*(double)(record.lat+last_lat));
 				dy = (double) (record.lat - last_lat);
 				ds = KMPRDEG * hypot (dx, dy);
 				dt = record.time - last_time;
@@ -407,48 +404,49 @@ main (int argc, char **argv)
 				last_time = record.time;
 			}
 			dist += ds;
-			
+
 			/* Check if record has the required fields */
-			
+
 			if (no_g && record.gmt[0] == GMTMGG_NODATA) continue;
 			if (no_m && record.gmt[1] == GMTMGG_NODATA) continue;
 			if (no_t && record.gmt[2] == GMTMGG_NODATA) continue;
-			
+
 			/* Check if time or dist falls outside specified range */
-		
+
 			if (dist < start_dist) continue;
 			if (dist > stop_dist) continue;
 			if (record.time < start_time) continue;
 			if (record.time > stop_time) continue;
-		
+
 			time = record.time;
 			lat = (double) record.lat*0.000001;
 			lon = (double) record.lon*0.000001;
-		
+
 			/* Check is lat/lon is outside specified area */
-		
+
 			if (lat < south || lat > north) continue;
+			while (lon > east) lon -= 360.0;
 			while (lon < west) lon += 360.0;
 			if (lon > east) continue;
 			while (lon > 360.0) lon -= 360.0;
-		
+
 			grv = (record.gmt[0] != GMTMGG_NODATA) ? (double) record.gmt[0]*0.1 : GMTMGG_NODATA;
 			mag = record.gmt[1];
 			top = record.gmt[2];
-			
+
 			ndata = (g && record.gmt[0] != GMTMGG_NODATA) ? 1 : 0;
 			ndata += (m && record.gmt[1] != GMTMGG_NODATA) ? 1 : 0;
 			ndata += (t && record.gmt[2] != GMTMGG_NODATA) ? 1 : 0;
 			if (ndata == 0 && wantgmt) continue;
-		
+
 			/* This record will now be printed out */
-		
+
 			for (i = 0; i < nval; i++) {
 				switch(no[i]) {
 					case 0:	/* Print out time */
 						if (binary) {
 							val = record.time;
-							fwrite ((void *)&val, sizeof (double), (size_t)1, stdout);
+							not_used = fwrite ((void *)&val, sizeof (double), (size_t)1, stdout);
 						}
 						else if (tsec)
 							printf ("%d", record.time);
@@ -466,81 +464,81 @@ main (int argc, char **argv)
 					case 1:	/* Print out longitude */
 						if (lon > 180.0 && greenwich) lon -= 360.0;
 						if (binary)
-							fwrite ((void *)&lon, sizeof (double), (size_t)1, stdout);
+							not_used = fwrite ((void *)&lon, sizeof (double), (size_t)1, stdout);
 						else
-							(gmtdefs.io_header[GMT_OUT]) ? printf("%.5lf", lon) : printf("%9.5lf", lon);
+							(GMT_io.io_header[GMT_OUT]) ? printf("%.5f", lon) : printf("%9.5f", lon);
 						break;
 					case 2:	/* Print out latitude */
 						if (binary)
-							fwrite ((void *)&lat, sizeof (double), (size_t)1, stdout);
+							not_used = fwrite ((void *)&lat, sizeof (double), (size_t)1, stdout);
 						else
-							(gmtdefs.io_header[GMT_OUT]) ? printf("%.5lf", lat) : printf("%8.5lf", lat);
+							(GMT_io.io_header[GMT_OUT]) ? printf("%.5f", lat) : printf("%8.5f", lat);
 						break;
 					case 3:	/* Print out gravity */
 						if (record.gmt[0] == GMTMGG_NODATA)
-							(binary) ? fwrite ((void *)&GMT_f_NaN, sizeof (double), (size_t)1, stdout) : printf ("NaN");
+							(binary) ? not_used = fwrite ((void *)&GMT_f_NaN, sizeof (double), (size_t)1, stdout) : printf ("NaN");
 						else {
 							if (id >= 0) grv -= bin[id]->dc_shift_gmt[0] + bin[id]->drift_rate_gmt[0] * record.time;
 							if (binary)
-								fwrite ((void *)&grv, sizeof (double), (size_t)1, stdout);
+								not_used = fwrite ((void *)&grv, sizeof (double), (size_t)1, stdout);
 							else
-		 						(gmtdefs.io_header[GMT_OUT]) ? printf("%.2lf", grv) : printf("%9.2lf", grv);
+		 						(GMT_io.io_header[GMT_OUT]) ? printf("%.2f", grv) : printf("%9.2f", grv);
 						}
 		 				break;
 					case 4:	/* Print out magnetics */
 						if (record.gmt[1] == GMTMGG_NODATA)
-							(binary) ? fwrite ((void *)&GMT_f_NaN, sizeof (double), (size_t)1, stdout) : printf ("NaN");
+							(binary) ? not_used = fwrite ((void *)&GMT_f_NaN, sizeof (double), (size_t)1, stdout) : printf ("NaN");
 						else {
 							if (id >= 0) mag -= bin[id]->dc_shift_gmt[1] + bin[id]->drift_rate_gmt[1] * record.time;
 							if (binary)
-								fwrite ((void *)&mag, sizeof (double), (size_t)1, stdout);
+								not_used = fwrite ((void *)&mag, sizeof (double), (size_t)1, stdout);
 							else
-		 						(gmtdefs.io_header[GMT_OUT]) ? printf("%.1lf", mag) : printf("%8.1lf", mag);
+		 						(GMT_io.io_header[GMT_OUT]) ? printf("%.1f", mag) : printf("%8.1f", mag);
 						}
 						break;
 					case 5:	/* Print out bathymetry */
 						if (record.gmt[2] == GMTMGG_NODATA)
-							(binary) ? fwrite ((void *)&GMT_f_NaN, sizeof (double), (size_t)1, stdout) : printf ("NaN");
+							(binary) ? not_used = fwrite ((void *)&GMT_f_NaN, sizeof (double), (size_t)1, stdout) : printf ("NaN");
 						else {
 							if (id >= 0) top -= bin[id]->dc_shift_gmt[2] + bin[id]->drift_rate_gmt[2] * record.time;
 							if (binary)
-								fwrite ((void *)&top, sizeof (double), (size_t)1, stdout);
+								not_used = fwrite ((void *)&top, sizeof (double), (size_t)1, stdout);
 							else
-								(gmtdefs.io_header[GMT_OUT]) ? printf("%.1lf", top) : printf("%8.1lf", top);
+								(GMT_io.io_header[GMT_OUT]) ? printf("%.1f", top) : printf("%8.1f", top);
 						}
 						break;
 					case 6:	/* Print out distance */
 						if (binary)
-							fwrite ((void *)&dist, sizeof (double), (size_t)1, stdout);
+							not_used = fwrite ((void *)&dist, sizeof (double), (size_t)1, stdout);
 						else
-							(gmtdefs.io_header[GMT_OUT]) ? printf("%.3lf", dist) : printf("%9.3lf", dist);
+							(GMT_io.io_header[GMT_OUT]) ? printf("%.3f", dist) : printf("%9.3f", dist);
 						break;
 					case 7:	/* Print out heading */
 						if (heading == GMTMGG_NODATA)
-							(binary) ? fwrite ((void *)&GMT_f_NaN, sizeof (double), (size_t)1, stdout) : printf ("NaN");
+							(binary) ? not_used = fwrite ((void *)&GMT_f_NaN, sizeof (double), (size_t)1, stdout) : printf ("NaN");
 						else {
 							if (binary)
-								fwrite ((void *)&heading, sizeof (double), (size_t)1, stdout);
+								not_used = fwrite ((void *)&heading, sizeof (double), (size_t)1, stdout);
 							else
-								(gmtdefs.io_header[GMT_OUT]) ? printf("%.1lf", heading) : printf("%6.1lf", heading);
+								(GMT_io.io_header[GMT_OUT]) ? printf("%.1f", heading) : printf("%6.1f", heading);
 						}
 						break;
 					case 8:	/* Print out velocity */
 						if (speed == GMTMGG_NODATA)
-							(binary) ? fwrite ((void *)&GMT_f_NaN, sizeof (double), (size_t)1, stdout) : printf ("NaN");
+							(binary) ? not_used = fwrite ((void *)&GMT_f_NaN, sizeof (double), (size_t)1, stdout) : printf ("NaN");
 						else {
 							if (binary)
-								fwrite ((void *)&speed, sizeof (double), (size_t)1, stdout);
+								not_used = fwrite ((void *)&speed, sizeof (double), (size_t)1, stdout);
 							else
-								(gmtdefs.io_header[GMT_OUT]) ? printf("%.2lf", speed) : printf("%6.2lf", speed);
+								(GMT_io.io_header[GMT_OUT]) ? printf("%.2f", speed) : printf("%6.2f", speed);
 						}
 						break;
 					case 9:	/* Print out weight */
 						if (binary)
-							fwrite ((void *)&weight, sizeof (double), (size_t)1, stdout);
+							not_used = fwrite ((void *)&weight, sizeof (double), (size_t)1, stdout);
 
 						else
-							(gmtdefs.io_header[GMT_OUT]) ? printf("%lg", weight) : printf("%6.2lf", weight);
+							(GMT_io.io_header[GMT_OUT]) ? printf("%g", weight) : printf("%6.2f", weight);
 						break;
 				}
 				if (!binary) ((i+1) < nval) ? printf("\t") : printf("\n");
@@ -549,14 +547,17 @@ main (int argc, char **argv)
 		fclose (fp);
 		GMT_free ((void *)gmt);
 	}
-	
+
+	gmtmgg_end ();
+	GMT_end (argc, argv);
+
 	exit (EXIT_SUCCESS);
 }
 
-int get_id (char *name)
+int get_id (char *name, int nlegs, struct CORR **bin)
 {
 	int left, right, mid, cmp;
-	
+
 	left = 0;
 	right = nlegs-1;
 	while (left <= right) {
