@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: gmtpath.c,v 1.3 2005/12/27 04:28:12 pwessel Exp $
+ *	$Id: gmtpath.c 9656 2012-01-13 18:29:02Z pwessel $
  *
- *    Copyright (c) 1991-2006 by P. Wessel and W. H. F. Smith
+ *    Copyright (c) 1991-2012 by P. Wessel and W. H. F. Smith
  *    See README file for copying and redistribution conditions.
  *--------------------------------------------------------------------*/
 /*
@@ -19,6 +19,8 @@ int main (int argc, char **argv)
 {
 	char path[BUFSIZ];
 	int i, error = FALSE;
+
+	argc = (int)GMT_begin (argc, argv);
 	
 	gmtmggpath_init();
 
@@ -37,5 +39,8 @@ int main (int argc, char **argv)
 		exit (EXIT_FAILURE);
 	}
 
+	gmtmgg_end ();
+	GMT_end (argc, argv);
+
 	exit (EXIT_SUCCESS);
 }
