@@ -1,12 +1,12 @@
 /*--------------------------------------------------------------------
- *	$Id: nearneighbor.c,v 1.38 2006/04/13 08:34:41 pwessel Exp $
+ *	$Id: nearneighbor.c 9729 2012-03-21 23:19:37Z pwessel $
  *
- *	Copyright (c) 1991-2006 by P. Wessel and W. H. F. Smith
- *	See COPYING file for copying and redistribution conditions.
+ *	Copyright (c) 1991-2012 by P. Wessel and W. H. F. Smith
+ *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
- *	the Free Software Foundation; version 2 of the License.
+ *	the Free Software Foundation; version 2 or any later version.
  *
  *	This program is distributed in the hope that it will be useful,
  *	but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -33,44 +33,47 @@
  
 #include "gmt.h"
 
+#define NN_DEF_SECTORS	4
+#define NN_MIN_SECTORS	2
+
 struct NEARNEIGHBOR_CTRL {	/* All control options for this program (except common args) */
 	/* active is TRUE if the option has been activated */
 	struct I {	/* -Idx[/dy] */
-		BOOLEAN active;
+		GMT_LONG active;
 		double xinc, yinc;
 	} I;
 	struct E {	/* -E<empty> */
-		BOOLEAN active;
-		double value;	
+		GMT_LONG active;
+		double value;
 	} E;
 	struct F {	/* -F */
-		BOOLEAN active;
+		GMT_LONG active;
 	} F;
 	struct G {	/* -G<grdfile> */
-		BOOLEAN active;
+		GMT_LONG active;
 		char *file;
 	} G;
 	struct L {	/* -L<flag> */
-		BOOLEAN active;
+		GMT_LONG active;
 		char mode[4];
 	} L;
 	struct N {	/* -N<sectors> */
-		BOOLEAN active;
-		int sectors;	
+		GMT_LONG active;
+		GMT_LONG sectors, min_sectors;
 	} N;
 	struct S {	/* -S<radius>[m|c|k|K] */
-		BOOLEAN active;
+		GMT_LONG active;
 		double radius;
 		char unit;
 	} S;
 	struct W {	/* -W */
-		BOOLEAN active;
+		GMT_LONG active;
 	} W;
 };
 
 struct NEARNEIGHBOR_NODE {	/* Structure with point id and distance pairs for all sectors */
 	float *distance;	/* Distance of nearest datapoint to this node per sector */
-	int *datum;		/* Point id of this data point */
+	GMT_LONG *datum;		/* Point id of this data point */
 };
 
 struct NEARNEIGHBOR_POINT {	/* Structure with input data constraints */
@@ -79,18 +82,19 @@ struct NEARNEIGHBOR_POINT {	/* Structure with input data constraints */
 
 int main (int argc, char **argv)
 {
-	int i, j, k, ij, i0, j0, *di, dj, sector, n, n_alloc = 5 * GMT_CHUNK, n_fields, nx_2;
-	int n_set, n_almost, n_none, n_files = 0, n_args, fno, one_or_zero, n_expected_fields, pad[4], distance_flag = 0;
-	int max_di, actual_max_di, ii, jj, n_req, x_wrap, y_wrap;
+	GMT_LONG i, j, k, i0, j0, ij, n, nm, n_alloc = GMT_CHUNK, n_set, n_filled, n_almost, n_none;
+	GMT_LONG *di = NULL, dj, sector, n_read, n_fields, ii, jj;
+	GMT_LONG n_files = 0, n_args, fno, n_expected_fields, distance_flag = 0;
+	GMT_LONG max_di, actual_max_di, n_req, x_wrap, y_wrap;
 
-	BOOLEAN go, error = FALSE, done = FALSE, nofile = TRUE;
-	BOOLEAN wrap_180, replicate_x, replicate_y;
+	GMT_LONG error = FALSE, done = FALSE, nofile = TRUE;
+	GMT_LONG wrap_180, replicate_x, replicate_y;
 
-	double weight, weight_sum, grd_sum, *x0, *y0, dx, dy, delta, distance = 0.0, factor;
-	double *in, *shrink, x_left, x_right, y_top, y_bottom, xinc2, yinc2, idx, idy;
+	double weight, weight_sum, grd_sum, *x0 = NULL, *y0 = NULL, dx, dy, delta, distance = 0.0, factor;
+	double *in = NULL, *shrink = NULL, x_left, x_right, y_top, y_bottom, xinc2, yinc2, idx, idy;
 	double half_y_width, y_width, half_x_width, x_width, three_over_radius;
 
-	float *grd;
+	float *grd = NULL;
 
 	char line[BUFSIZ];
 
@@ -98,23 +102,22 @@ int main (int argc, char **argv)
 
 	struct GRD_HEADER header;
 	struct GMT_EDGEINFO edgeinfo;
-	struct NEARNEIGHBOR_NODE **grid_node;
-	struct NEARNEIGHBOR_POINT  *point;
-	struct NEARNEIGHBOR_CTRL *Ctrl;
+	struct NEARNEIGHBOR_NODE **grid_node = NULL;
+	struct NEARNEIGHBOR_POINT  *point = NULL;
+	struct NEARNEIGHBOR_CTRL *Ctrl = NULL;
 
-	void assign_node (struct NEARNEIGHBOR_NODE **node, int n_sector, int sector, double distance, int id);
-	void *New_Nearneighbor_Ctrl (), Free_Nearneighbor_Ctrl (struct NEARNEIGHBOR_CTRL *C);
+	void assign_node (struct NEARNEIGHBOR_NODE **node, GMT_LONG n_sector, GMT_LONG sector, double distance, GMT_LONG id);
+	void free_node (struct NEARNEIGHBOR_NODE *node);
+	void *New_nearneighbor_Ctrl (), Free_nearneighbor_Ctrl (struct NEARNEIGHBOR_CTRL *C);
 
-	argc = GMT_begin (argc, argv);
+	argc = (int)GMT_begin (argc, argv);
+
+	Ctrl = (struct NEARNEIGHBOR_CTRL *) New_nearneighbor_Ctrl ();	/* Allocate and initialize a new control structure */
 
-	Ctrl = (struct NEARNEIGHBOR_CTRL *) New_Nearneighbor_Ctrl ();	/* Allocate and initialize a new control structure */
-	
 	GMT_boundcond_init (&edgeinfo);
 
 	GMT_grd_init (&header, argc, argv, FALSE);
 
-	pad[0] = pad[1] = pad[2] = pad[3] = 0;
-
 	for (i = 1; i < argc; i++) {
 		if (argv[i][0] == '-') {
 			switch (argv[i][1]) {
@@ -158,7 +161,7 @@ int main (int argc, char **argv)
 				case 'L':
 					if (argv[i][2]) {
 						Ctrl->L.active = TRUE;
-						strncpy (Ctrl->L.mode, &argv[i][2], 4);
+						strncpy (Ctrl->L.mode, &argv[i][2], (size_t)4);
 					}
 					else {
 						GMT_io.in_col_type[0] = GMT_io.out_col_type[0] = GMT_IS_LON;
@@ -168,7 +171,9 @@ int main (int argc, char **argv)
 					break;
 				case 'N':
 					Ctrl->N.active = TRUE;
-					Ctrl->N.sectors = atoi (&argv[i][2]);
+					n = sscanf (&argv[i][2], "%" GMT_LL "d/%" GMT_LL "d", &Ctrl->N.sectors, &Ctrl->N.min_sectors);
+					if (n < 1) Ctrl->N.sectors = NN_DEF_SECTORS;
+					if (n < 2) Ctrl->N.min_sectors = NN_MIN_SECTORS;
 					break;
 				case 'S':
 					Ctrl->S.active = TRUE;
@@ -191,37 +196,40 @@ int main (int argc, char **argv)
 	if (argc == 1 || GMT_give_synopsis_and_exit) {
 		fprintf (stderr, "nearneighbor %s - A \"Nearest neighbor\" gridding algorithm\n\n", GMT_VERSION);
 		fprintf(stderr, "usage: nearneighbor [xyzfile(s)] -G<out_grdfile> %s\n", GMT_I_OPT);
-		fprintf(stderr, "\t-N<sectors> %s -S<radius>[m|c|k|K] [-E<empty>] [-F]\n", GMT_Rgeo_OPT);
+		fprintf(stderr, "\t-N<sectors>[/<min_sectors>] %s -S<radius>[m|c|k|K] [-E<empty>] [-F]\n", GMT_Rgeo_OPT);
 		fprintf(stderr, "\t[%s] [-L<flags>] [-V ] [-W] [%s] [%s] [%s]\n\n", GMT_H_OPT, GMT_t_OPT, GMT_bi_OPT, GMT_f_OPT);
 		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
 		fprintf(stderr, "\t-G name of output grid.\n");
 		GMT_inc_syntax ('I', 0);
-		fprintf(stderr, "\t-N sets number of sectors. Default is quadrant search [4].\n");
+		fprintf(stderr, "\t-N sets number of sectors and minimum number to be filled.\n");
+		fprintf(stderr, "\t   Default is quadrant search [%d], requiring at least %d to be filled.\n", NN_DEF_SECTORS, NN_MIN_SECTORS);
 		GMT_explain_option ('R');
 		fprintf(stderr, "\t-S sets search radius in -R, -I units; append m or c for minutes or seconds.\n");
 		fprintf(stderr, "\t   Append k for km (implies -R,-I in degrees), use flat Earth approximation.\n");
 		fprintf(stderr, "\t   Append K for km (implies -R,-I in degrees), use exact geodesic distances.\n");
-		fprintf(stderr, "\t   If the current ELLIPSOID is Sphere then spherical great circle distances are used.\n");
+		fprintf(stderr, "\t   If the current ELLIPSOID is sperical then great circle distances are used.\n");
 		fprintf(stderr, "\n\tOPTIONS:\n");
 		fprintf(stderr, "\t-E value to use for empty nodes [Default is NaN].\n");
 		fprintf(stderr, "\t-F Force pixel registration [Default is gridline registration].\n");
 		GMT_explain_option ('H');
 		fprintf(stderr, "\t-L sets boundary conditions.  <flags> can be either\n");
 		fprintf(stderr, "\t   g for geographic boundary conditions, or one or both of\n");
-		fprintf(stderr, "\t   x for periodic boundary conditions on x\n");
-		fprintf(stderr, "\t   y for periodic boundary conditions on y\n");
+		fprintf(stderr, "\t   x for periodic boundary conditions on x,\n");
+		fprintf(stderr, "\t   y for periodic boundary conditions on y.\n");
 		GMT_explain_option ('V');
 		fprintf(stderr, "\t-W input file has observation weights in 4th column.\n");
 		GMT_explain_option (':');
 		GMT_explain_option ('i');
 		GMT_explain_option ('n');
-		fprintf(stderr, "\t   Default is 3 (or 4 if -W is set) columns\n");
+		fprintf(stderr, "\t   Default is 3 (or 4 if -W is set) columns.\n");
 		GMT_explain_option ('f');
 		GMT_explain_option ('.');
 
 		exit (EXIT_FAILURE);
 	}
 
+	GMT_check_lattice (&Ctrl->I.xinc, &Ctrl->I.yinc, &Ctrl->F.active, &Ctrl->I.active);
+
 	if (!project_info.region_supplied) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR:  Must specify -R option\n", GMT_program);
 		error++;
@@ -234,8 +242,8 @@ int main (int argc, char **argv)
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -N option:  Must specify a positive number of sectors\n", GMT_program);
 		error++;
 	}
-	if (Ctrl->S.radius <= 0.0) {
-		fprintf (stderr, "%s: GMT SYNTAX ERROR -S option:  Must specify a positive search radius\n", GMT_program);
+	if (Ctrl->S.radius <= 0.0 || GMT_is_dnan (Ctrl->S.radius)) {
+		fprintf (stderr, "%s: GMT SYNTAX ERROR.  Radius is NaN, zero or negative\n", GMT_program);
 		error++;
 	}
 	if (Ctrl->I.xinc <= 0.0 || Ctrl->I.yinc <= 0.0) {
@@ -249,27 +257,27 @@ int main (int argc, char **argv)
 	n_req = (Ctrl->W.active) ? 4 : 3;
 	if (GMT_io.binary[GMT_IN] && GMT_io.ncol[GMT_IN] == 0) GMT_io.ncol[GMT_IN] = n_req;
 	if (GMT_io.binary[GMT_IN] && n_req > GMT_io.ncol[GMT_IN]) {
-		fprintf (stderr, "%s: GMT SYNTAX ERROR.  binary input data must have at least %d columns\n", GMT_program, n_req);
+		fprintf (stderr, "%s: GMT SYNTAX ERROR.  binary input data must have at least %ld columns\n", GMT_program, n_req);
 		error++;
 	}
 	if (Ctrl->L.active && GMT_boundcond_parse (&edgeinfo, Ctrl->L.mode)) error++;
 
 	if (error) exit (EXIT_FAILURE);
 
-	GMT_put_history (argc, argv);	/* Update .gmtcommands4 */
-
 	if (GMT_io.binary[GMT_IN] && gmtdefs.verbose) {
 		char *type[2] = {"double", "single"};
-		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
+		fprintf (stderr, "%s: Expects %ld-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
 	}
 	if (Ctrl->S.unit == 'k') distance_flag = 1;
 	if (Ctrl->S.unit == 'K') distance_flag = 2;
 
+	if (Ctrl->N.sectors < Ctrl->N.min_sectors) Ctrl->N.min_sectors = Ctrl->N.sectors;	/* Minimum cannot be larger than desired */
+	
 	header.x_inc = Ctrl->I.xinc;
 	header.y_inc = Ctrl->I.yinc;
-	header.node_offset = Ctrl->F.active;
+	header.node_offset = (int)Ctrl->F.active;
 	GMT_RI_prepare (&header);	/* Ensure -R -I consistency and set nx, ny */
-	GMT_grd_RI_verify (&header, 1);
+	GMT_err_fail (GMT_grd_RI_verify (&header, 1), Ctrl->G.file);
 
 	n_expected_fields = (GMT_io.binary[GMT_IN]) ? GMT_io.ncol[GMT_IN] : 3 + Ctrl->W.active;
 
@@ -279,7 +287,6 @@ int main (int argc, char **argv)
         	n_files = 1;
         n_args = (argc > 1) ? argc : 2;
         
-	one_or_zero = !header.node_offset;
 	header.xy_off = 0.5 * header.node_offset;
 	xinc2 = header.xy_off * header.x_inc;
 	yinc2 = header.xy_off * header.y_inc;
@@ -288,17 +295,17 @@ int main (int argc, char **argv)
 
 	GMT_boundcond_param_prep (&header, &edgeinfo);
 
-	if (gmtdefs.verbose) fprintf (stderr, "%s:  Grid dimensions are nx = %d, ny = %d\n", GMT_program,
-		header.nx, header.ny);
+	if (gmtdefs.verbose) fprintf (stderr, "%s: Grid dimensions are nx = %d, ny = %d\n", GMT_program, header.nx, header.ny);
 
-	grid_node = (struct NEARNEIGHBOR_NODE **) GMT_memory (VNULL, (size_t)(header.nx * header.ny), sizeof (struct NEARNEIGHBOR_NODE *), GMT_program);
+	nm = GMT_get_nm (header.nx, header.ny);
+	grid_node = (struct NEARNEIGHBOR_NODE **) GMT_memory (VNULL, (size_t)nm, sizeof (struct NEARNEIGHBOR_NODE *), GMT_program);
 	point = (struct NEARNEIGHBOR_POINT *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (struct NEARNEIGHBOR_POINT), GMT_program);
 
-	di = (int *) GMT_memory (VNULL, (size_t)header.ny, sizeof (int), GMT_program);
-	shrink = (double *) GMT_memory (VNULL, (size_t)header.ny, sizeof (double), GMT_program);
+	di = (GMT_LONG *) GMT_memory (VNULL, header.ny, sizeof (GMT_LONG), GMT_program);
+	shrink = (double *) GMT_memory (VNULL, header.ny, sizeof (double), GMT_program);
 
-	x0 = (double *) GMT_memory (VNULL, (size_t)header.nx, sizeof (double), GMT_program);
-	y0 = (double *) GMT_memory (VNULL, (size_t)header.ny, sizeof (double), GMT_program);
+	x0 = (double *) GMT_memory (VNULL, header.nx, sizeof (double), GMT_program);
+	y0 = (double *) GMT_memory (VNULL, header.ny, sizeof (double), GMT_program);
 	for (i = 0; i < header.nx; i++) x0[i] = header.x_min + i * header.x_inc + xinc2;
 	for (j = 0; j < header.ny; j++) y0[j] = header.y_max - j * header.y_inc - yinc2;
 	if (distance_flag) {	/* Input data is geographical */
@@ -307,11 +314,11 @@ int main (int argc, char **argv)
 		actual_max_di = 0;
 		for (j = 0; j < header.ny; j++) {
 			shrink[j] = cosd (y0[j]);
-			di[j] = (fabs (y0[j]) == 90.0) ? max_di : (int)(ceil (Ctrl->S.radius / (project_info.KM_PR_DEG * header.x_inc * shrink[j])) + 0.1);
+			di[j] = (fabs (y0[j]) == 90.0) ? max_di : (int)(ceil (Ctrl->S.radius / (project_info.DIST_KM_PR_DEG * header.x_inc * shrink[j])) + 0.1);
 			if (di[j] > max_di) di[j] = max_di;
 			if (di[j] > actual_max_di) actual_max_di = di[j];
 		}
-		dj = (int) (ceil (Ctrl->S.radius / (project_info.KM_PR_DEG * header.y_inc)) + 0.1);
+		dj = (int) (ceil (Ctrl->S.radius / (project_info.DIST_KM_PR_DEG * header.y_inc)) + 0.1);
 	}
 	else {	/* Plain Cartesian data */
 		max_di = (int) (ceil (Ctrl->S.radius * idx) + 0.1);
@@ -337,17 +344,29 @@ int main (int argc, char **argv)
 
 	factor = Ctrl->N.sectors / (2.0 * M_PI);
 
-	x_left = header.x_min - actual_max_di * header.x_inc;	x_right = header.x_max + actual_max_di * header.x_inc;
+	/* To allow data points falling outside -R but within the search radius we extend the data domain in all directions */
+
+	x_left = header.x_min;	x_right = header.x_max;	/* This is what -R says */
+	if (GMT_io.in_col_type[0] != GMT_IS_LON || !GMT_grd_is_global (&header)) {
+		x_left  -= actual_max_di * header.x_inc;	/* OK to extend x-domain since not a periodic geographic grid */
+		x_right += actual_max_di * header.x_inc;
+	}
 	y_top = header.y_max + dj * header.y_inc;	y_bottom = header.y_min - dj * header.y_inc;
+	if (GMT_io.in_col_type[1] == GMT_IS_LAT) {	/* For geographic grids we must ensure the extended y-domain is physically possible */
+		if (y_bottom < -90.0) y_bottom = -90.0;
+		if (y_top > 90.0) y_top = 90.0;
+	}
 	x_width = header.x_max - header.x_min;		y_width = header.y_max - header.y_min;
 	half_x_width = 0.5 * x_width;			half_y_width = 0.5 * y_width;
-	nx_2 = edgeinfo.nxp / 2;
-	n = 0;
+	n = n_read = 0;
 	replicate_x = (edgeinfo.nxp && !header.node_offset);	/* Gridline registration has duplicate column */
 	replicate_y = (edgeinfo.nyp && !header.node_offset);	/* Gridline registration has duplicate row */
 	x_wrap = header.nx - 1;			/* Add to node index to go to right column */
 	y_wrap = (header.ny - 1) * header.nx;	/* Add to node index to go to bottom row */
 
+#ifdef DEBUG
+	GMT_memtrack_off (GMT_mem_keeper);
+#endif
 	for (fno = 1; !done && fno < n_args; fno++) {	/* Loop over input files, if any */
 		if (!nofile && argv[fno][0] == '-') continue;
 
@@ -369,44 +388,47 @@ int main (int argc, char **argv)
 
 		while ((n_fields = GMT_input (fp, &n_expected_fields, &in)) >= 0 && !(GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
 
+			n_read++;
+
 			if (GMT_io.status & GMT_IO_MISMATCH) {
-				fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %d\n", GMT_program, n_fields, n_expected_fields, n);
-				exit (EXIT_FAILURE);
+				fprintf (stderr, "%s: Mismatch between actual (%ld) and expected (%ld) fields near line %ld (skipped)\n", GMT_program, n_fields, n_expected_fields, n_read);
+				continue;
 			}
 
-			if (in[0] < x_left || in[0] > x_right) continue;
-			if (in[1] < y_bottom || in[1] > y_top) continue;
+			if (GMT_is_dnan (in[GMT_Z])) continue;					/* Skip if z = NaN */
+			if (GMT_y_is_outside (in[GMT_Y], y_bottom, y_top)) continue;		/* Outside y-range */
+			if (GMT_x_is_outside (&in[GMT_X], x_left, x_right)) continue;	/* Outside x-range (or longitude) */
 
-			point[n].x = (float)in[0];
-			point[n].y = (float)in[1];
-			point[n].z = (float)in[2];
+			point[n].x = (float)in[GMT_X];
+			point[n].y = (float)in[GMT_Y];
+			point[n].z = (float)in[GMT_Z];
 			if (Ctrl->W.active) point[n].w = (float)in[3];
 
 			/* Find indices of the node closest to this data point */
 
-			i0 = GMT_x_to_i(in[0], header.x_min, header.x_inc, header.xy_off, header.nx);
-			j0 = GMT_y_to_j(in[1], header.y_min, header.y_inc, header.xy_off, header.ny);
+			i0 = GMT_x_to_i(in[GMT_X], header.x_min, header.x_inc, header.xy_off, header.nx);
+			j0 = GMT_y_to_j(in[GMT_Y], header.y_min, header.y_inc, header.xy_off, header.ny);
 
 			/* Loop over all nodes within radius of this node */
 
 			for (j = j0 - dj; j <= (j0 + dj); j++) {
 
-				jj = j;
+				jj = (int)j;
 				if (GMT_y_out_of_bounds (&jj, &header, &edgeinfo, &wrap_180)) continue;	/* Outside y-range */
 
 				for (i = i0 - di[jj]; i <= (i0 + di[jj]); i++) {
 
-					ii = i;
+					ii = (int)i;
 					if (GMT_x_out_of_bounds (&ii, &header, &edgeinfo, wrap_180)) continue;	/* Outside x-range */ 
 
 					/* Here, (ii,jj) is index of a node (k) inside the grid */
 
-					distance = (GMT_distance_func) (x0[ii], y0[jj], in[0], in[1]);
+					distance = (GMT_distance_func) (x0[ii], y0[jj], in[GMT_X], in[GMT_Y]);
 
 					if (distance > Ctrl->S.radius) continue;	/* Data constraint is too far from this node */
 
-					k = jj * header.nx + ii;
-					dx = in[0] - x0[ii];	dy = in[1] - y0[jj];
+					k = GMT_IJ (jj, ii, header.nx);
+					dx = in[GMT_X] - x0[ii];	dy = in[GMT_Y] - y0[jj];
 
 					/* Check for wrap-around in x or y.  This should only occur if the
 					   search radius is larger than 1/2 the grid width/height so that
@@ -442,16 +464,19 @@ int main (int argc, char **argv)
 				}
 			}
 			n++;
-			if (gmtdefs.verbose && !(n%1000)) fprintf (stderr, "%s: Processed record %10d\r", GMT_program, n);
+			if (gmtdefs.verbose && !(n%1000)) fprintf (stderr, "%s: Processed record %10ld\r", GMT_program, n);
 			if (n == n_alloc) {
-				n_alloc += GMT_CHUNK;
+				n_alloc <<= 1;
 				point = (struct NEARNEIGHBOR_POINT *) GMT_memory ((void *)point, (size_t)n_alloc, sizeof (struct NEARNEIGHBOR_POINT), GMT_program);
 			}
 		}
 		if (fp != GMT_stdin) GMT_fclose (fp);
 	}
-	if (gmtdefs.verbose) fprintf (stderr, "%s: Processed record %10d\n", GMT_program, n);
+	if (gmtdefs.verbose) fprintf (stderr, "%s: Processed record %10ld\n", GMT_program, n);
 
+#ifdef DEBUG
+	GMT_memtrack_on (GMT_mem_keeper);
+#endif
 	point = (struct NEARNEIGHBOR_POINT *) GMT_memory ((void *)point, (size_t)n, sizeof (struct NEARNEIGHBOR_POINT), GMT_program);
 	grd = (float *) GMT_memory (VNULL, (size_t)(header.nx * header.ny), sizeof (float), GMT_program);
 
@@ -462,6 +487,9 @@ int main (int argc, char **argv)
 	if (!Ctrl->E.active) Ctrl->E.value = GMT_d_NaN;
 	three_over_radius = 3.0 / Ctrl->S.radius;
 
+#ifdef DEBUG
+	GMT_memtrack_off (GMT_mem_keeper);
+#endif
 	for (j = ij = 0; j < header.ny; j++) {
 		for (i = 0; i < header.nx; i++, ij++) {
 
@@ -471,10 +499,11 @@ int main (int argc, char **argv)
 				continue;
 			}
 
-			for (k = 0, go = TRUE; go && k < Ctrl->N.sectors; k++) if (grid_node[ij]->datum[k] < 0) go = FALSE;
-			if (!go) { 	/* Not full set of neighbors in all sectors, set to empty and goto next node */
+			for (k = 0, n_filled = 0; k < Ctrl->N.sectors; k++) if (grid_node[ij]->datum[k] >= 0) n_filled++;
+			if (n_filled < Ctrl->N.min_sectors) { 	/* Not minimum set of neighbors in all sectors, set to empty and goto next node */
 				n_almost++;
 				grd[ij] = (float)Ctrl->E.value;
+				free_node (grid_node[ij]);
 				continue;
 			}
 
@@ -483,27 +512,30 @@ int main (int argc, char **argv)
 			n_set++;
 			weight_sum = grd_sum = 0.0;	/* Initialize sums */
 			for (k = 0; k < Ctrl->N.sectors; k++) {
-				delta = three_over_radius * grid_node[ij]->distance[k];
-				weight = 1.0 / (1.0 + delta * delta);	/* This is distance weight */
-				if (Ctrl->W.active) weight *= point[grid_node[ij]->datum[k]].w;	/* This is observation weight */
-				grd_sum += weight * point[grid_node[ij]->datum[k]].z;
-				weight_sum += weight;
+				if (grid_node[ij]->datum[k] >= 0) {
+					delta = three_over_radius * grid_node[ij]->distance[k];
+					weight = 1.0 / (1.0 + delta * delta);	/* This is distance weight */
+					if (Ctrl->W.active) weight *= point[grid_node[ij]->datum[k]].w;	/* This is observation weight */
+					grd_sum += weight * point[grid_node[ij]->datum[k]].z;
+					weight_sum += weight;
+				}
 			}
 			grd[ij] = (float)(grd_sum / weight_sum);
+			free_node (grid_node[ij]);
 		}
-		if (gmtdefs.verbose) fprintf (stderr, "%s: Gridded row %6d\r", GMT_program, j);
+		if (gmtdefs.verbose) fprintf (stderr, "%s: Gridded row %10ld\r", GMT_program, j);
 	}
-	if (gmtdefs.verbose) fprintf (stderr, "%s: Gridded row %6d\n", GMT_program, j);
+	if (gmtdefs.verbose) fprintf (stderr, "%s: Gridded row %10ld\n", GMT_program, j);
+#ifdef DEBUG
+	GMT_memtrack_on (GMT_mem_keeper);
+#endif
 
-	if (GMT_write_grd (Ctrl->G.file, &header, grd, 0.0, 0.0, 0.0, 0.0, pad, FALSE)) {
-		fprintf (stderr, "%s: Error writing file %s\n", GMT_program, Ctrl->G.file);
-		exit (EXIT_FAILURE);
-	}
+	GMT_err_fail (GMT_write_grd (Ctrl->G.file, &header, grd, 0.0, 0.0, 0.0, 0.0, GMT_pad, FALSE), Ctrl->G.file);
 
 	if (gmtdefs.verbose) {
 		sprintf (line, "%s)\n", gmtdefs.d_format);
-		fprintf (stderr, "%s: %d nodes were assigned an average value\n", GMT_program, n_set);
-		fprintf (stderr, "%s: %d nodes failed sector criteria and %d nodes had no neighbor points (all set to ", GMT_program, n_almost, n_none);
+		fprintf (stderr, "%s: %ld nodes were assigned an average value\n", GMT_program, n_set);
+		fprintf (stderr, "%s: %ld nodes failed sector criteria and %ld nodes had no neighbor points (all set to ", GMT_program, n_almost, n_none);
 		(GMT_is_dnan (Ctrl->E.value)) ? fprintf (stderr, "NaN)\n") : fprintf (stderr, line, Ctrl->E.value);
 	}
 
@@ -515,27 +547,29 @@ int main (int argc, char **argv)
 	GMT_free ((void *)x0);
 	GMT_free ((void *)y0);
 
-	Free_Nearneighbor_Ctrl (Ctrl);	/* Deallocate control structure */
+	Free_nearneighbor_Ctrl (Ctrl);	/* Deallocate control structure */
 
 	GMT_end (argc, argv);
 
 	exit (EXIT_SUCCESS);
 }
 
-struct NEARNEIGHBOR_NODE *add_new_node(int n)
+struct NEARNEIGHBOR_NODE *add_new_node (GMT_LONG n)
 {
-	struct NEARNEIGHBOR_NODE *new, *add_new_node(int n);
-
+	struct NEARNEIGHBOR_NODE *new;
+	
 	new = (struct NEARNEIGHBOR_NODE *) GMT_memory (VNULL, (size_t)1, sizeof (struct NEARNEIGHBOR_NODE), GMT_program);
 	new->distance = (float *) GMT_memory (VNULL, (size_t)n, sizeof (float), GMT_program);
-	new->datum = (int *) GMT_memory (VNULL, (size_t)n, sizeof (int), GMT_program);
+	new->datum = (GMT_LONG *) GMT_memory (VNULL, (size_t)n, sizeof (GMT_LONG), GMT_program);
 	while (n > 0) new->datum[--n] = -1;
 
 	return (new);
 }
 
-void assign_node (struct NEARNEIGHBOR_NODE **node, int n_sector, int sector, double distance, int id)
+void assign_node (struct NEARNEIGHBOR_NODE **node, GMT_LONG n_sector, GMT_LONG sector, double distance, GMT_LONG id)
 {
+	struct NEARNEIGHBOR_NODE *add_new_node(GMT_LONG n);
+
 	/* Allocates node space if not already used and updates the value if closer to node */
 
 	if (!(*node)) *node = add_new_node (n_sector);
@@ -545,17 +579,28 @@ void assign_node (struct NEARNEIGHBOR_NODE **node, int n_sector, int sector, dou
 	}
 }
 
-void *New_Nearneighbor_Ctrl () {	/* Allocate and initialize a new control structure */
+void free_node (struct NEARNEIGHBOR_NODE *node)
+{
+	/* Frees allocated node space */
+
+	if (!node) return;	/* Nothing to do */
+	GMT_free ((void *)node->distance);
+	GMT_free ((void *)node->datum);
+	GMT_free ((void *)node);
+}
+
+void *New_nearneighbor_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct NEARNEIGHBOR_CTRL *C;
-	
-	C = (struct NEARNEIGHBOR_CTRL *) GMT_memory (VNULL, 1, sizeof (struct NEARNEIGHBOR_CTRL), "New_Nearneighbor_Ctrl");
-	
+
+	C = (struct NEARNEIGHBOR_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct NEARNEIGHBOR_CTRL), "New_nearneighbor_Ctrl");
+
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	C->N.sectors = 4;
+	C->N.min_sectors = 4;
 	return ((void *)C);
 }
 
-void Free_Nearneighbor_Ctrl (struct NEARNEIGHBOR_CTRL *C) {	/* Deallocate control structure */
-	if (C->G.file) GMT_free ((void *)C->G.file);	
-	GMT_free ((void *)C);	
+void Free_nearneighbor_Ctrl (struct NEARNEIGHBOR_CTRL *C) {	/* Deallocate control structure */
+	if (C->G.file) free ((void *)C->G.file);
+	GMT_free ((void *)C);
 }
