@@ -1,12 +1,12 @@
 /*--------------------------------------------------------------------
- *	$Id: project.c,v 1.42 2006/10/27 23:45:23 pwessel Exp $
+ *	$Id: project.c 9730 2012-03-22 04:37:33Z pwessel $
  *
- *	Copyright (c) 1991-2006 by P. Wessel and W. H. F. Smith
- *	See COPYING file for copying and redistribution conditions.
+ *	Copyright (c) 1991-2012 by P. Wessel and W. H. F. Smith
+ *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
- *	the Free Software Foundation; version 2 of the License.
+ *	the Free Software Foundation; version 2 or any later version.
  *
  *	This program is distributed in the hope that it will be useful,
  *	but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -42,49 +42,49 @@
 struct PROJECT_CTRL {	/* All control options for this program (except common args) */
 	/* active is TRUE if the option has been activated */
 	struct A {	/* -A<azimuth> */
-		BOOLEAN active;
+		GMT_LONG active;
 		double azimuth;
 	} A;
 	struct C {	/* -C<ox>/<oy> */
-		BOOLEAN active;
+		GMT_LONG active;
 		double x, y;
 	} C;
 	struct D {	/* -D<d_or_g> */
-		BOOLEAN active;
+		GMT_LONG active;
 		char set;
 	} D;
 	struct E {	/* -E<bx>/<by> */
-		BOOLEAN active;
+		GMT_LONG active;
 		double x, y;
 	} E;
 	struct F {	/* -F<flags> */
-		BOOLEAN active;
+		GMT_LONG active;
 		char col[PROJECT_N_FARGS];	/* Character codes for desired output in the right order */
 	} F;
 	struct G {	/* -G<inc> */
-		BOOLEAN active;
+		GMT_LONG active;
 		double inc;
 	} G;
 	struct L {	/* -L[w][<l_min>/<l_max>] */
-		BOOLEAN active;
-		BOOLEAN constrain;
+		GMT_LONG active;
+		GMT_LONG constrain;
 		double min, max;
 	} L;
 	struct N {	/* -N */
-		BOOLEAN active;
+		GMT_LONG active;
 	} N;
 	struct Q {	/* -Q */
-		BOOLEAN active;
+		GMT_LONG active;
 	} Q;
 	struct S {	/* -S */
-		BOOLEAN active;
+		GMT_LONG active;
 	} S;
 	struct T {	/* -T<px>/<py> */
-		BOOLEAN active;
+		GMT_LONG active;
 		double x, y;
 	} T;
 	struct W {	/* -W<w_min>/<w_max> */
-		BOOLEAN active;
+		GMT_LONG active;
 		double min, max;
 	} W;
 };
@@ -97,45 +97,43 @@ struct PROJECT_DATA {
 
 int main (int argc, char **argv)
 {
-	int	i, j, k, n_outputs, n_used, n_total_read, n_alloc = GMT_CHUNK;
-	int	n_files = 0, fno, n_args, n_fields, n_expected_fields;
-	int	n_total_used = 0, n_z = 0, n_items, output_choice[PROJECT_N_FARGS];
+	GMT_LONG	i, n_used, n_total_read, n_total_used = 0, n_alloc = GMT_CHUNK;
+	GMT_LONG	j, k, n_outputs;
+	GMT_LONG	n_files = 0, fno, n_args, n_fields, n_expected_fields;
+	GMT_LONG	n_z = 0, n_items, output_choice[PROJECT_N_FARGS];
 
 	double	xx, yy, cos_theta, sin_theta, sin_lat_to_pole = 1.0;
-	double	theta, d_along, *in, *out = (double *)NULL;
+	double	theta = 0.0, d_along, *in = NULL, *out = (double *)NULL;
 
 	double	a[3], b[3], x[3], xt[3], pole[3], center[3], e[9];
 
-	BOOLEAN	dateline = FALSE, error = FALSE, find_new_point = FALSE, first = TRUE, pure_ascii, skip;
-	BOOLEAN greenwich = FALSE, rads = TRUE, nofile = TRUE, done = FALSE, want_z_output = FALSE;
+	GMT_LONG	dateline = FALSE, error = FALSE, find_new_point = FALSE, first = TRUE, z_first = TRUE, pure_ascii, skip;
+	GMT_LONG greenwich = FALSE, nofile = TRUE, done = FALSE, want_z_output = FALSE;
 
 	FILE *fp = NULL;
 
-	char	modifier, record_str[BUFSIZ], heading[PROJECT_N_FARGS][GMT_TEXT_LEN];
+	char	modifier, record_str[BUFSIZ], heading[PROJECT_N_FARGS][GMT_TEXT_LEN], buffer[GMT_TEXT_LEN];
 	char	txt_a[GMT_LONG_TEXT], txt_b[GMT_LONG_TEXT];
 
-	struct PROJECT_DATA *p_data;
-	struct PROJECT_CTRL *Ctrl;
+	struct PROJECT_DATA *p_data = NULL;
+	struct PROJECT_CTRL *Ctrl = NULL;
 
 	int	compare_distances (const void *point_1, const void *point_2);
-	int	do_flat_earth ();
-	int	solve_right_spherical_triangle ();
-	int	sphere_azim_dist ();
-	double	oblique_setup (double plat, double plon, double *p, double clat, double clon, double *c, int c_given, int rads);
-	void	oblique_transform (double xlat, double xlon, double *x_t_lat, double *x_t_lon, double *p, double *c, int rads);
-	void	make_euler_matrix (double *p, double *e, double *theta, int rads);
+	double	oblique_setup (double plat, double plon, double *p, double *clat, double *clon, double *c, GMT_LONG c_given, GMT_LONG generate);
+	void	oblique_transform (double xlat, double xlon, double *x_t_lat, double *x_t_lon, double *p, double *c);
+	void	make_euler_matrix (double *p, double *e, double theta);
 	void	matrix_3v (double *a, double *x, double *b);
 	void	matrix_2v (double *a, double *x, double *b);
-	void	sphere_project_setup (double alat, double alon, double *a, double blat, double blon, double *b, double *azim, double *p, double *c, int two_pts, int rads);
-	void	flat_project_setup (double alat, double alon, double blat, double blon, double plat, double plon, double *azim, double *e, int two_pts, BOOLEAN pole_set);
+	void	sphere_project_setup (double alat, double alon, double *a, double blat, double blon, double *b, double azim, double *p, double *c, GMT_LONG two_pts);
+	void	flat_project_setup (double alat, double alon, double blat, double blon, double plat, double plon, double *azim, double *e, GMT_LONG two_pts, GMT_LONG pole_set);
 	void	copy_text_from_col3 (char *line, char *z_cols);
-	void	*New_Project_Ctrl (), Free_Project_Ctrl (struct PROJECT_CTRL *C);
+	void	*New_project_Ctrl (), Free_project_Ctrl (struct PROJECT_CTRL *C);
 
-	argc = GMT_begin (argc, argv);
+	argc = (int)GMT_begin (argc, argv);
 
-	Ctrl = (struct PROJECT_CTRL *) New_Project_Ctrl ();		/* Allocate and initialize defaults in a new control structure */
+	Ctrl = (struct PROJECT_CTRL *) New_project_Ctrl ();		/* Allocate and initialize defaults in a new control structure */
 
-	memset ((void *)output_choice, 0, PROJECT_N_FARGS * sizeof (int));
+	memset ((void *)output_choice, 0, PROJECT_N_FARGS * sizeof (GMT_LONG));
 
 	/* New in GMT 4: Must process -N before the rest to ensure col_types are set properly (there is no -H here)  */
 	for (i = 1; i < argc; i++) if (!strcmp (argv[i], "-N")) Ctrl->N.active = TRUE;
@@ -152,10 +150,12 @@ int main (int argc, char **argv)
 				/* Common parameters */
 
 				case 'H':
+				case 'M':
 				case 'V':
 				case ':':
 				case 'b':
 				case 'f':
+				case 'm':
 				case '\0':
 					error += GMT_parse_common_options (argv[i], 0, 0, 0, 0);
 					break;
@@ -223,9 +223,6 @@ int main (int argc, char **argv)
 						}
 					}
 					break;
-				case 'M':
-					GMT_multisegment (&argv[i][2]);
-					break;
 				case 'N': /* Handled above but still in argv */
 					Ctrl->N.active = TRUE;
 					break;
@@ -265,11 +262,11 @@ int main (int argc, char **argv)
 	}
 
 	if (argc == 1 || GMT_give_synopsis_and_exit) {
-		fprintf (stderr, "project %s - project data onto line or great circle, generate track, or translate coordiantes\n\n", GMT_VERSION);
+		fprintf (stderr, "project %s - Project data onto line or great circle, generate track, or translate coordinates\n\n", GMT_VERSION);
 		fprintf(stderr,"usage:	project [files] -C<ox>/<oy> [-A<azimuth>] [-D<d_or_g>] [-E<bx>/<by>]\n");
 		fprintf(stderr,"\t[-F<flags>] [-G<dist>] [%s] [-L[w][<l_min>/<l_max>]]\n", GMT_H_OPT);
-		fprintf(stderr,"\t[%s] [-N] [-Q] [-S] [-T<px>/<py>] [-V] [-W<w_min>/<w_max>]\n", GMT_M_OPT);
-		fprintf(stderr,"\t[%s] [%s] [%s]\n\n", GMT_t_OPT, GMT_b_OPT, GMT_f_OPT);
+		fprintf(stderr,"\t[-N] [-Q] [-S] [-T<px>/<py>] [-V] [-W<w_min>/<w_max>]\n");
+		fprintf(stderr,"\t[%s] [%s] [%s] [%s]\n\n", GMT_t_OPT, GMT_b_OPT, GMT_f_OPT, GMT_m_OPT);
 
 		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
 
@@ -298,16 +295,15 @@ int main (int argc, char **argv)
 		fprintf(stderr,"\t  r,s is the projected position of x,y (taking q = 0) in the (x,y) coordinate system.\n");
 		fprintf(stderr,"\t  p,q may be scaled from degrees into kilometers by the -Q option.  See -L, -Q, -W.\n");
 		fprintf(stderr,"\t  Note z refers to all input data columns beyond the required x,y\n");
-		fprintf(stderr,"\t  [Default is all fields, i.e. -Fxyzpqrs]\n");
-		fprintf(stderr,"\t  If -G is set, -F is not available and output defaults to rsp\n");
-		fprintf(stderr,"\t-G means Generate (r,s,p) points along profile every <dist> units. (No input data used.)\n");
+		fprintf(stderr,"\t  [Default is all fields, i.e., -Fxyzpqrs].\n");
+		fprintf(stderr,"\t  If -G is set, -F is not available and output defaults to rsp.\n");
+		fprintf(stderr,"\t-G means Generate (r,s,p) points along profile every <dist> units (No input data used).\n");
 		fprintf(stderr,"\t   If E given, will generate from C to E; else must give -L<l_min>/<l_max> for length.\n");
 		GMT_explain_option ('H');
 		fprintf(stderr,"\t-L Check the Length along the projected track and use only certain points.\n");
 		fprintf(stderr,"\t  -Lw will use only those points Within the span from C to E (Must have set -E).\n");
 		fprintf(stderr,"\t  -L<l_min>/<l_max> will only use points whose p is [l_min <= p <= l_max].\n");
 		fprintf(stderr,"\t  Default uses all points.  Note p = 0 at C and increases toward E in azim direction.\n");
-		GMT_explain_option ('M');
 		fprintf(stderr,"\t-N means Flat_earth; a cartesian projection is made.  Default is spherical.\n");
 		fprintf(stderr,"\t-Q means convert to Map units, so x,y,r,s are degrees,\n");
 		fprintf(stderr,"\t  while p,q,dist,l_min,l_max,w_min,w_max are km.\n");
@@ -321,10 +317,11 @@ int main (int argc, char **argv)
 		GMT_explain_option (':');
 		GMT_explain_option ('i');
 		GMT_explain_option ('n');
-		fprintf(stderr,"\t  Default is 2 input columns (x, y)\n");
+		fprintf(stderr,"\t  Default is 2 input columns (x, y).\n");
 		GMT_explain_option ('o');
 		GMT_explain_option ('n');
 		GMT_explain_option ('f');
+		GMT_explain_option ('m');
 		GMT_explain_option ('.');
 		exit (EXIT_FAILURE);
 	}
@@ -396,7 +393,7 @@ int main (int argc, char **argv)
 	}
 
 	if (n_outputs > PROJECT_N_FARGS) {
-		fprintf (stderr, "%s: GMT SYNTAX ERROR -F option: Too many output columns selected (%d)\n", GMT_program, n_outputs);
+		fprintf (stderr, "%s: GMT SYNTAX ERROR -F option: Too many output columns selected (%ld)\n", GMT_program, n_outputs);
 		error++;
 	}
 	if (GMT_io.binary[GMT_IN] && GMT_io.io_header[GMT_IN]) {
@@ -421,11 +418,9 @@ int main (int argc, char **argv)
 	
 	if (error) exit (EXIT_FAILURE);
 
-	GMT_put_history (argc, argv);	/* Update .gmtcommands4 */
-
 	if (GMT_io.binary[GMT_IN] && gmtdefs.verbose) {
 		char *type[2] = {"double", "single"};
-		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
+		fprintf (stderr, "%s: Expects %ld-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
 	}
 
 #ifdef SET_IO_MODE
@@ -436,9 +431,9 @@ int main (int argc, char **argv)
 
 	if (n_outputs == 0 && !(Ctrl->G.active) ) {	/* Generate default -F setting (all) */
 		n_outputs = PROJECT_N_FARGS;
-		for (i = 0; i < 2; i++) output_choice[i] = i;
+		for (i = 0; i < 2; i++) output_choice[i] = (int)i;
 		output_choice[2] = -1;
-		for (i = 3; i < n_outputs; i++) output_choice[i] = i-1;
+		for (i = 3; i < n_outputs; i++) output_choice[i] = (int)i - 1;
 		find_new_point = TRUE;
 	}
 
@@ -449,46 +444,45 @@ int main (int argc, char **argv)
 	/* Set up rotation matrix e for flat earth, or pole and center for spherical; get Ctrl->L.min, Ctrl->L.max if stay_within  */
 
 	if (Ctrl->N.active) {
-		flat_project_setup(Ctrl->C.y, Ctrl->C.x, Ctrl->E.y, Ctrl->E.x, Ctrl->T.y, Ctrl->T.x, &Ctrl->A.azimuth, e, Ctrl->E.active, Ctrl->T.active);
-		/* Azimuth is now changed to cartesian theta in radians */
+		theta = Ctrl->A.azimuth;
+		flat_project_setup(Ctrl->C.y, Ctrl->C.x, Ctrl->E.y, Ctrl->E.x, Ctrl->T.y, Ctrl->T.x, &theta, e, Ctrl->E.active, Ctrl->T.active);
+		/* Azimuth (theta) is now cartesian in degrees */
 		if (Ctrl->L.constrain) {
 			Ctrl->L.min = 0.0;
 			xx = Ctrl->E.x - Ctrl->C.x;
 			yy = Ctrl->E.y - Ctrl->C.y;
 			Ctrl->L.max = d_sqrt(xx*xx + yy*yy);
-			if (Ctrl->Q.active) Ctrl->L.max *= project_info.KM_PR_DEG;
+			if (Ctrl->Q.active) Ctrl->L.max *= project_info.DIST_KM_PR_DEG;
 		}
 	}
 	else {
 		if (Ctrl->T.active) {
-			sin_lat_to_pole = oblique_setup(Ctrl->T.y, Ctrl->T.x, pole, Ctrl->C.y, Ctrl->C.x, center, Ctrl->T.active, rads);
+			sin_lat_to_pole = oblique_setup(Ctrl->T.y, Ctrl->T.x, pole, &Ctrl->C.y, &Ctrl->C.x, center, Ctrl->C.active, Ctrl->G.active);
 		}
 		else {
-			sphere_project_setup(Ctrl->C.y, Ctrl->C.x, a, Ctrl->E.y, Ctrl->E.x, b, &Ctrl->A.azimuth, pole, center, Ctrl->E.active, rads);
+			sphere_project_setup(Ctrl->C.y, Ctrl->C.x, a, Ctrl->E.y, Ctrl->E.x, b, Ctrl->A.azimuth, pole, center, Ctrl->E.active);
 		}
-		/* Azimuth is now changed to radians  */
 		if (Ctrl->L.constrain) {
 			Ctrl->L.min = 0.0;
 			Ctrl->L.max = GMT_dot3v(a,b);
-			Ctrl->L.max = d_acos(Ctrl->L.max) * R2D;
-			if (Ctrl->Q.active) Ctrl->L.max *= project_info.KM_PR_DEG;
+			Ctrl->L.max = d_acosd(Ctrl->L.max);
+			if (Ctrl->Q.active) Ctrl->L.max *= project_info.DIST_KM_PR_DEG;
 		}
 	}
 
 	/* Now things are initialized.  We will work in degrees for awhile, so we convert things:  */
 
 	if (Ctrl->Q.active) {
-		Ctrl->G.inc /= project_info.KM_PR_DEG;
-		Ctrl->L.min /= project_info.KM_PR_DEG;
-		Ctrl->L.max /= project_info.KM_PR_DEG;
-		Ctrl->W.min /= project_info.KM_PR_DEG;
-		Ctrl->W.max /= project_info.KM_PR_DEG;
+		Ctrl->G.inc /= project_info.DIST_KM_PR_DEG;
+		Ctrl->L.min /= project_info.DIST_KM_PR_DEG;
+		Ctrl->L.max /= project_info.DIST_KM_PR_DEG;
+		Ctrl->W.min /= project_info.DIST_KM_PR_DEG;
+		Ctrl->W.max /= project_info.DIST_KM_PR_DEG;
 	}
 
 	/*  Now we are ready to work  */
 
-	n_used = 0;
-	n_total_read = 0;
+	n_used = n_total_read = 0;
 
 	if (Ctrl->G.active) {	/* Not input data expected, just generate track from arguments given */
 		n_outputs = 3;
@@ -500,20 +494,24 @@ int main (int argc, char **argv)
 		d_along = Ctrl->L.min;
 		while ((Ctrl->L.max - d_along) > (GMT_CONV_LIMIT*Ctrl->G.inc)) {
 			p_data[n_used].a[2] = d_along;
+			p_data[n_used].t = NULL;	/* Initialize since that is not done by realloc */
+			p_data[n_used].z = NULL;	/* Initialize since that is not done by realloc */
 			n_used++;
 			d_along = Ctrl->L.min + n_used * Ctrl->G.inc;
 			if (n_used == (n_alloc-1)) {
-				n_alloc += GMT_CHUNK;
+				n_alloc <<= 1;
 				p_data = (struct PROJECT_DATA *) GMT_memory ((void *)p_data, (size_t)n_alloc, sizeof (struct PROJECT_DATA), GMT_program);
 			}
 		}
 		p_data[n_used].a[2] = Ctrl->L.max;
+		p_data[n_used].t = NULL;	/* Initialize since that is not done by realloc */
+		p_data[n_used].z = NULL;	/* Initialize since that is not done by realloc */
 		n_used ++;
 
 		/* We need to find r,s  */
 
 		if (Ctrl->N.active) {
-			sincos (Ctrl->A.azimuth, &sin_theta, &cos_theta);
+			sincosd (90.0 + theta, &sin_theta, &cos_theta);
 			for (i = 0; i < n_used; i++) {
 				p_data[i].a[4] = Ctrl->C.x + p_data[i].a[2] * cos_theta;
 				p_data[i].a[5] = Ctrl->C.y + p_data[i].a[2] * sin_theta;
@@ -522,16 +520,11 @@ int main (int argc, char **argv)
 			}
 		}
 		else {
-			xx = Ctrl->C.x;
-			yy = Ctrl->C.y;
-			GMT_geo_to_cart(&yy, &xx, x, rads);
+			GMT_geo_to_cart(Ctrl->C.y, Ctrl->C.x, x, TRUE);
 			for (i = 0; i < n_used; i++) {
-				theta = p_data[i].a[2] / sin_lat_to_pole;
-				make_euler_matrix(pole, e, &theta, rads);
+				make_euler_matrix(pole, e, p_data[i].a[2] / sin_lat_to_pole);
 				matrix_3v(e,x,xt);
-				GMT_cart_to_geo(&yy, &xx, xt, rads);
-				p_data[i].a[4] = xx;
-				p_data[i].a[5] = yy;
+				GMT_cart_to_geo(&(p_data[i].a[5]), &(p_data[i].a[4]), xt, TRUE);
 				while (greenwich && p_data[i].a[4] < 0.0) p_data[i].a[4] += 360.0;
 				while (dateline && p_data[i].a[4] > 180.0) p_data[i].a[4] -= 360.0;
 			}
@@ -541,26 +534,26 @@ int main (int argc, char **argv)
 
 		if (Ctrl->Q.active) {
 			for (i = 0; i < n_used; i++) {
-				p_data[i].a[2] *= project_info.KM_PR_DEG;
-				p_data[i].a[3] *= project_info.KM_PR_DEG;
+				p_data[i].a[2] *= project_info.DIST_KM_PR_DEG;
+				p_data[i].a[3] *= project_info.DIST_KM_PR_DEG;
 			}
 		}
 
 		/* Now output generated track */
 
 		if (!GMT_io.binary[GMT_OUT]) {
-			if (GMT_io.io_header[GMT_OUT]) fprintf (GMT_stdout, "lon\tlat\tdist\n");
+			if (GMT_io.io_header[GMT_OUT]) {sprintf (buffer, "lon%slat%sdist\n", gmtdefs.field_delimiter, gmtdefs.field_delimiter);	GMT_fputs(buffer, GMT_stdout);}
+		}
 
-			for (i = 0; i < n_used; i++) {
-				for (j = 0; j < n_outputs; j++) out[j] = p_data[i].a[output_choice[j]];
-				GMT_output (GMT_stdout, n_outputs, out);
-			}
+		for (i = 0; i < n_used; i++) {
+			for (j = 0; j < n_outputs; j++) out[j] = p_data[i].a[output_choice[j]];
+			GMT_output (GMT_stdout, n_outputs, out);
 		}
 	}
 
 	else {	/* Must read input file */
 
-		n_expected_fields = (GMT_io.ncol[GMT_IN]) ? GMT_io.ncol[GMT_IN] : BUFSIZ;
+		n_expected_fields = (GMT_io.ncol[GMT_IN]) ? GMT_io.ncol[GMT_IN] : GMT_MAX_COLUMNS;
 
 		if (n_files > 0)
 			nofile = FALSE;
@@ -598,11 +591,6 @@ int main (int argc, char **argv)
 			}
 
 			n_fields = GMT_input (fp, &n_expected_fields, &in);
-			n_z = n_expected_fields - 2;
-			if (n_z == 0 && want_z_output) {
-				fprintf (stderr, "%s: No data columns, cannot use z flag in -F\n", GMT_program);
-				exit (EXIT_FAILURE);
-			}
 			n_used = 0;
 
 			while (! (GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
@@ -611,17 +599,25 @@ int main (int argc, char **argv)
 					GMT_write_segmentheader (GMT_stdout, n_expected_fields);
 					n_fields = GMT_input (fp, &n_expected_fields, &in);
 				}
-
-				while (! (GMT_io.status & (GMT_IO_SEGMENT_HEADER | GMT_IO_EOF))) {	/* Keep going until FALSE or = 2 segment header */
-					if (GMT_io.status & GMT_IO_MISMATCH) {
-						fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %d\n", GMT_program, n_fields, n_expected_fields, n_total_read);
+				if (z_first) {
+					n_z = n_expected_fields - 2;
+					if (n_z == 0 && want_z_output) {
+						fprintf (stderr, "%s: No data columns, cannot use z flag in -F\n", GMT_program);
 						exit (EXIT_FAILURE);
 					}
+					z_first = FALSE;
+				}
 
-					xx = in[0];
-					yy = in[1];
+				while (! (GMT_io.status & (GMT_IO_SEGMENT_HEADER | GMT_IO_EOF))) {	/* Keep going until FALSE or = 2 segment header */
 
 					n_total_read ++;
+					if (GMT_io.status & GMT_IO_MISMATCH) {
+						fprintf (stderr, "%s: Mismatch between actual (%ld) and expected (%ld) fields near line %ld (skipped)\n", GMT_program, n_fields, n_expected_fields, n_total_read);
+						continue;
+					}
+
+					xx = in[GMT_X];
+					yy = in[GMT_Y];
 
 					if (Ctrl->N.active) {
 						x[0] = xx - Ctrl->C.x;
@@ -629,7 +625,7 @@ int main (int argc, char **argv)
 						matrix_2v (e,x,xt);
 					}
 					else {
-						oblique_transform(yy, xx, &xt[1], &xt[0], pole, center, rads);
+						oblique_transform(yy, xx, &xt[1], &xt[0], pole, center);
 					}
 
 					skip = ((Ctrl->L.active && (xt[0] < Ctrl->L.min || xt[0] > Ctrl->L.max)) || (Ctrl->W.active && (xt[1] < Ctrl->W.min || xt[1] > Ctrl->W.max)));
@@ -643,19 +639,27 @@ int main (int argc, char **argv)
 					p_data[n_used].a[1] = yy;
 					p_data[n_used].a[2] = xt[0];
 					p_data[n_used].a[3] = xt[1];
+					p_data[n_used].t = NULL;	/* Initialize since that is not done by realloc */
+					p_data[n_used].z = NULL;	/* Initialize since that is not done by realloc */
 					if (n_z) {	/* Copy over z column(s) */
+#ifdef DEBUG
+						GMT_memtrack_off (GMT_mem_keeper);	/* Because it gives way too many pointers to search through */
+#endif
 						if (pure_ascii) {	/* Must store all text beyond x,y columns */
 							p_data[n_used].t = (char *) GMT_memory (VNULL, strlen (GMT_io.current_record), sizeof (char), GMT_program);
 							copy_text_from_col3 (GMT_io.current_record, p_data[n_used].t);
 						}
 						else {
 							p_data[n_used].z = (double *) GMT_memory (VNULL, (size_t)n_z, sizeof (double), GMT_program);
-							memcpy ((void *)p_data[n_used].z, (void *)&in[2], (size_t)(n_z * sizeof(double)));
+							memcpy ((void *)p_data[n_used].z, (void *)&in[GMT_Z], (size_t)(n_z * sizeof(double)));
 						}
+#ifdef DEBUG
+						GMT_memtrack_on (GMT_mem_keeper);
+#endif
 					}
 					n_used++;
 					if (n_used == n_alloc) {
-						n_alloc += GMT_CHUNK;
+						n_alloc <<= 1;
 						p_data = (struct PROJECT_DATA *) GMT_memory ((void *)p_data, (size_t)n_alloc, sizeof (struct PROJECT_DATA), GMT_program);
 					}
 
@@ -669,7 +673,7 @@ int main (int argc, char **argv)
 				if (find_new_point) {	/* We need to find r,s  */
 
 					if (Ctrl->N.active) {
-						sincos (Ctrl->A.azimuth, &sin_theta, &cos_theta);
+						sincosd (theta, &sin_theta, &cos_theta);
 						for (i = 0; i < n_used; i++) {
 							p_data[i].a[4] = Ctrl->C.x + p_data[i].a[2] * cos_theta;
 							p_data[i].a[5] = Ctrl->C.y + p_data[i].a[2] * sin_theta;
@@ -678,16 +682,11 @@ int main (int argc, char **argv)
 						}
 					}
 					else {
-						xx = Ctrl->C.x;
-						yy = Ctrl->C.y;
-						GMT_geo_to_cart(&yy, &xx, x, rads);
+						GMT_geo_to_cart(Ctrl->C.y, Ctrl->C.x, x, TRUE);
 						for (i = 0; i < n_used; i++) {
-							theta = p_data[i].a[2];
-							make_euler_matrix(pole, e, &theta, rads);
+							make_euler_matrix(pole, e, p_data[i].a[2]);
 							matrix_3v(e,x,xt);
-							GMT_cart_to_geo(&yy, &xx, xt, rads);
-							p_data[i].a[4] = xx;
-							p_data[i].a[5] = yy;
+							GMT_cart_to_geo(&(p_data[i].a[5]), &(p_data[i].a[4]), xt, TRUE);
 							while (greenwich && p_data[i].a[4] < 0.0) p_data[i].a[4] += 360.0;
 							while (dateline && p_data[i].a[4] > 180.0) p_data[i].a[4] -= 360.0;
 						}
@@ -698,8 +697,8 @@ int main (int argc, char **argv)
 
 				if (Ctrl->Q.active) {
 					for (i = 0; i < n_used; i++) {
-						p_data[i].a[2] *= project_info.KM_PR_DEG;
-						p_data[i].a[3] *= project_info.KM_PR_DEG;
+						p_data[i].a[2] *= project_info.DIST_KM_PR_DEG;
+						p_data[i].a[3] *= project_info.DIST_KM_PR_DEG;
 					}
 				}
 
@@ -709,10 +708,10 @@ int main (int argc, char **argv)
 					if (first && GMT_io.io_header[GMT_OUT]) {
 						for (j = 0; j < n_outputs; j++) {
 							if (output_choice[j] == -1)
-								fprintf (GMT_stdout, "%s", heading[6]);
+								GMT_fputs (heading[6], GMT_stdout);
 							else
-								fprintf (GMT_stdout, "%s", heading[output_choice[j]]);
-							(j == (n_outputs - 1)) ? fprintf (GMT_stdout, "\n") : fprintf (GMT_stdout, "\t");
+								GMT_fputs (heading[output_choice[j]], GMT_stdout);
+							(j == (n_outputs - 1)) ? GMT_fputs ("\n", GMT_stdout) : GMT_fputs (gmtdefs.field_delimiter, GMT_stdout);
 						}
 						first = FALSE;
 					}
@@ -726,13 +725,13 @@ int main (int argc, char **argv)
 				if (n_z && pure_ascii) {
 					for (i = 0; i < n_used; i++) {
 						for (j = 0; j < n_outputs; j++) {
-							if (output_choice[j] == -1) {	/* Output all z columns as one string */
-								fprintf (GMT_stdout, "%s", p_data[i].t);
-								GMT_free ((void *)p_data[i].t);
+							if (output_choice[j] == -1)	/* Output all z columns as one string */
+								GMT_fputs (p_data[i].t, GMT_stdout);
+							else {
+								sprintf (buffer, gmtdefs.d_format, p_data[i].a[output_choice[j]]);
+								GMT_fputs (buffer, GMT_stdout);
 							}
-							else
-								fprintf (GMT_stdout, gmtdefs.d_format, p_data[i].a[output_choice[j]]);
-							(j == (n_outputs - 1)) ? fprintf (GMT_stdout, "\n") : fprintf (GMT_stdout, "\t");
+							(j == (n_outputs - 1)) ? GMT_fputs ("\n", GMT_stdout) : GMT_fputs (gmtdefs.field_delimiter, GMT_stdout);
 						}
 					}
 				}
@@ -741,7 +740,6 @@ int main (int argc, char **argv)
 						for (j = k = 0; j < n_outputs; j++) {
 							if (output_choice[j] == -1) {	/* Copy over all z columns */
 								memcpy ((void *)&out[k], (void *)p_data[i].z, (size_t)(n_z * sizeof (double)));
-								GMT_free ((void *)p_data[i].z);
 								k += n_z;
 							}
 							else
@@ -760,12 +758,22 @@ int main (int argc, char **argv)
 		}
 	}
 
-	if (gmtdefs.verbose) fprintf(stderr, "%s: %d read, %d used\n", GMT_program, n_total_read, n_total_used);
+	if (gmtdefs.verbose) fprintf(stderr, "%s: %ld read, %ld used\n", GMT_program, n_total_read, n_total_used);
 
+#ifdef DEBUG
+	if (n_z) GMT_memtrack_off (GMT_mem_keeper);
+#endif
+	for (i = 0; i < n_alloc; i++) {
+		if (p_data[i].t) GMT_free ((void *)p_data[i].t);
+		if (p_data[i].z) GMT_free ((void *)p_data[i].z);
+	}
+#ifdef DEBUG
+	if (n_z) GMT_memtrack_on (GMT_mem_keeper);
+#endif
 	GMT_free ((void *)p_data);
 	GMT_free ((void *)out);
 
-	Free_Project_Ctrl (Ctrl);	/* Deallocate control structure */
+	Free_project_Ctrl (Ctrl);	/* Deallocate control structure */
 
 	GMT_end (argc, argv);
 
@@ -787,7 +795,7 @@ int compare_distances (const void *point_1, const void *point_2)
 		return (0);
 }
 
-double oblique_setup (double plat, double plon, double *p, double clat, double clon, double *c, int c_given, int rads)
+double oblique_setup (double plat, double plon, double *p, double *clat, double *clon, double *c, GMT_LONG c_given, GMT_LONG generate)
 {
 	/* routine sets up a unit 3-vector p, the pole of an 
 	   oblique projection, given plat, plon, the position 
@@ -800,8 +808,46 @@ double oblique_setup (double plat, double plon, double *p, double clat, double c
 	   N pole.  In either case, a unit 3-vector c is created
 	   which is the directed normal to the plane of the central
 	   meridian, pointing in the positive normal (east) sense.
-	   rads = TRUE if we need to convert plat, plon, clat, clon
-	   from degrees to radians.  */
+	   Latitudes and longitudes are in degrees. */
+
+	double	s[3];  /* s points to the south pole  */
+	double	x[3];  /* tmp vector  */
+	double cp, sin_lat_to_pole;
+
+	s[0] = s[1] = 0.0;
+	s[2] = -1.0;
+
+	GMT_geo_to_cart(plat, plon, p, TRUE);
+
+	if (c_given) {	/* s points to user's clat, clon  */
+		GMT_geo_to_cart(*clat, *clon, s, TRUE);
+	}
+	GMT_cross3v(p, s, x);
+	GMT_normalize3v(x);
+	GMT_cross3v(x, p, c);
+	GMT_normalize3v(c);
+	cp = GMT_dot3v (p, c);
+	if (!generate) memcpy ((void *)c, (void *)x, 3*sizeof(double));
+	GMT_cart_to_geo(clat, clon, c, TRUE);	/* return the possibly adjusted center  */
+	sin_lat_to_pole = d_sqrt (1.0 - cp * cp);
+	return (sin_lat_to_pole);
+}
+
+#if 0
+double oblique_setup (double plat, double plon, double *p, double *clat, double *clon, double *c, GMT_LONG c_given)
+{
+	/* routine sets up a unit 3-vector p, the pole of an 
+	   oblique projection, given plat, plon, the position 
+	   of this pole in the usual coordinate frame.
+	   c_given = TRUE means that clat, clon are to be used
+	   as the usual coordinates of a point through which the
+	   user wants the central meridian of the oblique
+	   projection to go.  If such a point is not given, then
+	   the central meridian will go through p and the usual
+	   N pole.  In either case, a unit 3-vector c is created
+	   which is the directed normal to the plane of the central
+	   meridian, pointing in the positive normal (east) sense.
+	   Latitudes and longitudes are in degrees. */
 
 	double	s[3];  /* s points to the south pole  */
 	double cp, sin_lat_to_pole;
@@ -809,10 +855,10 @@ double oblique_setup (double plat, double plon, double *p, double clat, double c
 	s[0] = s[1] = 0.0;
 	s[2] = -1.0;
 
-	GMT_geo_to_cart(&plat, &plon, p, rads);
+	GMT_geo_to_cart(plat, plon, p, TRUE);
 
 	if (c_given) {	/* s points to user's clat, clon  */
-		GMT_geo_to_cart(&clat, &clon, s, rads);
+		GMT_geo_to_cart(*clat, *clon, s, TRUE);
 	}
 	GMT_cross3v(p, s, c);
 	GMT_normalize3v(c);
@@ -820,53 +866,44 @@ double oblique_setup (double plat, double plon, double *p, double clat, double c
 	sin_lat_to_pole = d_sqrt (1.0 - cp * cp);
 	return (sin_lat_to_pole);
 }
+#endif
 
-void oblique_transform (double xlat, double xlon, double *x_t_lat, double *x_t_lon, double *p, double *c, int rads)
+void oblique_transform (double xlat, double xlon, double *x_t_lat, double *x_t_lon, double *p, double *c)
 {
 	/* routine takes the point x at conventional (xlat, xlon) and
 	   computes the transformed coordinates (x_t_lat, x_t_lon) in
 	   an oblique reference frame specified by the unit 3-vectors
 	   p (the pole) and c (the directed normal to the oblique
 	   central meridian).  p and c have been computed earlier by
-	   the routine oblique_setup().  rads = TRUE if lats and lons
-	   are in degrees.  */
+	   the routine oblique_setup().
+	   Latitudes and longitudes are in degrees. */
 
 	double	x[3], p_cross_x[3], temp1, temp2;
 
-	GMT_geo_to_cart(&xlat, &xlon, x, rads);
+	GMT_geo_to_cart(xlat, xlon, x, TRUE);
 
 	temp1 = GMT_dot3v(x,p);
-	*x_t_lat = d_asin(temp1);
+	*x_t_lat = d_asind(temp1);
 
 	GMT_cross3v(p,x,p_cross_x);
 	GMT_normalize3v(p_cross_x);
 
 	temp1 = GMT_dot3v(p_cross_x, c);
 	temp2 = GMT_dot3v(x, c);
-	*x_t_lon = copysign( d_acos(temp1), temp2);
-
-	if (rads) {
-		*x_t_lat *= R2D;
-		*x_t_lon *= R2D;
-	}
+	*x_t_lon = copysign(d_acosd(temp1), temp2);
 }
 
-void make_euler_matrix (double *p, double *e, double *theta, int rads)
+void make_euler_matrix (double *p, double *e, double theta)
 {
 	/* Routine to fill an euler matrix e with the elements
 	   needed to rotate a 3-vector about the pole p through
-	   an angle theta.  p is a unit 3-vector.  If rads = TRUE,
-	   we have to convert theta from degrees into radians before
-	   we use it.  */
+	   an angle theta (in degrees).  p is a unit 3-vector.
+	   Latitudes and longitudes are in degrees. */
 
 	double	cos_theta, sin_theta, one_minus_cos_theta;
 	double	pxsin, pysin, pzsin, temp;
 
-	if (rads) {
-		*theta *= D2R;
-	}
-	cos_theta = cos(*theta);
-	sin_theta = sin(*theta);
+	sincosd (theta, &sin_theta, &cos_theta);
 	one_minus_cos_theta = 1.0 - cos_theta;
 
 	pxsin = p[0] * sin_theta;
@@ -912,7 +949,7 @@ void matrix_2v (double *a, double *x, double *b)
 	b[1] = x[0]*a[2] + x[1]*a[3];
 }
 
-void sphere_project_setup (double alat, double alon, double *a, double blat, double blon, double *b, double *azim, double *p, double *c, int two_pts, int rads)
+void sphere_project_setup (double alat, double alon, double *a, double blat, double blon, double *b, double azim, double *p, double *c, GMT_LONG two_pts)
 {
 	/* routine to initialize a pole vector, p, and a central meridian 
 	   normal vector, c, for use in projecting points onto a great circle.
@@ -934,28 +971,27 @@ void sphere_project_setup (double alat, double alon, double *a, double blat, dou
 	   create p_temp = normalized(a X n), where n is the north pole.
 	   p_temp is then rotated about a through the angle azim to give p.
 	   After p is found, then c = normalized(p X a).
+
+	   Latitudes and longitudes are in degrees.
 	*/
 
 	double	e[9];	/* Euler rotation matrix, if needed  */
-	double neg_azim;
 
 	/* First find p vector  */
 
 	if (two_pts) {
-		GMT_geo_to_cart(&alat, &alon, a, rads);
-		GMT_geo_to_cart(&blat, &blon, b, rads);
+		GMT_geo_to_cart(alat, alon, a, TRUE);
+		GMT_geo_to_cart(blat, blon, b, TRUE);
 		GMT_cross3v(a, b, p);
 		GMT_normalize3v(p);
 	}
 	else {
-		GMT_geo_to_cart(&alat, &alon, a, rads);
+		GMT_geo_to_cart(alat, alon, a, TRUE);
 		b[0] = b[1] = 0.0;	/* set b to north pole  */
 		b[2] = 1.0;
 		GMT_cross3v(a, b, c);	/* use c for p_temp  */
 		GMT_normalize3v(c);
-		neg_azim = -(*azim);
-		make_euler_matrix(a, e, &neg_azim, rads);
-		if (rads) *azim *= D2R;
+		make_euler_matrix(a, e, -azim);
 		matrix_3v(e, c, p);	/* c (p_temp) rotates to p  */
 	}
 
@@ -965,32 +1001,28 @@ void sphere_project_setup (double alat, double alon, double *a, double blat, dou
 	GMT_normalize3v(c);
 }
 
-void flat_project_setup (double alat, double alon, double blat, double blon, double plat, double plon, double *azim, double *e, int two_pts, BOOLEAN pole_set)
+void flat_project_setup (double alat, double alon, double blat, double blon, double plat, double plon, double *azim, double *e, GMT_LONG two_pts, GMT_LONG pole_set)
 {
 	/* Sets up stuff for rotation of cartesian 2-vectors, analogous
-	   to the spherical three vector stuff above.  Also change azim
-	   to the cartesian theta, counterclockwise from the x axis.  */
+	   to the spherical three vector stuff above.
+	   Output is the negative cartesian azimuth in degrees.
+	   Latitudes and longitudes are in degrees. */
 
-	if (two_pts) {
-		*azim = d_atan2((blat - alat), (blon - alon));
-	}
-	else if (pole_set) {
-		*azim = d_atan2((plat - alat), (plon - alon)) - 0.5 * M_PI;
-	}
-	else {
-		/* *azim = D2R * (90.0 - *azim); */
-		*azim = -(*azim) * D2R;
-	}
+	if (two_pts)
+		*azim = 90.0 - d_atan2d(blat - alat, blon - alon);
+	else if (pole_set)
+		*azim = 180.0 - d_atan2d(plat - alat, plon - alon);
 
-	e[0] = e[3] = cos(*azim);
-	e[1] = sin(*azim);
+	*azim = -(*azim);
+	e[0] = e[3] = cosd(*azim);
+	e[1] = sind(*azim);
 	e[2] = -e[1];
 }
 
 void copy_text_from_col3 (char *line, char *z_cols)
 {	/* returns the input line starting at the 3rd column */
 
-	int i;
+	GMT_LONG i;
 
 	/* First replace any commas with spaces */
 
@@ -999,16 +1031,16 @@ void copy_text_from_col3 (char *line, char *z_cols)
 	sscanf (line, "%*s %*s %[^\n]", z_cols);
 }
 
-void *New_Project_Ctrl () {	/* Allocate and initialize a new control structure */
+void *New_project_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct PROJECT_CTRL *C;
 	
-	C = (struct PROJECT_CTRL *) GMT_memory (VNULL, 1, sizeof (struct PROJECT_CTRL), "New_Project_Ctrl");
+	C = (struct PROJECT_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct PROJECT_CTRL), "New_project_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	
 	return ((void *)C);
 }
 
-void Free_Project_Ctrl (struct PROJECT_CTRL *C) {	/* Deallocate control structure */
+void Free_project_Ctrl (struct PROJECT_CTRL *C) {	/* Deallocate control structure */
 	GMT_free ((void *)C);	
 }
