@@ -1,14 +1,14 @@
 /*--------------------------------------------------------------------
- *	$Id: pssegyz.c,v 1.20 2006/10/11 02:52:18 remko Exp $
+ *	$Id: pssegyz.c 9656 2012-01-13 18:29:02Z pwessel $
  *
- *    Copyright (c) 1999-2006 by T. Henstock
+ *    Copyright (c) 1999-2012 by T. Henstock
  *    See README file for copying and redistribution conditions.
  *--------------------------------------------------------------------*/
 /* pssegyz program to plot segy files in 3d in postscript with variable trace spacing option
  * uses routines from the GMT pslib (the postscript imagemask for plotting a
  * 1-bit depth bitmap which will not obliterate stuff underneath!
  *
- * Author:	Tim Henstock (Timothy.J.Henstock@soc.soton.ac.uk)
+ * Author:	Tim Henstock (then@noc.soton.ac.uk)
  * Date:	17-Nov-97
  * Version:	1.0
  *
@@ -53,13 +53,17 @@ int bm_nx, bm_ny;
 
 int main (int argc, char **argv)
 {
-	int error = FALSE;
+	GMT_LONG error = FALSE;
 	int xplot_cdp = FALSE, xplot_offset = FALSE, fix_x = FALSE, byte_x = 0; /* parameters for location plot */
 	int yplot_cdp = FALSE, yplot_offset = FALSE, fix_y = FALSE, byte_y = 0;
 	int doclip = FALSE;
 	int normalize = FALSE, do_fill = FALSE, negative = FALSE, plot_wig = FALSE;
 	int no_zero = FALSE, polarity=1;
-	int swap_bytes = !WORDS_BIGENDIAN;
+#ifdef WORDS_BIGENDIAN
+	int swap_bytes = FALSE;
+#else
+	int swap_bytes = TRUE;
+#endif
 
 	int i, nm;
 	int ix, iz, n_traces=10000, n_samp=0, n_sampr=0, shade[3]={0,0,0}, trans[3]={-1,-1,-1};
@@ -76,14 +80,14 @@ int main (int argc, char **argv)
 	float toffset=0.0;
 
 	char input[512];
-	char *text;
-	char *head;
-	long *head2;
+	char *text = NULL;
+	char *head = NULL;
+	int head2;
 
 
 	char reelhead[3200];
-	float *data;
-	SEGYHEAD *header;
+	float *data = NULL;
+	SEGYHEAD *header = NULL;
 	SEGYREEL binhead;
 
 	FILE *fpi = NULL;
@@ -92,7 +96,7 @@ int main (int argc, char **argv)
 	input[0] = 0;
 	w = e = s = n = 0.0;
 
-	argc = GMT_begin (argc, argv);
+	argc = (int)GMT_begin (argc, argv);
 
 	for (i = 1; i < argc; i++) {
 		if (argv[i][0] == '-') {
@@ -109,7 +113,7 @@ int main (int argc, char **argv)
 					error += GMT_parse_common_options (argv[i], &w, &e, &s, &n);
 					break;
                                 case 'E':
-					sscanf (&argv[i][2], "%lf/%lf", &z_project.view_azimuth, &z_project.view_elevation);
+					error += GMT_get_proj3D (&argv[i][2], &z_project.view_azimuth, &z_project.view_elevation);
 					break;
 				/* parameters for wiggle mode */
 				case 'F':
@@ -243,7 +247,7 @@ int main (int argc, char **argv)
 	if (argc == 1 || GMT_give_synopsis_and_exit) {
 		fprintf (stderr, "pssegyz %s - Plot a segy file in PostScript\n\n", GMT_VERSION);
 		fprintf (stderr, "usage: pssegyz [<segyfile>] %s %s -D<dev>\n", GMT_Jx_OPT, GMT_Rx_OPT);
-		fprintf (stderr, "	[-E<az/el>] [-C<clip>] [-B<bias>] [-N] [-Z]\n");
+		fprintf (stderr, "	[%s] [-C<clip>] [-B<bias>] [-N] [-Z]\n", GMT_E_OPT);
 		fprintf (stderr, "	[-F<gray>|<r/g/b>] [-I] [-W] [-S<x>/<y>]\n");
 		fprintf (stderr, "	[-X<dx>] [-Y<dz>] [-L<nsamp>] [-M<ntraces>] \n");
 		fprintf (stderr, "	[-U<redvel>] [-A] [-O] [-K] [-P]\n");
@@ -299,7 +303,7 @@ int main (int argc, char **argv)
  		error++;
  		fprintf(stderr,"%s: SYNTAX ERROR: Must specify a positive deviation\n",GMT_program);
  	}
- 	if (project_info.projection >2){
+ 	if (!GMT_IS_LINEAR){
  		fprintf(stderr,"%s: WARNING: you asked for a non-rectangular projection. \n It will probably still work, but be prepared for problems\n",GMT_program);
  	}
   	if (!project_info.z_bottom && !project_info.z_top){ /* no z range in the -R parameter */
@@ -324,10 +328,8 @@ int main (int argc, char **argv)
 
 	if (fpi == NULL) fpi = stdin;
 
-	GMT_put_history (argc, argv);   /* Update .gmtcommands4 */
-
 /* set up map projection and PS plotting */
-	GMT_map_setup (w, e, s, n);
+	GMT_err_fail (GMT_map_setup (w, e, s, n), "");
 	GMT_plotinit (argc, argv);
 
         /*if (project_info.three_D) ps_transrotate (-z_project.xmin, -z_project.ymin, 0.0);*/
@@ -380,7 +382,7 @@ use a few of these*/
 	}
 
 	if(gmtdefs.verbose) 
-		fprintf(stderr, "%s: Number of samples is %d\n", GMT_program, n_samp);
+		fprintf(stderr, "%s: Number of samples is %dl\n", GMT_program, n_samp);
 
 	if(binhead.dsfc != 5) fprintf(stderr, "%s: WARNING data not in IEEE format\n", GMT_program);
 
@@ -398,25 +400,24 @@ use a few of these*/
 	}
 
 
-	bitmap = (unsigned char *) GMT_memory (NULL, nm, sizeof (char), "pssegyz");
+	bitmap = (unsigned char *) GMT_memory (NULL, (size_t)nm, sizeof (char), "pssegyz");
 
 	ix=0;
 	while ((ix<n_traces) && (header = get_segy_header(fpi))){   /* read traces one by one */
 	/* check true location header for x */
 		if (xplot_offset){ /* plot traces by offset, cdp, or input order */
-			long offset = ((swap_bytes)? GMT_swab4(header->sourceToRecDist): header->sourceToRecDist);
+			int32_t offset = ((swap_bytes)? GMT_swab4(header->sourceToRecDist): header->sourceToRecDist);
 			x0 = (double) offset;
 		}
 		else if (xplot_cdp){
-			long cdpval = ((swap_bytes)? GMT_swab4(header->cdpEns): header->cdpEns);
+			int32_t cdpval = ((swap_bytes)? GMT_swab4(header->cdpEns): header->cdpEns);
 			x0 = (double) cdpval;
 		}
 		else if (byte_x){ /* ugly code - want to get value starting at byte_x of header into a double... */
-			long plot_loc;
 		   	head = (char *) header;
-			head2 = (long *) &(head[byte_x]);
-			plot_loc = ((swap_bytes)? GMT_swab4(*head2): *head2);
-		  	x0 = (double) plot_loc;
+			memcpy(&head2, &head[byte_x], 4); /* edited to fix bug where 8bytes were copied from head.
+                                                Caused by casting to a long directly from char array*/ 
+			x0 = (double) ((swap_bytes)? GMT_swab4(head2): head2);
 		}
 		else if (fix_x)
 			x0 /= dx;
@@ -425,19 +426,18 @@ use a few of these*/
 
 	/* now do same for y */
 		if (yplot_offset){ /* plot traces by offset, cdp, or input order */
-			long offset = ((swap_bytes)? GMT_swab4(header->sourceToRecDist): header->sourceToRecDist);
+			int32_t offset = ((swap_bytes)? GMT_swab4(header->sourceToRecDist): header->sourceToRecDist);
 			y0 = (double) offset;
 		}
 		else if (yplot_cdp){
-			long cdpval = ((swap_bytes)? GMT_swab4(header->cdpEns): header->cdpEns);
+			int32_t cdpval = ((swap_bytes)? GMT_swab4(header->cdpEns): header->cdpEns);
 			y0 = (double) cdpval;
 		}
 		else if (byte_y){
-		  	long plot_loc;
 			head =  (char *) header;
-			head2 = (long *) &(head[byte_y]);
-			plot_loc = ((swap_bytes)? GMT_swab4(*head2): *head2);
-			y0 = (double) plot_loc;
+			memcpy(&head2, &head[byte_y], 4); /* edited to fix bug where 8bytes were copied from head.
+                                                Caused by casting to a long directly from char array*/ 
+			y0 = (double) ((swap_bytes)? GMT_swab4(head2): head2);
 		}
 		else if (fix_y)
 			y0 /= dx;
