@@ -1,11 +1,11 @@
 /*--------------------------------------------------------------------
- *	$Id: grdrotater.c,v 1.16 2006/04/13 06:20:35 pwessel Exp $
+ *	$Id: grdrotater.c 9671 2012-02-21 03:19:51Z pwessel $
  *
- *   Copyright (c) 1999-2006 by P. Wessel
+ *   Copyright (c) 1999-2012 by P. Wessel
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; version 2 of the License.
+ *   the Free Software Foundation; version 2 or any later version.
  *
  *   This program is distributed in the hope that it will be useful,
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,7 +15,7 @@
  *   Contact info: www.soest.hawaii.edu/pwessel
  *--------------------------------------------------------------------*/
 /*
- * grdrotater will read a grdfile, apply a finite rotation to the grid
+ * grdrotater will read a grid file, apply a finite rotation to the grid
  * coordinates, and then interpolate the old grid at the new coordinates.
  *
  * Author:	Paul Wessel
@@ -29,30 +29,29 @@
 
 int main (int argc, char **argv)
 {
-	int i, j, k, ij, ii, jj, nx, ny, mx = 0, my, ix, iy;
-	int iii, jjj, ii_o, jj_o;
-
-	BOOLEAN error = FALSE, bilinear = FALSE, user_polygon_given = FALSE;
-	BOOLEAN inside, node = FALSE, outline = TRUE, skip_grid = FALSE;
+	GMT_LONG i, j, k, ij, ii, jj, nx, ny, mx = 0, my, nm, ix, iy;
+	GMT_LONG iii, jjj, ii_o, jj_o;
+	GMT_LONG error = FALSE, interpolant = BCR_BICUBIC, user_polygon_given = FALSE;
+	GMT_LONG inside, node = FALSE, outline = TRUE, skip_grid = FALSE, global = FALSE;
 	
-	float *original_grid = VNULL, *rotated_grid;
+	float *original_grid = VNULL, *rotated_grid = NULL;
+
 	char txt_a[GMT_TEXT_LEN], txt_b[GMT_TEXT_LEN], txt_c[GMT_TEXT_LEN];
+	char *grdfile = NULL, *outfile = NULL, *pol_file = NULL;
 
 	double  xx, yy, lon, x_rot, y_rot, w_rot = 0.0, P_original[3], P_rotated[3], R[3][3];
 	double west, east, south, north, data_west, data_east, data_south, data_north;
-	double w180, w360, e180, e360, threshold = 1.0, out[2], *grd_x, *grd_y;
+	double w180, w360, e180, e360, threshold = 1.0, out[2], *grd_x = NULL, *grd_y = NULL, *grd_yc = NULL;
 	double wp180, wp360, ep180, ep360, spol, npol;
 
-	char *grdfile, *outfile, *pol_file;
-
 	struct GRD_HEADER g_head, r_head;
-	struct GMT_TABLE *pol;
+	struct GMT_TABLE *pol = NULL;
 	struct GMT_EDGEINFO edgeinfo;
 	struct GMT_BCR bcr;
 
 	void get_grid_path (struct GRD_HEADER *h, struct GMT_TABLE **p);
 
-	argc = GMT_begin (argc, argv);
+	argc = (int)GMT_begin (argc, argv);
 
 	grdfile = outfile = pol_file = CNULL;
 	west = east = south = north = 0.0;
@@ -65,10 +64,12 @@ int main (int argc, char **argv)
 				/* Common parameters */
 
 				case 'H':
+				case 'M':
 				case 'R':
 				case 'V':
 				case ':':
 				case 'b':
+				case 'm':
 				case '\0':
 					error += GMT_parse_common_options (argv[i], &west, &east, &south, &north);
 					break;
@@ -88,15 +89,28 @@ int main (int argc, char **argv)
 				case 'G':
 					outfile = &argv[i][2];
 					break;
-				case 'M':
-					GMT_multisegment (&argv[i][2]);
-					break;
 				case 'N':
 					outline = FALSE;
 					break;
 				case 'Q':
-					bilinear = TRUE;
-					threshold = (argv[i][2]) ? atof (&argv[i][2]) : 1.0;
+					interpolant = BCR_BILINEAR;
+					for (j = 2; j < 5 && argv[i][j]; j++) {
+						switch (argv[i][j]) {
+							case 'n':
+								interpolant = BCR_NEARNEIGHBOR; break;
+							case 'l':
+								interpolant = BCR_BILINEAR; break;
+							case 'b':
+								interpolant = BCR_BSPLINE; break;
+							case 'c':
+								interpolant = BCR_BICUBIC; break;
+							case '/':
+							default:
+								threshold = atof (&argv[i][j]);
+								if (j == 2 && threshold < GMT_SMALL) interpolant = BCR_NEARNEIGHBOR;
+								j = 5; break;
+						}
+					}
 					break;
 				case 'S':
 					skip_grid = TRUE;
@@ -115,27 +129,28 @@ int main (int argc, char **argv)
 	if (argc == 1 || GMT_give_synopsis_and_exit) {
 		fprintf (stderr,"grdrotater %s - Finite rotation reconstruction of geographic grid\n\n", GMT_VERSION);
 		fprintf (stderr, "usage: grdrotater <grdfile> -T<plon>/<plat>/<pomega> -G<rotgrid> [-F<polygonfile>]\n");
-		fprintf (stderr, "\t[%s] [%s] [-N] [-Q[<value>]] [%s] [-S] [-V] [%s]\n", GMT_H_OPT, GMT_M_OPT, GMT_Rgeo_OPT, GMT_t_OPT);
-		fprintf (stderr, "\t[%s] > projpol\n\n", GMT_b_OPT);
+		fprintf (stderr, "\t[%s] [-N] [-Q[b|c|l|n][[/]<threshold>]] [%s] [-S] [-V] [%s]\n", GMT_H_OPT, GMT_Rgeo_OPT, GMT_t_OPT);
+		fprintf (stderr, "\t[%s] [%s] > projpol\n\n", GMT_b_OPT, GMT_m_OPT);
 
 		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
 
 		fprintf (stderr,"\t<grdfile> is a gridded data file in geographic coordinates to be rotated\n");
 		fprintf (stderr,"\t-G is the output filename of the new, rotated grid.  The boundary of the\n");
 		fprintf (stderr,"\t   original grid (or a subset; see -F) after rotation is written to stdout\n");
+		fprintf (stderr,"\t   unless the grid is global.\n");
 		fprintf (stderr,"\t-T sets the rotation pole and opening angle for the finite rotation (all in degrees)\n");
 		fprintf (stderr,"\n\tOPTIONS:\n");
 		fprintf (stderr,"\t-F specifies a multi-segment closed polygon file that describes the area of the grid\n");
 		fprintf (stderr,"\t   that should be projected [Default projects entire grid]\n");
 		GMT_explain_option ('H');
-		GMT_explain_option ('M');
 		fprintf (stderr, "\t-N Do not output the rotated polygon or grid outline\n");
 		fprintf (stderr, "\t-Q Quick mode, use bilinear rather than bicubic [Default] interpolation.\n");
-		fprintf (stderr, "\t   Optionally, append <value> in the 0 <= value <= 1 range.\n");
-		fprintf (stderr, "\t   [Default = 1 requires all 4 nodes to be non-NaN.], <value> = 0.5\n");
-		fprintf (stderr, "\t   will interpolate about 1/2 way from a non-NaN to a NaN node, while\n");
-		fprintf (stderr, "\t   0.1 will go about 90%% of the way, etc. -Q0 will return the value of the \n");
-		fprintf (stderr, "\t   nearest node instead of interpolating.\n");
+		fprintf (stderr, "\t   Alternatively, select interpolation mode by adding b = B-spline, c = bicubic,\n");
+		fprintf (stderr, "\t   l = bilinear, or n = nearest-neighbor.\n");
+		fprintf (stderr, "\t   Optionally, append <threshold> in the range [0,1]. [Default = 1 requires all\n");
+		fprintf (stderr, "\t   4 or 16 nodes to be non-NaN.], <threshold> = 0.5 will interpolate about 1/2 way\n");
+		fprintf (stderr, "\t   from a non-NaN to a NaN node, while 0.1 will go about 90%% of the way, etc.\n");
+		fprintf (stderr, "\t   -Q0 will return the value of the nearest node instead of interpolating (Same as -Qn).\n");
 		GMT_explain_option ('R');
 		fprintf (stderr, "\t-S Do NOT rotate the grid - just produce the rotated outline\n");
 		GMT_explain_option ('V');
@@ -144,6 +159,7 @@ int main (int argc, char **argv)
 		GMT_explain_option ('n');
 		fprintf (stderr, "\t   Default is 2 input columns\n");
 		GMT_explain_option ('o');
+		GMT_explain_option ('m');
 		GMT_explain_option ('.');
 		exit (EXIT_FAILURE);
 	}
@@ -179,7 +195,7 @@ int main (int argc, char **argv)
 	}
 	if (error) exit (EXIT_FAILURE);
 
-	GMT_put_history (argc, argv);	/* Update .gmtcommands4 */
+	GMT_lat_swap_init ();	/* Initialize auxiliary latitude machinery */
 
 	/* Check limits and get data file */
 
@@ -187,10 +203,7 @@ int main (int argc, char **argv)
 
 		GMT_grd_init (&g_head, argc, argv, FALSE);
 
-		if (GMT_read_grd_info (grdfile, &g_head)) {
-			fprintf (stderr, "%s: Error opening file %s\n", GMT_program, grdfile);
-			exit (EXIT_FAILURE);
-		}
+		GMT_err_fail (GMT_read_grd_info (grdfile, &g_head), grdfile);
 	
 		/* Determine what wesn to pass to map_setup */
 
@@ -206,10 +219,11 @@ int main (int argc, char **argv)
 
 		/* Determine the wesn to be used to read the grdfile; or exit if file is outside -R */
 
-		if (GMT_grd_setregion (&g_head, &data_west, &data_east, &data_south, &data_north)) {	/* No grid to plot; just do empty map and exit */
+		if (GMT_grd_setregion (&g_head, &data_west, &data_east, &data_south, &data_north, interpolant)) {
 			fprintf (stderr, "%s: No grid values inside selected region - aborting\n", GMT_program);
 			exit (EXIT_FAILURE);
 		}
+		global = (GMT_IS_ZERO (data_east - data_west - 360.0) && GMT_IS_ZERO (data_north - data_south - 180.0));
 	}
 	if (!skip_grid) {
 		/* Read data */
@@ -219,21 +233,19 @@ int main (int argc, char **argv)
 		ny = irint ( (data_north - data_south) / g_head.y_inc) + !g_head.node_offset;
 		mx = nx + 4;	my = ny + 4;
 		GMT_pad[0] = GMT_pad[1] = GMT_pad[2] = GMT_pad[3] = 2;
+		nm = GMT_get_nm (mx, my);
 
-		original_grid = (float *) GMT_memory (VNULL, (size_t)(mx * my), sizeof (float), GMT_program);
-		if (GMT_read_grd (grdfile, &g_head, original_grid, data_west, data_east, data_south, data_north, GMT_pad, FALSE)) {
-			fprintf (stderr, "%s: Error reading file %s\n", GMT_program, grdfile);
-			exit (EXIT_FAILURE);
-		}
+		original_grid = (float *) GMT_memory (VNULL, (size_t)nm, sizeof (float), GMT_program);
+		GMT_err_fail (GMT_read_grd (grdfile, &g_head, original_grid, data_west, data_east, data_south, data_north, GMT_pad, FALSE), grdfile);
 
 		GMT_boundcond_init (&edgeinfo);
-		if (fabs (fabs (g_head.x_max - g_head.x_min) - 360.0) < GMT_CONV_LIMIT) GMT_boundcond_parse (&edgeinfo, "g");
+		if (GMT_360_RANGE (g_head.x_max, g_head.x_min)) GMT_boundcond_parse (&edgeinfo, "g");
 	
 		GMT_boundcond_param_prep (&g_head, &edgeinfo);
 
 		/* Initialize bcr structure:  */
 
-		GMT_bcr_init (&g_head, GMT_pad, bilinear, threshold, &bcr);
+		GMT_bcr_init (&g_head, GMT_pad, interpolant, threshold, &bcr);
 		if (threshold == 0.0) node = TRUE;
 
 		/* Set boundary conditions  */
@@ -243,10 +255,10 @@ int main (int argc, char **argv)
 	
 	if (user_polygon_given)	/* Read the user's polygon file */
 		GMT_import_table ((void *)pol_file, GMT_IS_FILE, &pol, 0.0, (west < 0.0 && east > 0.0), TRUE, TRUE);
-	else				/* Make a single grid-outline polygon */
+	else if (!global)				/* Make a single grid-outline polygon */
 		get_grid_path (&g_head, &pol);
 
-	make_rot_matrix (x_rot, y_rot, w_rot, R);	/* Make rotation matrix from rotation parameters */
+	spotter_make_rot_matrix (x_rot, y_rot, w_rot, R);	/* Make rotation matrix from rotation parameters */
 	
 	if (gmtdefs.verbose) fprintf (stderr, "%s: Reconstruct polygon outline\n", GMT_program);
 	
@@ -254,13 +266,15 @@ int main (int argc, char **argv)
 	
 	ix = (gmtdefs.xy_toggle[1]);	iy = 1 - ix;	/* These are used for output purposes only */
 	west = w360 = w180 = DBL_MAX;	east = e360 = e180 = -DBL_MAX;	south = DBL_MAX;	north = -DBL_MAX;
-	for (i = 0; i < pol->n_segments; i++) {
+	for (i = 0; !global && i < pol->n_segments; i++) {
 		if (outline) GMT_write_segmentheader (GMT_stdout, 2);
 		wp360 = wp180 = DBL_MAX;	ep360 = ep180 = -DBL_MAX;	spol = DBL_MAX;	npol = -DBL_MAX;
 		for (j = 0; j < pol->segment[i]->n_rows; j++) {
-			GMT_geo_to_cart (&pol->segment[i]->coord[GMT_Y][j], &pol->segment[i]->coord[GMT_X][j], P_original, TRUE);	/* Convert to a Cartesian x,y,z vector; TRUE since we have degrees */
-			matrix_vect_mult (R, P_original, P_rotated);				/* Rotate the vector */
+			pol->segment[i]->coord[GMT_Y][j] = GMT_lat_swap (pol->segment[i]->coord[GMT_Y][j], GMT_LATSWAP_G2O);	/* Convert to geocentric */
+			GMT_geo_to_cart (pol->segment[i]->coord[GMT_Y][j], pol->segment[i]->coord[GMT_X][j], P_original, TRUE);	/* Convert to a Cartesian x,y,z vector; TRUE since we have degrees */
+			spotter_matrix_vect_mult (R, P_original, P_rotated);				/* Rotate the vector */
 			GMT_cart_to_geo (&pol->segment[i]->coord[GMT_Y][j], &pol->segment[i]->coord[GMT_X][j], P_rotated, TRUE);	/* Recover lon lat representation; TRUE to get degrees */
+			pol->segment[i]->coord[GMT_Y][j] = GMT_lat_swap (pol->segment[i]->coord[GMT_Y][j], GMT_LATSWAP_O2G);	/* Convert back to geodetic */
 			if (pol->segment[i]->coord[GMT_Y][j] < south) south = pol->segment[i]->coord[GMT_Y][j];
 			if (pol->segment[i]->coord[GMT_Y][j] > north) north = pol->segment[i]->coord[GMT_Y][j];
 			if (pol->segment[i]->coord[GMT_Y][j] < spol) spol = pol->segment[i]->coord[GMT_Y][j];
@@ -297,7 +311,7 @@ int main (int argc, char **argv)
 	}
 
 	if (skip_grid) {
-		GMT_free_table (pol);
+		if (!global) GMT_free_table (pol);
 	
 		if (gmtdefs.verbose) fprintf (stderr, "%s: Done!\n", GMT_program);
 		GMT_end (argc, argv);
@@ -305,7 +319,7 @@ int main (int argc, char **argv)
 		exit (EXIT_SUCCESS);
 	}
 	
-	/* Then, find min/max of reconstucted outline */
+	/* Then, find min/max of reconstructed outline */
 	
 	if (fabs (e360 - w360) < 180.0) {
 		west = w360;
@@ -319,7 +333,14 @@ int main (int argc, char **argv)
 		west = 0.0;
 		east = 360.0;
 	}
-	
+	if (global) {
+		west = g_head.x_min;	east = g_head.x_max;
+		south = g_head.y_min;	north = g_head.y_max;
+	}
+	/* Adjust longitude range, as indicated by OUTPUT_DEGREE_FORMAT */
+	GMT_lon_range_adjust (GMT_io.geo.range, &west);
+	GMT_lon_range_adjust (GMT_io.geo.range, &east);
+	if (west >= east) east += 360.0;
 	GMT_grd_init (&r_head, argc, argv, FALSE);
 	r_head.node_offset = g_head.node_offset;
 	r_head.x_inc = g_head.x_inc;
@@ -333,33 +354,37 @@ int main (int argc, char **argv)
 	rotated_grid = (float *) GMT_memory (VNULL, (size_t)(r_head.nx * r_head.ny), sizeof (float), GMT_program);
 	grd_x = (double *) GMT_memory (VNULL, (size_t)r_head.nx, sizeof (double), GMT_program);
 	grd_y = (double *) GMT_memory (VNULL, (size_t)r_head.ny, sizeof (double), GMT_program);
+	grd_yc = (double *) GMT_memory (VNULL, (size_t)r_head.ny, sizeof (double), GMT_program);
 	r_head.xy_off = g_head.xy_off;
 	/* Precalculate node coordinates in both degrees and radians */
 	for (i = 0; i < r_head.nx; i++) grd_x[i] = GMT_i_to_x (i, r_head.x_min, r_head.x_max, r_head.x_inc, r_head.xy_off, r_head.nx);
 	for (j = 0; j < r_head.ny; j++) grd_y[j] = GMT_j_to_y (j, r_head.y_min, r_head.y_max, r_head.y_inc, r_head.xy_off, r_head.ny);
+	for (j = 0; j < r_head.ny; j++) grd_yc[j] = GMT_lat_swap (grd_y[j], GMT_LATSWAP_G2O);
 
 	/* Loop over all nodes in the new rotated grid and find those inside the reconstructed polygon */
 	
 	if (gmtdefs.verbose) fprintf (stderr, "%s: Interpolate reconstructed grid\n", GMT_program);
 
-	make_rot_matrix (x_rot, y_rot, -w_rot, R);	/* Make inverse rotation using negative angle */
+	spotter_make_rot_matrix (x_rot, y_rot, -w_rot, R);	/* Make inverse rotation using negative angle */
 	for (j = ij = 0; j < r_head.ny; j++) {
 		for (i = 0; i < r_head.nx; i++, ij++) {
 			rotated_grid[ij] = GMT_f_NaN;
-			inside = FALSE;
-			k = 0;
-			while (k < pol->n_segments && !inside) {	/* Use degrees since function expects it */
-				inside = (GMT_inonout_sphpol (grd_x[i], grd_y[j], pol->segment[k]) > 0);
-				k++;
+			if (!global) {
+				inside = FALSE;
+				k = 0;
+				while (k < pol->n_segments && !inside) {	/* Use degrees since function expects it */
+					inside = (GMT_inonout_sphpol (grd_x[i], grd_y[j], pol->segment[k]) > 0);
+					k++;
+				}
+				if (!inside) continue;	/* Outside the polygon(s) */
 			}
-			if (!inside) continue;	/* Outside the polygon(s) */
 			
 			/* Here we are inside; get the coordinates and rotate back to original grid coordinates */
 			
-			xx = grd_x[i];	yy = grd_y[j];
-			GMT_geo_to_cart (&yy, &xx, P_rotated, TRUE);	/* Convert degree lon,lat to a Cartesian x,y,z vector */
-			matrix_vect_mult (R, P_rotated, P_original);	/* Rotate the vector */
-			GMT_cart_to_geo (&yy, &xx, P_original, TRUE);	/* Recover degree lon lat representation */
+			GMT_geo_to_cart (grd_yc[j], grd_x[i], P_rotated, TRUE);	/* Convert degree lon,lat to a Cartesian x,y,z vector */
+			spotter_matrix_vect_mult (R, P_rotated, P_original);	/* Rotate the vector */
+			GMT_cart_to_geo (&yy, &xx, P_original, TRUE);		/* Recover degree lon lat representation */
+			yy = GMT_lat_swap (yy, GMT_LATSWAP_O2G);		/* Convert back to geodetic */
 			xx -= 360.0;
 			while (xx < g_head.x_min) xx += 360.0;	/* Make sure we deal with 360 issues */
 			if (node) {
@@ -374,7 +399,7 @@ int main (int argc, char **argv)
 	
 	/* Also loop over original node locations to make sure the nearest nodes are set */
 
-	for (i = 0; i < pol->n_segments; i++) {
+	for (i = 0; !global && i < pol->n_segments; i++) {
 		for (j = 0; j < pol->segment[i]->n_rows; j++) {
 			lon = pol->segment[i]->coord[GMT_X][j];
 			while (lon < r_head.x_min) lon += 360.0;
@@ -387,10 +412,10 @@ int main (int argc, char **argv)
 					if (iii < 0 || iii >= r_head.nx) continue;
 					ij = jjj * r_head.nx + iii;
 					if (!GMT_is_fnan (rotated_grid[ij])) continue;	/* Already done this */
-					xx = grd_x[iii];	yy = grd_y[jjj];
-					GMT_geo_to_cart (&yy, &xx, P_rotated, TRUE);	/* Convert degree lon,lat to a Cartesian x,y,z vector */
-					matrix_vect_mult (R, P_rotated, P_original);	/* Rotate the vector */
-					GMT_cart_to_geo (&xx, &yy, P_original, TRUE);	/* Recover degree lon lat representation */
+					GMT_geo_to_cart (grd_yc[jjj], grd_x[iii], P_rotated, TRUE);	/* Convert degree lon,lat to a Cartesian x,y,z vector */
+					spotter_matrix_vect_mult (R, P_rotated, P_original);	/* Rotate the vector */
+					GMT_cart_to_geo (&xx, &yy, P_original, TRUE);		/* Recover degree lon lat representation */
+					yy = GMT_lat_swap (yy, GMT_LATSWAP_O2G);		/* Convert back to geodetic */
 					ii_o = GMT_x_to_i (xx, g_head.x_min, g_head.x_inc, g_head.xy_off, g_head.nx);
 					if (ii_o < 0 || ii_o >= g_head.nx) continue;
 					jj_o = GMT_y_to_j (yy, g_head.y_min, g_head.y_inc, g_head.xy_off, g_head.ny);
@@ -409,17 +434,15 @@ int main (int argc, char **argv)
 	strcpy (r_head.x_units, "degree");
 	strcpy (r_head.y_units, "degree");
 	sprintf (r_head.remark, "Grid rotated using lon lat omega = %g %g %g", x_rot, y_rot, w_rot);
-	if (GMT_write_grd (outfile, &r_head, rotated_grid, 0.0, 0.0, 0.0, 0.0, GMT_pad, FALSE)) {
-		fprintf (stderr, "%s: Error writing file %s\n", GMT_program, outfile);
-		exit (EXIT_FAILURE);
-	}
+	GMT_err_fail (GMT_write_grd (outfile, &r_head, rotated_grid, 0.0, 0.0, 0.0, 0.0, GMT_pad, FALSE), outfile);
 
 	GMT_free ((void *)original_grid);
 	GMT_free ((void *)rotated_grid);
 	GMT_free ((void *)grd_x);
 	GMT_free ((void *)grd_y);
+	GMT_free ((void *)grd_yc);
 	
-	GMT_free_table (pol);
+	if (!global) GMT_free_table (pol);
 
 	if (gmtdefs.verbose) fprintf (stderr, "%s: Done!\n", GMT_program);
 	GMT_end (argc, argv);
@@ -434,28 +457,28 @@ void get_grid_path (struct GRD_HEADER *h, struct GMT_TABLE **p)
 	 * Note that the path is the same for pixel or grid-registered grids.
 	 */
 
-	int np = 0, add, i, j;
+	GMT_LONG np = 0, add, i, j;
 	struct GMT_TABLE *pol;
 	
-	pol = (struct GMT_TABLE *)GMT_memory (VNULL, 1, sizeof (struct GMT_TABLE), GMT_program);
-	pol->segment = (struct GMT_LINE_SEGMENT **) GMT_memory (VNULL, 1, sizeof (struct GMT_LINE_SEGMENT *), GMT_program);
-	pol->segment[0] = (struct GMT_LINE_SEGMENT *) GMT_memory (VNULL, 1, sizeof (struct GMT_LINE_SEGMENT), GMT_program);
-	pol->segment[0]->coord = (double **)GMT_memory (VNULL, 2, sizeof (double *), GMT_program);
-	pol->segment[0]->min = (double *)GMT_memory (VNULL, 2, sizeof (double), GMT_program);
-	pol->segment[0]->max = (double *)GMT_memory (VNULL, 2, sizeof (double), GMT_program);
+	pol = (struct GMT_TABLE *)GMT_memory (VNULL, (size_t)1, sizeof (struct GMT_TABLE), GMT_program);
+	pol->segment = (struct GMT_LINE_SEGMENT **) GMT_memory (VNULL, (size_t)1, sizeof (struct GMT_LINE_SEGMENT *), GMT_program);
+	pol->segment[0] = (struct GMT_LINE_SEGMENT *) GMT_memory (VNULL, (size_t)1, sizeof (struct GMT_LINE_SEGMENT), GMT_program);
+	pol->segment[0]->coord = (double **)GMT_memory (VNULL, (size_t)2, sizeof (double *), GMT_program);
+	pol->segment[0]->min = (double *)GMT_memory (VNULL, (size_t)2, sizeof (double), GMT_program);
+	pol->segment[0]->max = (double *)GMT_memory (VNULL,(size_t) 2, sizeof (double), GMT_program);
 		
 	/* Add south border w->e */
 	if (h->y_min == -90.0) {	/* If at the S pole we just add it twice for end longitudes */
 		add = 2;
-		pol->segment[0]->coord[GMT_X] = (double *)GMT_memory (VNULL, add, sizeof (double), GMT_program);
-		pol->segment[0]->coord[GMT_Y] = (double *)GMT_memory (VNULL, add, sizeof (double), GMT_program);
+		pol->segment[0]->coord[GMT_X] = (double *)GMT_memory (VNULL, (size_t)add, sizeof (double), GMT_program);
+		pol->segment[0]->coord[GMT_Y] = (double *)GMT_memory (VNULL, (size_t)add, sizeof (double), GMT_program);
 		pol->segment[0]->coord[GMT_X][0] = h->x_min;	pol->segment[0]->coord[GMT_X][1] = h->x_max;
 		pol->segment[0]->coord[GMT_Y][0] = pol->segment[0]->coord[GMT_Y][1] = h->y_min;
 	}
 	else {				/* Loop along south border from west to east */
 		add = h->nx - !h->node_offset;
-		pol->segment[0]->coord[GMT_X] = (double *)GMT_memory (VNULL, add, sizeof (double), GMT_program);
-		pol->segment[0]->coord[GMT_Y] = (double *)GMT_memory (VNULL, add, sizeof (double), GMT_program);
+		pol->segment[0]->coord[GMT_X] = (double *)GMT_memory (VNULL, (size_t)add, sizeof (double), GMT_program);
+		pol->segment[0]->coord[GMT_Y] = (double *)GMT_memory (VNULL, (size_t)add, sizeof (double), GMT_program);
 		for (i = 0; i < add; i++) {
 			pol->segment[0]->coord[GMT_X][i] = GMT_i_to_x (i, h->x_min, h->x_max, h->x_inc, 0.0, h->nx);
 			pol->segment[0]->coord[GMT_Y][i] = h->y_min;
@@ -464,8 +487,8 @@ void get_grid_path (struct GRD_HEADER *h, struct GMT_TABLE **p)
 	np += add;
 	/* Add east border s->n */
 	add = h->ny - !h->node_offset;
-	pol->segment[0]->coord[GMT_X] = (double *)GMT_memory ((void *)pol->segment[0]->coord[GMT_X], add + np, sizeof (double), GMT_program);
-	pol->segment[0]->coord[GMT_Y] = (double *)GMT_memory ((void *)pol->segment[0]->coord[GMT_Y], add + np, sizeof (double), GMT_program);
+	pol->segment[0]->coord[GMT_X] = (double *)GMT_memory ((void *)pol->segment[0]->coord[GMT_X], (size_t)(add + np), sizeof (double), GMT_program);
+	pol->segment[0]->coord[GMT_Y] = (double *)GMT_memory ((void *)pol->segment[0]->coord[GMT_Y], (size_t)(add + np), sizeof (double), GMT_program);
 	for (j = 0; j < add; j++) {	/* Loop along east border from south to north */
 		pol->segment[0]->coord[GMT_X][np+j] = h->x_max;
 		pol->segment[0]->coord[GMT_Y][np+j] = GMT_j_to_y (h->ny - 1 - j, h->y_min, h->y_max, h->y_inc, 0.0, h->ny);
@@ -474,15 +497,15 @@ void get_grid_path (struct GRD_HEADER *h, struct GMT_TABLE **p)
 	/* Add north border e->w */
 	if (h->y_max == 90.0) {	/* If at the N pole we just add it twice for end longitudes */
 		add = 2;
-		pol->segment[0]->coord[GMT_X] = (double *)GMT_memory ((void *)pol->segment[0]->coord[GMT_X], add + np, sizeof (double), GMT_program);
-		pol->segment[0]->coord[GMT_Y] = (double *)GMT_memory ((void *)pol->segment[0]->coord[GMT_Y], add + np, sizeof (double), GMT_program);
+		pol->segment[0]->coord[GMT_X] = (double *)GMT_memory ((void *)pol->segment[0]->coord[GMT_X], (size_t)(add + np), sizeof (double), GMT_program);
+		pol->segment[0]->coord[GMT_Y] = (double *)GMT_memory ((void *)pol->segment[0]->coord[GMT_Y], (size_t)(add + np), sizeof (double), GMT_program);
 		pol->segment[0]->coord[GMT_X][np] = h->x_max;	pol->segment[0]->coord[GMT_X][np+1] = h->x_min;
 		pol->segment[0]->coord[GMT_Y][np] = pol->segment[0]->coord[GMT_Y][np+1] = h->y_max;
 	}
 	else {			/* Loop along north border from east to west */
 		add = h->nx - !h->node_offset;
-		pol->segment[0]->coord[GMT_X] = (double *)GMT_memory ((void *)pol->segment[0]->coord[GMT_X], add + np, sizeof (double), GMT_program);
-		pol->segment[0]->coord[GMT_Y] = (double *)GMT_memory ((void *)pol->segment[0]->coord[GMT_Y], add + np, sizeof (double), GMT_program);
+		pol->segment[0]->coord[GMT_X] = (double *)GMT_memory ((void *)pol->segment[0]->coord[GMT_X], (size_t)(add + np), sizeof (double), GMT_program);
+		pol->segment[0]->coord[GMT_Y] = (double *)GMT_memory ((void *)pol->segment[0]->coord[GMT_Y], (size_t)(add + np), sizeof (double), GMT_program);
 		for (i = 0; i < add; i++) {
 			pol->segment[0]->coord[GMT_X][np+i] = GMT_i_to_x (h->nx - 1 - i, h->x_min, h->x_max, h->x_inc, 0.0, h->nx);
 			pol->segment[0]->coord[GMT_Y][np+i] = h->y_max;
@@ -491,8 +514,8 @@ void get_grid_path (struct GRD_HEADER *h, struct GMT_TABLE **p)
 	np += add;
 	/* Add west border n->s */
 	add = h->ny - !h->node_offset;
-	pol->segment[0]->coord[GMT_X] = (double *)GMT_memory ((void *)pol->segment[0]->coord[GMT_X], add + np + 1, sizeof (double), GMT_program);
-	pol->segment[0]->coord[GMT_Y] = (double *)GMT_memory ((void *)pol->segment[0]->coord[GMT_Y], add + np + 1, sizeof (double), GMT_program);
+	pol->segment[0]->coord[GMT_X] = (double *)GMT_memory ((void *)pol->segment[0]->coord[GMT_X], (size_t)(add + np + 1), sizeof (double), GMT_program);
+	pol->segment[0]->coord[GMT_Y] = (double *)GMT_memory ((void *)pol->segment[0]->coord[GMT_Y], (size_t)(add + np + 1), sizeof (double), GMT_program);
 	for (j = 0; j < add; j++) {	/* Loop along west border from north to south */
 		pol->segment[0]->coord[GMT_X][np+j] = h->x_min;
 		pol->segment[0]->coord[GMT_Y][np+j] = GMT_j_to_y (j, h->y_min, h->y_max, h->y_inc, 0.0, h->ny);
@@ -506,6 +529,7 @@ void get_grid_path (struct GRD_HEADER *h, struct GMT_TABLE **p)
 	pol->segment[0]->min[GMT_Y] = h->y_min;	pol->segment[0]->max[GMT_Y] = h->y_max;
 	pol->segment[0]->pole = 0;
 	pol->n_segments = 1;
+	pol->segment[0]->n_columns = 2;
 	
 	*p = pol;
 }
