@@ -1,11 +1,11 @@
 /*--------------------------------------------------------------------
- *	$Id: hotspotter.c,v 1.22 2006/04/10 05:47:30 pwessel Exp $
+ *	$Id: hotspotter.c 9656 2012-01-13 18:29:02Z pwessel $
  *
- *   Copyright (c) 1999-2006 by P. Wessel
+ *   Copyright (c) 1999-2012 by P. Wessel
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; version 2 of the License.
+ *   the Free Software Foundation; version 2 or any later version.
  *
  *   This program is distributed in the hope that it will be useful,
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -99,7 +99,7 @@
  * -> Plate motion models (stage poles):
  *
  * Duncan, R.A., and D. Clague, 1985, Pacific plate motion recorded by linear
- *    volcanic chains, in: A.E.M. Nairn, F. G. stehli, S. Uyeda (eds.), The
+ *    volcanic chains, in: A.E.M. Nairn, F. G. Stehli, S. Uyeda (eds.), The
  *    Ocean Basins and Margins, Vol. 7A, Plenum, New York, pp. 89-121.
  * Wessel and Kroenke, 1997, (see above)
  *
@@ -110,34 +110,33 @@
 int main (int argc, char **argv)
 {
 
-	int n_smts;		/* Number of seamounts read */
-	int n_stages;		/* Number of stage rotations (poles) */
-	int n_chunk;		/* Number of path values returned by libspotter functions */
-	int n_track;		/* Number of points along a single flowline */
-	int node_x_width;	/* Number of x-nodes covered by the seamount in question (y-dependent) */
-	int node_y_width;	/* Number of y-nodes covered by the seamount */
-	int n_args;		/* Command line argument counter */
-	int n_files = 0;	/* Number of input files on the command line */
-	int node;		/* The current node index */
-	int one_or_zero;	/* Will be 1 or 0 depending on grid format (-F) */
-	int n_fields, n_expected_fields;
-	int n_read = 0;		/* Number of records read */
-
-	size_t n_nodes;		/* Number of nodes in the output CVA grid */
+	GMT_LONG n_smts;		/* Number of seamounts read */
+	GMT_LONG n_stages;		/* Number of stage rotations (poles) */
+	GMT_LONG n_chunk;		/* Number of path values returned by libspotter functions */
+	GMT_LONG n_track;		/* Number of points along a single flowline */
+	GMT_LONG node_x_width;		/* Number of x-nodes covered by the seamount in question (y-dependent) */
+	GMT_LONG node_y_width;		/* Number of y-nodes covered by the seamount */
+	GMT_LONG n_args;		/* Command line argument counter */
+	GMT_LONG n_files = 0;		/* Number of input files on the command line */
+	GMT_LONG node;			/* The current node index */
+	GMT_LONG one_or_zero;		/* Will be 1 or 0 depending on grid format (-F) */
+	GMT_LONG n_fields, n_expected_fields;
+	GMT_LONG n_read = 0;		/* Number of records read */
+	GMT_LONG n_nodes;		/* Number of nodes in the output CVA grid */
+	GMT_LONG node_offset = 0;
 
 	/* Misc. counters */
 
-	int i, j, kx, ky, m, ii, jj, i0, j0, k0;
-	int dummy[4], fno;
+	GMT_LONG i, j, kx, ky, m, ii, jj, i0, j0, k0, dummy[4], fno;
 
-	BOOLEAN	error = FALSE;		/* TRUE when arguments are wrong */
-	BOOLEAN nofile = TRUE;		/* TRUE when data are to be read from stdin */
-	BOOLEAN truncate_ages = FALSE;	/* Truncate all seamounts ages > upper_age */
-	BOOLEAN finite = FALSE;		/* TRUE if stage pole file contains finite rotation poles instead */
-	BOOLEAN done;
-	BOOLEAN resize = FALSE;
+	GMT_LONG	error = FALSE;		/* TRUE when arguments are wrong */
+	GMT_LONG nofile = TRUE;		/* TRUE when data are to be read from stdin */
+	GMT_LONG truncate_ages = FALSE;	/* Truncate all seamounts ages > upper_age */
+	GMT_LONG finite = FALSE;		/* TRUE if stage pole file contains finite rotation poles instead */
+	GMT_LONG normalize = FALSE;	/* TRUE if we want CVA in percents */
+	GMT_LONG done;
 
-	float *CVA;			/* The CVA surface we seek to calculate */
+	float *CVA = NULL;		/* The CVA surface we seek to calculate */
 
 	double sampling_int_in_km;	/* Sampling interval along flowline (in km) */
 	double sampling_factor = 0.5;	/* Sets how many points along flowline per node */
@@ -148,10 +147,10 @@ int main (int argc, char **argv)
 	double r_smt;			/* Seamount radius (input km, stored as radians) */
 	double t_smt;			/* Seamount upper age (up to age of seafloor) */
 	double norm;			/* Normalization factor based on r_smt */
-	double *xpos, *ypos;		/* Coordinates of the output grid (in radians) */
-	double *c;			/* Array with one flowline */
+	double *xpos = NULL, *ypos = NULL;	/* Coordinates of the output grid (in radians) */
+	double *c = NULL;		/* Array with one flowline */
 	double dx, dy;			/* x,y distance from projected seamount center to nearest node */
-	double *latfactor, *ilatfactor;	/* Arrays of latitudinal-dependent x-scales (cos(lat)-stuff) */
+	double *latfactor = NULL, *ilatfactor = NULL;	/* Arrays of latitudinal-dependent x-scales (cos(lat)-stuff) */
 	double x_part, y_part;		/* Components of radius from projected seamount center to current node */
 	double y_part2;			/* y_part squared */
 	double r2;			/* Radius squared from projected seamount center to current node */
@@ -160,22 +159,24 @@ int main (int argc, char **argv)
 	double i_xinc_r, i_yinc_r;	/* Inverse grid spacing in 1/radians */
 	double offset;			/* 0.0 or 0.5, depending onf -F */
 	double half_dx, half_dy;	/* Half grid sizes or 0.0, depending on -F */
-	double *in;			/* GMT read array */
+	double *in = NULL;			/* GMT read array */
+	double south_c, north_c, yg;	/* Same but geocentric */
 
 	char *CVA_file = CNULL;		/* Output name of CVA grid */
 	char *euler_file = CNULL;	/* Input name of file with stage (Euler) rotations */
 	char line[BUFSIZ];		/* Read buffer */
-	char *processed_node;		/* Pointer to array with TRUE/FALSE values for each grid node */
+	char *processed_node = NULL;	/* Pointer to array with TRUE/FALSE values for each grid node */
+	char *not_used = NULL;
 
-	FILE *fp;			/* Input data file pointer */
+	FILE *fp = NULL;		/* Input data file pointer */
 
 	struct GRD_HEADER CVA_map;	/* Header structure for output CVA grid */
 
-	struct EULER *p;		/* Array of structures with Euler stage rotations */
+	struct EULER *p = NULL;		/* Array of structures with Euler stage rotations */
 
 	/* ------------------- END OF DECLARATIONS ------------------------------------------------------------*/
 
-	argc = GMT_begin (argc, argv);			/* Initialize GMT environment */
+	argc = (int)GMT_begin (argc, argv);			/* Initialize GMT environment */
 
 	GMT_grd_init (&CVA_map, argc, argv, FALSE);	/* Initialize grid structure */
 
@@ -227,7 +228,7 @@ int main (int argc, char **argv)
 					upper_age = atof (&argv[i][2]);
 					break;
 				case 'S':
-					resize = TRUE;
+					normalize = TRUE;
 					break;
 				case 'T':
 					truncate_ages = TRUE;
@@ -245,11 +246,13 @@ int main (int argc, char **argv)
 	if (argc == 1 || GMT_give_synopsis_and_exit) {
 		fprintf (stderr, "%s %s - Create CVA image from seamount locations\n\n", GMT_program, SPOTTER_VERSION);
 		fprintf (stderr, "usage: %s [<files>] -E<stage_file> -G<CVAgrid> %s\n", GMT_program, GMT_Id_OPT);
-		fprintf (stderr, "\t%s [-C] [-D<factor>] [-F] [%s] [-N<upper_age>] [-T] [-V] [%s] [%s]\n\n", GMT_Rgeo_OPT, GMT_H_OPT, GMT_t_OPT, GMT_bi_OPT);
+		fprintf (stderr, "\t%s [-C] [-D<factor>] [-F] [%s] [-N<upper_age>]\n", GMT_Rgeo_OPT, GMT_H_OPT);
+		fprintf (stderr, "\t[-S] [-T] [-V] [%s] [%s]\n\n", GMT_t_OPT, GMT_bi_OPT);
 
 		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
 
 		fprintf (stderr, "\t<files> is one or more seamount (x,y,z,r,t) files\n");
+		fprintf (stderr, "\t-E specifies the rotations to be used (see man page for format)\n\n");
 		fprintf (stderr, "\t-G Specify file name for output CVA grid.\n");
 		GMT_explain_option ('H');
 		fprintf (stderr, "\t-I specifies grid interval(s); Append m [or c] to <dx> and/or <dy> for minutes [or seconds].\n");
@@ -259,6 +262,7 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\t-D Scale affecting distance between points along flowline [0.5]\n");
 		fprintf (stderr, "\t-F Force pixel-registration [Default is gridline registration]\n");
 		fprintf (stderr, "\t-N sets upper age in m.y. for seamounts whose plate age is NaN [180]\n");
+		fprintf (stderr, "\t-S Normalize CVA grid to percentages of the CVA maximum\n");
 		fprintf (stderr, "\t-T truncate all ages to max age in stage pole model [Default extrapolates]\n");
 		GMT_explain_option ('V');
 		GMT_explain_option (':');
@@ -268,6 +272,9 @@ int main (int argc, char **argv)
 		exit (EXIT_FAILURE);
 	}
 
+	GMT_check_lattice (&CVA_map.x_inc, &CVA_map.y_inc, &node_offset, NULL);
+	CVA_map.node_offset = (int)node_offset;
+
 	if (!project_info.region_supplied) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR:  Must specify -R option\n", GMT_program);
 		error++;
@@ -292,10 +299,10 @@ int main (int argc, char **argv)
 
 	if (error) exit (EXIT_FAILURE);
 
-	GMT_put_history (argc, argv);	/* Update .gmtcommands4 */
-
 	/* ------------------- END OF PROCESSING COMMAND LINE ARGUMENTS  --------------------------------------*/
 
+	GMT_lat_swap_init ();	/* Initialize auxiliary latitude machinery */
+
 	/* Load in the Euler stage poles */
 
 	n_stages = spotter_init (euler_file, &p, TRUE, finite, FALSE, &upper_age, gmtdefs.verbose);
@@ -317,17 +324,19 @@ int main (int argc, char **argv)
 		half_dx = half_dy = 0.0;
 	}
 
-	CVA_map.nx = irint ((CVA_map.x_max - CVA_map.x_min) / CVA_map.x_inc) + one_or_zero;
-	CVA_map.ny = irint ((CVA_map.y_max - CVA_map.y_min) / CVA_map.y_inc) + one_or_zero;
-	n_nodes = CVA_map.nx * CVA_map.ny;
-	CVA = (float *) GMT_memory (VNULL, n_nodes, sizeof(float), GMT_program);
+	CVA_map.nx = (int)(irint ((CVA_map.x_max - CVA_map.x_min) / CVA_map.x_inc) + one_or_zero);
+	CVA_map.ny = (int)(irint ((CVA_map.y_max - CVA_map.y_min) / CVA_map.y_inc) + one_or_zero);
+	n_nodes = GMT_get_nm (CVA_map.nx, CVA_map.ny);
+	CVA = (float *) GMT_memory (VNULL, (size_t)n_nodes, sizeof(float), GMT_program);
 
-	/* Assing grid-region variables in radians to avoid conversions inside convolution loop */
+	/* Assign grid-region variables in radians to avoid conversions inside convolution loop */
 
 	west   = CVA_map.x_min * D2R;
 	east   = CVA_map.x_max * D2R;
 	south  = CVA_map.y_min * D2R;
 	north  = CVA_map.y_max * D2R;
+	south_c  = GMT_lat_swap (CVA_map.y_min, GMT_LATSWAP_G2O) * D2R;	/* Convert to geocentric */ 
+	north_c  = GMT_lat_swap (CVA_map.y_max, GMT_LATSWAP_G2O) * D2R;	/* Convert to geocentric */ 
 	i_xinc_r = 1.0 / xinc_r;
 	i_yinc_r = 1.0 / yinc_r;
 
@@ -360,7 +369,7 @@ int main (int argc, char **argv)
 
 	if (GMT_io.binary[GMT_IN] && gmtdefs.verbose) {
 		char *type[2] = {"double", "single"};
-		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
+		fprintf (stderr, "%s: Expects %ld-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
 	}
 
 	if (n_files > 0)
@@ -389,7 +398,7 @@ int main (int argc, char **argv)
 
 		if (gmtdefs.verbose) (nofile) ? fprintf (stderr, "%s: Reading stdin\n", GMT_program) : fprintf (stderr, "%s: Working on file %s\n", GMT_program, argv[fno]);
 
-		if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) GMT_fgets (line, BUFSIZ, fp);
+		if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) not_used = GMT_fgets (line, BUFSIZ, fp);
 
 		while ((n_fields = GMT_input (fp, &n_expected_fields, &in)) >= 0 && !(GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
 			n_read++;
@@ -401,20 +410,18 @@ int main (int argc, char **argv)
 			if (GMT_io.status & GMT_IO_EOF) continue;
 
 			if (GMT_io.status & GMT_IO_MISMATCH) {
-				fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %d (skipped)\n", GMT_program, n_fields, n_expected_fields, n_read);
+				fprintf (stderr, "%s: Mismatch between actual (%ld) and expected (%ld) fields near line %ld (skipped)\n", GMT_program, n_fields, n_expected_fields, n_read);
 				continue;
 			}
 
+			in[GMT_Y] = GMT_lat_swap (in[GMT_Y], GMT_LATSWAP_G2O);	/* Convert to geocentric */
+
 			/* STEP 1: Read information about a single seamount from input record */
 
 			x_smt = in[0] * D2R;	/* Seamount positions in RADIANS */
 			y_smt = in[1] * D2R;
 			z_smt = in[2];
 			r_smt = in[3];
-			if (resize) {
-				z_smt = 1.0;
-				/* r_smt = 2.0 * yinc_r * EQ_RAD; */
-			}
 			if (GMT_is_dnan (in[4])) {	/* Age is NaN, assign value */
 				t_smt = upper_age;
 			}
@@ -425,7 +432,7 @@ int main (int argc, char **argv)
 						t_smt = upper_age;
 					}
 					else {
-						fprintf (stderr, "%s: Seamounts near line %d has age (%g) > oldest stage (%g) (skipped)\n", GMT_program, n_read, t_smt, upper_age);
+						fprintf (stderr, "%s: Seamounts near line %ld has age (%g) > oldest stage (%g) (skipped)\n", GMT_program, n_read, t_smt, upper_age);
 						continue;
 					}
 				}
@@ -435,11 +442,11 @@ int main (int argc, char **argv)
 
 			r_smt /= EQ_RAD;				/* Converts radius in km to radians */
 			norm = -4.5 / (r_smt * r_smt);			/* Gaussian normalization */
-			node_y_width = (int)ceil (i_yinc_r * r_smt);	/* y-node coverage */
+			node_y_width = (GMT_LONG)ceil (i_yinc_r * r_smt);	/* y-node coverage */
 
 			/* STEP 2: Calculate this seamount's flowline */
 
-			n_chunk = spotter_forthtrack (&x_smt, &y_smt, &t_smt, 1, p, n_stages, sampling_int_in_km, 0.0, FALSE, NULL, &c);
+			n_chunk = spotter_forthtrack (&x_smt, &y_smt, &t_smt, (GMT_LONG)1, p, n_stages, sampling_int_in_km, 0.0, FALSE, NULL, &c);
 
 			/* STEP 3: Convolve this flowline with seamount shape and add to CVA grid */
 
@@ -458,7 +465,7 @@ int main (int argc, char **argv)
 
 				/* First throw out points outside specified grid region */
 
-				if (c[ky] < south || c[ky] > north) continue;	/* Latitude outside region */
+				if (c[ky] < south_c || c[ky] > north_c) continue;	/* Latitude outside region */
 
 				if (c[kx] > east) c[kx] -= TWO_PI;
 				while (c[kx] < west) c[kx] += TWO_PI;
@@ -466,15 +473,16 @@ int main (int argc, char **argv)
 
 				/* OK, this point is within our region, get node index */
 
-				i = (int) floor (((c[kx] - west)  * i_xinc_r) + offset);
-				j = (int) floor (((north - c[ky]) * i_yinc_r) + offset);
+				yg = GMT_lat_swap (c[ky], GMT_LATSWAP_O2G);	/* Convert back to geodetic */
+				i = (GMT_LONG) floor (((c[kx] - west)  * i_xinc_r) + offset);
+				j = (GMT_LONG) floor (((north - yg) * i_yinc_r) + offset);
 				node = j * CVA_map.nx + i;
 
 				if (!processed_node[node]) {	/* Have not added to the CVA at this node yet */
 
 					/* Shape is z_smt * exp (r^2 * norm) */
 
-					node_x_width = (int) ceil (r_smt * ilatfactor[j]);
+					node_x_width = (GMT_LONG) ceil (r_smt * ilatfactor[j]);
 					dx = c[kx] - xpos[i];
 					dy = c[ky] - ypos[j];
 
@@ -505,7 +513,7 @@ int main (int argc, char **argv)
 
 			n_smts++;	/* Go to next seamount */
 
-			if (gmtdefs.verbose && !(n_smts%100)) fprintf (stderr, "%s: Processed %5d seamounts\r", GMT_program, n_smts);
+			if (gmtdefs.verbose && !(n_smts%100)) fprintf (stderr, "%s: Processed %5ld seamounts\r", GMT_program, n_smts);
 		}
 
 		if (fp != stdin) GMT_fclose (fp);
@@ -513,16 +521,32 @@ int main (int argc, char **argv)
 
 	/* OK, Done processing, time to write out */
 
-	if (gmtdefs.verbose) fprintf (stderr, "%s: Processed %5d seamounts\n", GMT_program, n_smts);
+	if (gmtdefs.verbose) fprintf (stderr, "%s: Processed %5ld seamounts\n", GMT_program, n_smts);
 
+	if (normalize) {	/* Convert CVA values to percent of CVA maximum */
+		size_t k;
+		double scale;
+		
+		if (gmtdefs.verbose) fprintf (stderr, "%s: Normalize CVS grid to percentages of max CVA\n", GMT_program);
+		CVA_map.z_min = +DBL_MAX;
+		CVA_map.z_max = -DBL_MAX;
+		for (k = 0; k < (size_t)n_nodes; k++) {
+			if (GMT_is_fnan (CVA[k])) continue;
+			if (CVA[k] < CVA_map.z_min) CVA_map.z_min = CVA[k];
+			if (CVA[k] > CVA_map.z_max) CVA_map.z_max = CVA[k];
+		}
+		if (gmtdefs.verbose) fprintf (stderr, "%s: CVA min/max: %g %g -> ", GMT_program, CVA_map.z_min, CVA_map.z_max);
+		scale = 100.0 / CVA_map.z_max;
+		for (k = 0; k < (size_t)n_nodes; k++) CVA[k] *= (float)scale;
+		CVA_map.z_min *= scale;
+		CVA_map.z_max *= scale;
+		if (gmtdefs.verbose) fprintf (stderr, "%g %g\n", CVA_map.z_min, CVA_map.z_max);
+	}
 	if (gmtdefs.verbose) fprintf (stderr, "%s: Write CVA grid %s\n", GMT_program, CVA_file);
 
 	dummy[0] = dummy[1] = dummy[2] = dummy[3] = 0;	/* No grid boundary padding today */
 
-	if (GMT_write_grd (CVA_file, &CVA_map, CVA, 0.0, 0.0, 0.0, 0.0, dummy, FALSE)) {
-		fprintf (stderr, "%s: Error writing file %s\n", GMT_program, CVA_file);
-		exit (EXIT_FAILURE);
-	}
+	GMT_err_fail (GMT_write_grd (CVA_file, &CVA_map, CVA, 0.0, 0.0, 0.0, 0.0, dummy, FALSE), CVA_file);
 
 	/* Clean up memory */
 
@@ -532,6 +556,7 @@ int main (int argc, char **argv)
 	GMT_free ((void *)ilatfactor);
 	GMT_free ((void *)xpos);
 	GMT_free ((void *)ypos);
+	GMT_free ((void *)p);
 
 	if (gmtdefs.verbose) fprintf (stderr, "%s: Done\n", GMT_program);
 
