@@ -1,11 +1,11 @@
 /*--------------------------------------------------------------------
- *	$Id: originator.c,v 1.19 2004/04/25 08:47:07 pwessel Exp $
+ *	$Id: originator.c 9681 2012-02-25 20:38:25Z pwessel $
  *
- *   Copyright (c) 2000-2004 by P. Wessel
+ *   Copyright (c) 2000-2012 by P. Wessel
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; version 2 of the License.
+ *   the Free Software Foundation; version 2 or any later version.
  *
  *   This program is distributed in the hope that it will be useful,
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -88,67 +88,68 @@
  
 #include "spotter.h"
 
-#define KM_PR_RAD (R2D * KM_PR_DEG)
+#define KM_PR_RAD (R2D * project_info.DIST_KM_PR_DEG)
 
-struct HOTSPOT {
-	double lon, lat;	/* Current location of hot spot */
+struct HOTSPOT_ORIGINATOR {
+	struct HOTSPOT *h;	/* Pointer to regular HOTSPOT structure */
+	/* Extra variables needed for this program */
 	double np_dist;		/* Distance to nearest point on the current flowline */
 	double np_sign;		/* "Sign" of this distance (see code) */
 	double np_time;		/* Predicted time at nearest point */
 	double np_lon;		/* Longitude of nearest point on the current flowline */
 	double np_lat;		/* Latitude  of nearest point on the current flowline */
-	int nearest;		/* Point id of current flowline node points closest to hotspot */
-	int id;			/* Hot spot id flag */
-	int stage;		/* Stage to which seamount belongs */
-	char name[20];		/* Name of hotspot */
-	char abbrev[4];		/* 3-char abbreviation of hotspot name */
+	GMT_LONG nearest;	/* Point id of current flowline node points closest to hotspot */
+	GMT_LONG stage;		/* Stage to which seamount belongs */
 };
 
-int hotspot_init (char *file, struct HOTSPOT **p);
-int comp_hs (const void *p1, const void *p2);
-
-main (int argc, char **argv)
+int main (int argc, char **argv)
 {
-	int i, j, k, n, kk, ns, nh, nc, np, x, y, n_max_spots, fno, n_files = 0, n_args;
-	int n_fields, n_expected_fields, n_read, n_skipped = 0, n_best_hs = 1, n_out, tdz_output = 0;
-		
-	BOOLEAN	error = FALSE, done, nofile = TRUE, truncate_ages = FALSE, first = TRUE, better;
-	BOOLEAN finite = FALSE;		/* TRUE if stage pole file contains finite rotation poles instead */
-	
-	double x_smt, y_smt, z_smt, r_smt, t_smt, *c, *in, d_km = 5.0, upper_age = 180.0, dist, plon, plat, out[3];
-	double max_dist = 1.0e100, hx_dist, hx_dist_km, dist_NA, dist_NX, del_dist, dt, A[3], H[3], N[3], X[3], dlon;
-	
-	char *hsfile = CNULL, *efile = CNULL, age[32];
-	
-	FILE *fp;
-	
-	struct EULER *p;
-	struct HOTSPOT *hotspot, *hot;
+	GMT_LONG x, y, n_max_spots, fno, n_files = 0, n_args, n_input = 5;
+	GMT_LONG n_fields, n_expected_fields, n_best_hs = 1, tdz_output = 0, n_out = 0;
+	GMT_LONG  i, j, k, n, kk, ns, nh, nc, np, n_read, n_skipped = 0;
+
+	GMT_LONG	error = FALSE, done, nofile = TRUE, truncate_ages = FALSE, first = TRUE, better, use_ID = FALSE;
+	GMT_LONG finite = FALSE, degree = FALSE;		/* fitite is TRUE if stage pole file contains finite rotation poles instead */
+
+	double x_smt, y_smt, z_smt, r_smt, t_smt, *c = NULL, *in = NULL, d_km = 5.0, upper_age = 180.0, dist, out[5];
+	double max_dist = 1.0e100, hx_dist, hx_dist_km, dist_NA, dist_NX, del_dist, dt = 0.0, A[3], H[3], N[3], X[3];
+	double dlon, fix_r, fix_t, yg, yc;
+
+	char *hsfile = CNULL, *efile = CNULL, age[GMT_TEXT_LEN], buffer[BUFSIZ], *not_used = NULL;
+	char fmt1[BUFSIZ], fmt2[BUFSIZ];
+
+	FILE *fp = NULL;
+
+	struct EULER *p = NULL;
+	struct HOTSPOT *orig_hotspot = NULL;
+	struct HOTSPOT_ORIGINATOR *hotspot = NULL, *hot = NULL;
+
+	int comp_hs (const void *p1, const void *p2);
+
+	argc = (int)GMT_begin (argc, argv);
 
-	argc = GMT_begin (argc, argv);
-		
 	GMT_io.in_col_type[0] = GMT_io.out_col_type[0] = GMT_IS_LON;
 	GMT_io.in_col_type[1] = GMT_io.out_col_type[1] = GMT_IS_LAT;
 
 	/* Check command line arguments */
-	
+
 	for (i = 1; i < argc; i++) {
 		if (argv[i][0] == '-') {
 			switch (argv[i][1]) {
-			
+
 				/* Common parameters */
-			
+
 				case 'H':
 				case 'V':
 				case ':':
 				case '\0':
-					error += GMT_get_common_args (argv[i], 0, 0, 0, 0);
+					error += GMT_parse_common_options (argv[i], 0, 0, 0, 0);
 					break;
-				
+
 				/* Supplemental parameters */
-				
+
 				case 'b':
-					error += GMT_io_selection (&argv[i][2]);
+					error += GMT_parse_b_option (&argv[i][2]);
 					break;
 
 				case 'C':	/* Use finite rotation poles */
@@ -167,13 +168,21 @@ main (int argc, char **argv)
 				case 'L':
 					n_out = 3;
 					switch (argv[i][2]) {
+						case 'L':
+							degree = TRUE;
 						case 'l':
 							tdz_output = 3;
 							n_out = 5;
 							break;
 						case 'w':
+						case 'W':
 							tdz_output = 2;
 							break;
+						case 'T':
+							degree = TRUE;
+						case 't':
+							tdz_output = 1;
+							break;
 						default:
 							tdz_output = 1;
 							break;
@@ -182,6 +191,10 @@ main (int argc, char **argv)
 				case 'N':
 					upper_age = atof (&argv[i][2]);
 					break;
+				case 'Q':
+					sscanf (&argv[i][2], "%lg/%lg", &fix_r, &fix_t);
+					n_input = 3;
+					break;
 				case 'S':
 					n_best_hs = atoi (&argv[i][2]);
 					break;
@@ -191,6 +204,9 @@ main (int argc, char **argv)
 				case 'W':
 					max_dist = atof (&argv[i][2]);
 					break;
+				case 'Z':
+					use_ID = TRUE;
+					break;
 				default:
 					error = TRUE;
 					GMT_default_error (argv[i][1]);
@@ -201,39 +217,43 @@ main (int argc, char **argv)
 			n_files++;
 	}
 
-	if (GMT_io.binary[GMT_IN] && gmtdefs.io_header[GMT_IN]) {
+	if (GMT_io.binary[GMT_IN] && GMT_io.io_header[GMT_IN]) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary input data cannot have header -H\n", GMT_program);
 		error++;
 	}
-        if (GMT_io.binary[GMT_IN] && GMT_io.ncol[GMT_IN] == 0) GMT_io.ncol[GMT_IN] = 5;
-        if (GMT_io.binary[GMT_IN] && GMT_io.ncol[GMT_IN] < 5) {
-                fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary input data (-bi) must have at least 5 columns\n", GMT_program);
+        if (GMT_io.binary[GMT_IN] && GMT_io.ncol[GMT_IN] == 0) GMT_io.ncol[GMT_IN] = n_input;
+        if (GMT_io.binary[GMT_IN] && GMT_io.ncol[GMT_IN] < n_input) {
+                fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary input data (-bi) must have at least %ld columns\n", GMT_program, n_input);
 		error++;
 	}
-	
-	if (argc == 1 || GMT_quick) {
+
+	if (argc == 1 || GMT_give_synopsis_and_exit) {
 		fprintf (stderr, "%s %s - Associate seamounts with hotspot point sources\n\n", GMT_program, GMT_VERSION);
 		fprintf (stderr, "usage: %s [<xyfiles>] -E<euler_file> -F<hotspot_file> [-C] [-D<d_km>]\n", GMT_program);
-		fprintf (stderr, "	[-H] [-L] [-N<upper_age>] [-S<n_hs>] [-T] [-V] [-W<maxdist>] [-:]\n\n");
-		
-		if (GMT_quick) exit (-1);
-		
+		fprintf (stderr, "	[-H] [-L[flag]] [-N<upper_age>] [-Qr/t] [-S<n_hs>] [-T] [-V] [-W<maxdist>] [-Z] [%s]\n\n", GMT_t_OPT);
+
+		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
+
 		fprintf (stderr, "\txyfiles is one or more seamount (x,y,z,r,t) files\n");
-		fprintf (stderr, "\t-E Specify file name for Euler poles.\n");
+		fprintf (stderr, "\t-E specifies the rotations to be used (see man page for format)\n\n");
 		fprintf (stderr, "\t-F Specify file name for hotspot locations.\n");
 		fprintf (stderr, "\n\tOPTIONS:\n");
 		fprintf (stderr, "\t-C The file given with -E contains finite rotation poles [Default is stage poles]\n");
 		fprintf (stderr, "\t-D set sampling interval in km along tracks [5].\n");
 		GMT_explain_option ('H');
-		fprintf (stderr, "\t-L Output (time, dist, z) triplets for nearest hotspot only (ignores -S).\n");
+		fprintf (stderr, "\t-L Output information for closest approach for nearest hotspot only (ignores -S).\n");
+		fprintf (stderr, "\t   -Lt gives (time, dist, z) [Default].\n");
 		fprintf (stderr, "\t   -Lw gives (omega, dist, z).\n");
 		fprintf (stderr, "\t   -Ll gives (lon, lat, time, dist, z).\n");
+		fprintf (stderr, "\t   dist is in km; use upper case T,W,L to get dist in spherical degrees.\n");
 		fprintf (stderr, "\t-N set age (in m.y.) for seafloor where age == NaN [180].\n");
+		fprintf (stderr, "\t-Q input files has (x,y,z) only. Append constant r/t to use.\n");
 		fprintf (stderr, "\t-S Report the <n_hs> closest hotSpots [1].\n");
 		fprintf (stderr, "\t-T Truncate seamount ages exceeding the upper age set with -N [no truncation] \n");
 		GMT_explain_option ('V');
 		fprintf (stderr, "\t-W Only report seamounts whose closest encounter to a hotspot is less than <maxdist> km\n");
 		fprintf (stderr, "\t   [Default reports for all seamounts] \n");
+		fprintf (stderr, "\t-Z Write hotspot ID number rather than hotspot TAG\n");
 		GMT_explain_option (':');
 		GMT_explain_option ('i');
 		GMT_explain_option ('n');
@@ -257,19 +277,27 @@ main (int argc, char **argv)
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -W:  Must specify a positive distance in km\n", GMT_program);
 		error = TRUE;
 	}
-	
+
 	if (error) exit (EXIT_FAILURE);
 
-	nh = hotspot_init (hsfile, &hotspot);
+	GMT_lat_swap_init ();	/* Initialize auxiliary latitude machinery */
+
+	nh = spotter_hotspot_init (hsfile, &orig_hotspot);
 	if (n_best_hs <= 0 || n_best_hs > nh) {
-		fprintf (stderr, "%s: GMT SYNTAX ERROR -S:  Give value between 1 and %d\n", GMT_program, nh);
+		fprintf (stderr, "%s: GMT SYNTAX ERROR -S:  Give value between 1 and %ld\n", GMT_program, nh);
 		exit (EXIT_FAILURE);
 	}
 	n_max_spots = MIN (n_best_hs, nh);
 
+	hotspot = (struct HOTSPOT_ORIGINATOR *) GMT_memory (VNULL, (size_t)nh, sizeof (struct HOTSPOT_ORIGINATOR), GMT_program);
+	for (i = 0; i < nh; i++) {
+		hotspot[i].h = &orig_hotspot[i];	/* Point to the original hotspot structures */
+		hotspot[i].np_dist = 1.0e100;
+	}
+	
 	ns = spotter_init (efile, &p, TRUE, finite, FALSE, &upper_age, gmtdefs.verbose);
 
-	hot = (struct HOTSPOT *) GMT_memory (VNULL, nh, sizeof (struct HOTSPOT), GMT_program);
+	hot = (struct HOTSPOT_ORIGINATOR *) GMT_memory (VNULL, (size_t)nh, sizeof (struct HOTSPOT_ORIGINATOR), GMT_program);
 
 	x = (gmtdefs.xy_toggle[1]) ? 1 : 0;	y = 1 - x;		/* Set up which columns have x and y on output */
 
@@ -278,16 +306,22 @@ main (int argc, char **argv)
 	else
 		n_files = 1;
 	n_args = (argc > 1) ? argc : 2;
+
+	sprintf (fmt1, "%s%s%s%s%s%s%s%s%s", gmtdefs.d_format, gmtdefs.field_delimiter, gmtdefs.d_format, gmtdefs.field_delimiter, gmtdefs.d_format, gmtdefs.field_delimiter, gmtdefs.d_format, gmtdefs.field_delimiter, gmtdefs.d_format);
+	if (use_ID)
+		sprintf (fmt2, "%s%%d%s%%ld%s%s%s%s", gmtdefs.field_delimiter, gmtdefs.field_delimiter, gmtdefs.field_delimiter, gmtdefs.d_format, gmtdefs.field_delimiter, gmtdefs.d_format);
+	else
+		sprintf (fmt2, "%s%%s%s%%ld%s%s%s%s", gmtdefs.field_delimiter, gmtdefs.field_delimiter, gmtdefs.field_delimiter, gmtdefs.d_format, gmtdefs.field_delimiter, gmtdefs.d_format);
 	
 	done = FALSE;
-	n_expected_fields = (GMT_io.ncol[GMT_IN]) ? GMT_io.ncol[GMT_IN] : 5;
+	n_expected_fields = (GMT_io.ncol[GMT_IN]) ? GMT_io.ncol[GMT_IN] : n_input;
 	n = 0;
 	if (!tdz_output) n_out = n_expected_fields;
 	for (fno = 1; !done && fno < n_args; fno++) {	/* Loop over all input files */
 
 		if (!nofile && argv[fno][0] == '-') continue;
 		if (nofile) {
-			fp = stdin;
+			fp = GMT_stdin;
 			done = TRUE;
 		}
 		else if ((fp = GMT_fopen (argv[fno], GMT_io.r_mode)) == NULL) {
@@ -296,17 +330,17 @@ main (int argc, char **argv)
 		}
 
 		if (!nofile && gmtdefs.verbose) fprintf (stderr, "%s: Working on file %s\n", GMT_program, argv[fno]);
-		
+
 		n_read = 0;
-		if (gmtdefs.io_header[GMT_IN]) {
-			for (i = 0; i < gmtdefs.n_header_recs; i++) {
-				GMT_fgets (GMT_io.segment_header, BUFSIZ, fp);
-				if (first && !GMT_io.binary[GMT_OUT] && gmtdefs.io_header[GMT_OUT]) fprintf (GMT_stdout, "%s", GMT_io.segment_header);
+		if (GMT_io.io_header[GMT_IN]) {
+			for (i = 0; i < GMT_io.n_header_recs; i++) {
+				not_used = GMT_fgets (GMT_io.segment_header, BUFSIZ, fp);
+				if (first && !GMT_io.binary[GMT_OUT] && GMT_io.io_header[GMT_OUT]) GMT_fputs (GMT_io.segment_header, GMT_stdout);
 				n_read++;
 			}
 			first = FALSE;
 		}
-		
+
 		while ((n_fields = GMT_input (fp, &n_expected_fields, &in)) >= 0 && !(GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
 			n_read++;
 			while ((GMT_io.status & GMT_IO_SEGMENT_HEADER) && !(GMT_io.status & GMT_IO_EOF)) {
@@ -317,9 +351,14 @@ main (int argc, char **argv)
 			if (GMT_io.status & GMT_IO_EOF) continue;
 
 			if (GMT_io.status & GMT_IO_MISMATCH) {
-				fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %d (skipped)\n", GMT_program, n_fields, n_expected_fields, n_read);
+				fprintf (stderr, "%s: Mismatch between actual (%ld) and expected (%ld) fields near line %ld (skipped)\n", GMT_program, n_fields, n_expected_fields, n_read);
 				continue;
 			}
+			
+			if (n_input == 3) {	/* set constant r,t values */
+				in[3] = fix_r;
+				in[4] = fix_t;
+			}
 			if (GMT_is_dnan (in[4])) {	/* Age is NaN, assign value */
 				t_smt = upper_age;
 			}
@@ -330,7 +369,7 @@ main (int argc, char **argv)
 						t_smt = upper_age;
 					}
 					else {
-						fprintf (stderr, "%s: Seamounts near line %d has age (%lg) > oldest stage (%lg) (skipped)\n", GMT_program, n_read, t_smt, upper_age);
+						fprintf (stderr, "%s: Seamounts near line %ld has age (%g) > oldest stage (%g) (skipped)\n", GMT_program, n_read, t_smt, upper_age);
 						continue;
 					}
 				}
@@ -339,22 +378,25 @@ main (int argc, char **argv)
 				n_skipped++;
 				continue;
 			}
+						
 			x_smt = in[0] * D2R;
-			y_smt = in[1] * D2R;
+			y_smt = GMT_lat_swap (in[GMT_Y], GMT_LATSWAP_G2O) * D2R;	/* Convert to geocentric */
 			z_smt = in[2];
 			r_smt = in[3];
 
-			if (gmtdefs.verbose && !(n % 10)) fprintf (stderr, "%s: Working on seamount # %5d\r", GMT_program, n);
+			if (gmtdefs.verbose && !(n % 10)) fprintf (stderr, "%s: Working on seamount # %5ld\r", GMT_program, n);
 
-			nc = spotter_forthtrack (&x_smt, &y_smt, &t_smt, 1, p, ns, d_km, 0.0, TRUE, NULL, &c);
+			nc = spotter_forthtrack (&x_smt, &y_smt, &t_smt, (GMT_LONG)1, p, ns, d_km, 0.0, TRUE, NULL, &c);
 
-			np = (int) c[0];
+			np = (GMT_LONG) c[0];
 
-			memcpy ((void *)hot, (void *)hotspot, nh * sizeof (struct HOTSPOT));
+			memcpy ((void *)hot, (void *)hotspot, nh * sizeof (struct HOTSPOT_ORIGINATOR));
 
 			for (kk = 0, k = 1; kk < np; kk++, k += 3) {	/* For this seamounts track */
+				yg = GMT_lat_swap (R2D*c[k+1], GMT_LATSWAP_O2G);	/* Convert back to geodetic */
 				for (j = 0; j < nh; j++) {	/* For all hotspots */
-					dist = GMT_great_circle_dist (hot[j].lon, hot[j].lat, R2D * c[k], R2D * c[k+1]) * KM_PR_DEG;
+					dist = GMT_great_circle_dist (hot[j].h->lon, hot[j].h->lat, R2D * c[k], yg);
+					if (!degree) dist *= project_info.DIST_KM_PR_DEG;
 					if (dist < hot[j].np_dist) {
 						hot[j].np_dist = dist;
 						hot[j].nearest = kk;	/* Index of nearest point on the flowline */
@@ -362,17 +404,17 @@ main (int argc, char **argv)
 				}
 			}
 			for (j = 0; j < nh; j++) {
-			
-				plon = hot[j].lon;	plat = hot[j].lat;
-				GMT_geo_to_cart (&plat, &plon, H, TRUE);	/* 3-D Cartesian vector of this hotspot */
+
+				yc = GMT_lat_swap (hot[j].h->lat, GMT_LATSWAP_G2O);	/* Convert to geocentric */
+				GMT_geo_to_cart (yc, hot[j].h->lon, H, TRUE);	/* 3-D Cartesian vector of this hotspot */
 
 				/* Fine-tune the nearest point by considering intermediate points along greatcircle between knot points */
-				
+
 				k = 3 * hot[j].nearest + 1;			/* Corresponding index for x into the (x,y,t) array c */
-				GMT_geo_to_cart (&c[k+1], &c[k], N, FALSE);	/* 3-D vector of nearest node to this hotspot */
+				GMT_geo_to_cart (c[k+1], c[k], N, FALSE);	/* 3-D vector of nearest node to this hotspot */
 				better = FALSE;
 				if (hot[j].nearest > 0) {	/* There is a point along the flowline before the nearest node */
-					GMT_geo_to_cart (&c[k-2], &c[k-3], A, FALSE);	/* 3-D vector of end of this segment */
+					GMT_geo_to_cart (c[k-2], c[k-3], A, FALSE);	/* 3-D vector of end of this segment */
 					if (GMT_great_circle_intersection (A, N, H, X, &hx_dist) == 0) {	/* X is between A and N */
 						hx_dist_km = d_acos (hx_dist) * KM_PR_RAD;
 						if (hx_dist_km < hot[j].np_dist) {	/* This intermediate point is even closer */
@@ -387,7 +429,7 @@ main (int argc, char **argv)
 					}
 				}
 				if (hot[j].nearest < (np-1) ) {	/* There is a point along the flowline after the nearest node */
-					GMT_geo_to_cart (&c[k+4], &c[k+3], A, FALSE);	/* 3-D vector of end of this segment */
+					GMT_geo_to_cart (c[k+4], c[k+3], A, FALSE);	/* 3-D vector of end of this segment */
 					if (GMT_great_circle_intersection (A, N, H, X, &hx_dist) == 0) {	/* X is between A and N */
 						hx_dist_km = d_acos (hx_dist) * KM_PR_RAD;
 						if (hx_dist_km < hot[j].np_dist) {	/* This intermediate point is even closer */
@@ -406,28 +448,28 @@ main (int argc, char **argv)
 				}
 				else {	/* Just use node coordinates */
 					hot[j].np_lon  = c[k] * R2D;	/* Longitude of the flowline's closest approach to hotspot */
-					hot[j].np_lat  = c[k+1] * R2D;	/* Latitude  of the flowline's closest approach to hotspot */
+					hot[j].np_lat  = GMT_lat_swap (c[k+1] * R2D, GMT_LATSWAP_O2G);	/* Geodetic latitude  of the flowline's closest approach to hotspot */
 					hot[j].np_time = c[k+2];	/* Predicted time at the flowline's closest approach to hotspot */
 				}
-			
+
 				/* Assign sign to distance: If the vector from the hotspot pointing up along the trail is positive
 				 * x-axis and y-axis is normal to that, flowlines whose closest approach point's longitude is
 				 * further east are said to have negative distance. */
 				 
-				dlon = fmod (hot[j].lon - hot[j].np_lon, 360.0);
+				dlon = fmod (hot[j].h->lon - hot[j].np_lon, 360.0);
 				if (fabs (dlon) > 180.0) dlon = copysign (360.0 - fabs (dlon), -dlon);
 				hot[j].np_sign = copysign (1.0, dlon);
 				 
 				/* Assign stage id for this point on the flowline */
-				
+
 				k = 0;
 				while (k < ns && hot[j].np_time <= p[k].t_stop) k++;
 				hot[j].stage = ns - k;
 				if (hot[j].stage == 0) hot[j].stage++;
 			}
-			
-			if (nh > 1) qsort ((void *)hot, nh, sizeof(struct HOTSPOT), comp_hs);
-			
+
+			if (nh > 1) qsort ((void *)hot, (size_t)nh, sizeof(struct HOTSPOT_ORIGINATOR), comp_hs);
+
 			if (hot[0].np_dist < max_dist) {
 				if (tdz_output == 1) {	/* Want time, dist, z output */
 					out[0] = hot[0].np_time;
@@ -435,7 +477,7 @@ main (int argc, char **argv)
 					out[2] = z_smt;
 					GMT_output (GMT_stdout, n_out, out);
 				}
-				else if (tdz_output == 2) {	/* Want time, dist, z output */
+				else if (tdz_output == 2) {	/* Want omega, dist, z output */
 					out[0] = spotter_t2w (p, ns, hot[0].np_time);
 					out[1] = hot[0].np_dist * hot[0].np_sign;
 					out[2] = z_smt;
@@ -449,73 +491,53 @@ main (int argc, char **argv)
 					out[4] = z_smt;
 					GMT_output (GMT_stdout, n_out, out);
 				}
-				else {	/* COnventional originator output */
+				else {	/* Conventional originator output */
 					if (t_smt == 180.0)
 						strcpy (age, "NaN");
 					else
-						sprintf (age, "%lg\0", t_smt);
-					fprintf (GMT_stdout, "%lg\t%lg\t%lg\t%lg\t%s", in[x], in[y], z_smt, r_smt, age);
-					for (j = 0; j < n_max_spots; j++) fprintf (GMT_stdout, "\t%s\t%d\t%lg\t%lg", hot[j].abbrev, hot[j].stage, hot[j].np_time, hot[j].np_dist);
-					fprintf (GMT_stdout, "\n");
+						sprintf (age, "%g", t_smt);
+					sprintf (buffer, fmt1, in[x], in[y], z_smt, r_smt, age);
+					GMT_fputs (buffer, GMT_stdout);
+					if (use_ID)
+						for (j = 0; j < n_max_spots; j++) {
+							sprintf (buffer, fmt2, hot[j].h->id, hot[j].stage, hot[j].np_time, hot[j].np_dist);
+							GMT_fputs (buffer, GMT_stdout);
+						}
+					else
+						for (j = 0; j < n_max_spots; j++) {
+							sprintf (buffer, fmt2, hot[j].h->abbrev, hot[j].stage, hot[j].np_time, hot[j].np_dist);
+							GMT_fputs (buffer, GMT_stdout);
+						}
+					GMT_fputs ("\n", GMT_stdout);
 				}
 			}
 
 			GMT_free ((void *)c);
 			n++;
 		}
-		
-		if (fp != stdin) fclose (fp);
+
+		if (fp != GMT_stdin) GMT_fclose (fp);
 	}
-		
-	if (gmtdefs.verbose) fprintf (stderr, "%s: Working on seamount # %5d\n", GMT_program, n);
+
+	if (gmtdefs.verbose) fprintf (stderr, "%s: Working on seamount # %5ld\n", GMT_program, n);
 
 	GMT_free ((void *)hotspot);
+	GMT_free ((void *)orig_hotspot);
 	GMT_free ((void *)hot);
 	GMT_free ((void *)p);
-	
+
 	GMT_end (argc, argv);
+
+	exit (EXIT_SUCCESS);
 }
 
 int comp_hs (const void *p1, const void *p2)
 {
-	struct HOTSPOT *a, *b;
+	struct HOTSPOT_ORIGINATOR *a, *b;
 
-	a = (struct HOTSPOT *) p1;
-	b = (struct HOTSPOT *) p2;
+	a = (struct HOTSPOT_ORIGINATOR *) p1;
+	b = (struct HOTSPOT_ORIGINATOR *) p2;
 	if (a->np_dist < b->np_dist) return (-1);
 	if (a->np_dist > b->np_dist) return (1);
 	return (0);
 }
-
-int hotspot_init (char *file, struct HOTSPOT **p)
-{
-	FILE *fp;
-	struct HOTSPOT *e;
-	char buffer[BUFSIZ];
-	int i = 0;
-	size_t n_alloc = GMT_CHUNK;
-
-	if ((fp = fopen (file, "r")) == NULL) {
-		fprintf (stderr, "%s: Cannot open file %s - aborts\n", GMT_program, file);
-		exit (EXIT_FAILURE);
-	}
-
-	e = (struct HOTSPOT *) GMT_memory (VNULL, n_alloc, sizeof (struct HOTSPOT), GMT_program);
-
-	while (fgets (buffer, 512, fp) != NULL) {
-		if (buffer[0] == '#' || buffer[0] == '\n') continue;
-		sscanf (buffer, "%lf %lf %s %s", &e[i].lon, &e[i].lat, e[i].abbrev, e[i].name);
-		e[i].id = i;
-		e[i].np_dist = 1.0e100;
-		i++;
-		if ((size_t)i == n_alloc) {
-			n_alloc += GMT_CHUNK;
-			e = (struct HOTSPOT *) GMT_memory ((void *)e, n_alloc, sizeof (struct HOTSPOT), GMT_program);
-		}
-	}
-	fclose (fp);
-	e = (struct HOTSPOT *) GMT_memory ((void *)e, (size_t)i, sizeof (struct HOTSPOT), GMT_program);
-	*p = e;
-
-	return (i);
-}
