@@ -1,4 +1,4 @@
-/*	$Id: x_over.c,v 1.6 2006/04/10 04:43:31 pwessel Exp $
+/*	$Id: x_over.c 9545 2011-07-27 19:31:54Z pwessel $
  *
  * X_OVER will compute cross-overs between 2 legs (or internal cross-overs
  * if both legs are the same) and write out time,lat,lon,cross-over values,
@@ -54,8 +54,8 @@
 
 typedef int HANDLE;
 
-BOOLEAN over_lap (float xx[2][2], float yy[2][2]);
-BOOLEAN find_cross (double *xc, double *yc, double *tc, double *dc, float *hc, double *xvalues, int *pt, float xx[2][2], float yy[2][2]);
+GMT_LONG over_lap (float xx[2][2], float yy[2][2]);
+GMT_LONG find_cross (double *xc, double *yc, double *tc, double *dc, float *hc, double *xvalues, GMT_LONG *pt, float xx[2][2], float yy[2][2]);
 
 static char *oformat = "%9.5f %9.5f %10.1f %10.1f %9.2f %9.2f %9.2f %8.1f %8.1f %8.1f %5.1f %5.1f\n";
 static int ttime[MAXPOINTS];		/* time (in seconds) at each point */
@@ -71,7 +71,7 @@ static float lat[MAXPOINTS];		/* latitude for each point */
 static float lon[MAXPOINTS];		/* longitude for each point */
 				/* Min/max coordinates for pieces of the legs */
 static float grid_section[4][MAXSECT+1][MAXBLOCK][2];
-BOOLEAN gmt_flag[3] = {FALSE, FALSE, FALSE};		/* TRUE (for each data type) if it exists */
+GMT_LONG gmt_flag[3] = {FALSE, FALSE, FALSE};		/* TRUE (for each data type) if it exists */
 
 int main (int argc, char *argv[])
 {
@@ -79,36 +79,36 @@ int main (int argc, char *argv[])
   char legfile[100];		/* Filename for leg */
   char info[11];		/* Info-header from input-file */
   char tmp[10];
+  
+  GMT_LONG n_pts_pr_blk[2];		/* points pr block for each leg */
+  GMT_LONG last_pt[2];		/* no of points in last section */
+  GMT_LONG pnt_begin[2];		/* first point to check in current section */
+  GMT_LONG pnt_end[2];		/* last point to check in current section */
+  GMT_LONG rec, ind, delta, rec_no;	/* misc. counters */
+  GMT_LONG pnt[2];
 
-  int n_pts_pr_blk[2];		/* points pr block for each leg */
-  int last_pt[2];		/* no of points in last section */
-  int pnt_begin[2];		/* first point to check in current section */
-  int pnt_end[2];		/* last point to check in current section */
-  int rec, ind, delta, rec_no;	/* misc. counters */
-  int pnt[2];
-
-  int nxovers;			/* no of crossovers found */
-  int nlegs;			/* 2 if external, 1 if internal */
+  GMT_LONG nxovers;			/* no of crossovers found */
+  GMT_LONG nlegs;			/* 2 if external, 1 if internal */
   int year[2];			/* year of leg */
-  int n_pts_pr_sect[2];		/* points pr section for each leg */
-  int n_sect_pr_blk[2];		/* sections pr block for each leg */
-  int last_blk[2];		/* no of blocks - 1 for each leg */
-  int last_sect[2];		/* no of sections - 1 for each leg */
-  int blk_begin[2];		/* first block to check for each leg */
-  int blk_end[2];		/* last block to check for each leg */
-  int sect_begin[2];		/* first sect to check in current block */
-  int sect_end[2];		/* last sect to check in current block */
-  int latitude, longitude;	/* lat/lon * 1.0E6 from .gmt-file */
-  int leg, sct[2], blk[2], sect, ntot = 0;
-  int arg, block, mode, i;		/* Misc. counters */
+  GMT_LONG n_pts_pr_sect[2];		/* points pr section for each leg */
+  GMT_LONG n_sect_pr_blk[2];		/* sections pr block for each leg */
+  GMT_LONG last_blk[2];		/* no of blocks - 1 for each leg */
+  GMT_LONG last_sect[2];		/* no of sections - 1 for each leg */
+  GMT_LONG blk_begin[2];		/* first block to check for each leg */
+  GMT_LONG blk_end[2];		/* last block to check for each leg */
+  GMT_LONG sect_begin[2];		/* first sect to check in current block */
+  GMT_LONG sect_end[2];		/* last sect to check in current block */
+  GMT_LONG latitude, longitude;	/* lat/lon * 1.0E6 from .gmt-file */
+  GMT_LONG leg, sct[2], blk[2], sect, ntot = 0;
+  GMT_LONG arg, block, mode, i;		/* Misc. counters */
 
-  BOOLEAN shift_lon = FALSE;	/* TRUE if areas cross datumline */
-  BOOLEAN internal = FALSE;	/* TRUE if leg1 = leg2 */
-  BOOLEAN x_flag;		/* TRUE if crossover is found */
-  BOOLEAN error = FALSE;	/* TRUE for invalid arguments */
-  BOOLEAN verbose = FALSE;	/* Work in silence */
-  BOOLEAN ok;			/* misc. booleans */
-  BOOLEAN first = TRUE;		/* Used to print out header before first xover */
+  GMT_LONG shift_lon = FALSE;	/* TRUE if areas cross datumline */
+  GMT_LONG internal = FALSE;	/* TRUE if leg1 = leg2 */
+  GMT_LONG x_flag;		/* TRUE if crossover is found */
+  GMT_LONG error = FALSE;	/* TRUE for invalid arguments */
+  GMT_LONG verbose = FALSE;	/* Work in silence */
+  GMT_LONG ok;			/* misc. booleans */
+  GMT_LONG first = TRUE;		/* Used to print out header before first xover */
 
   double x_cross, y_cross;	/* Coordinates of crossover */
   double t_old = -1.0e38;	/* Time of previous crossover */
@@ -128,8 +128,9 @@ int main (int argc, char *argv[])
   float h_cross[2];			/* Heading at crossover along each leg */
   float d_factor[3];			/* Scale factors if applicable */
   float last_lon;			/* Used to check if we cross Greenwich */
+  size_t not_used = 0;
 
-  FILE *fp;				/* File pointer */
+  FILE *fp = NULL;			/* File pointer */
 
   d_factor[0] = (float)0.1;			/* Scale mGal*10 to mGal */
   d_factor[1] = d_factor[2] = 1.0;	/* Leave mag/bath untouched */
@@ -249,7 +250,7 @@ int main (int argc, char *argv[])
     }
     ntot += npoints[leg];
     if (ntot > MAXPOINTS) {
-      fprintf(stderr,"x_over : ntot = %d, must allocate more memory\n",ntot);
+      fprintf(stderr,"x_over : ntot = %ld, must allocate more memory\n",ntot);
       exit (EXIT_FAILURE);
     }
     n_pts_pr_sect[leg] = NPPS;
@@ -263,24 +264,24 @@ int main (int argc, char *argv[])
     last_sect[leg] = (npoints[leg]%n_pts_pr_blk[leg])/n_pts_pr_sect[leg];
     last_pt[leg] = npoints[leg] - last_blk[leg]*n_pts_pr_blk[leg]
     		   - last_sect[leg]*n_pts_pr_sect[leg];
-    fread((void *)(&ttime[offset[leg]]), (size_t)4, (size_t)1, fp);
-    fread((void *)(&latitude), (size_t)4, (size_t)1, fp);
-    fread((void *)(&longitude), (size_t)4, (size_t)1, fp);
+    not_used = fread((void *)(&ttime[offset[leg]]), (size_t)4, (size_t)1, fp);
+    not_used = fread((void *)(&latitude), (size_t)4, (size_t)1, fp);
+    not_used = fread((void *)(&longitude), (size_t)4, (size_t)1, fp);
     lat[offset[leg]] = (float)(latitude *0.000001);
     lon[offset[leg]] = (float)(longitude *0.000001);
     /* Make sure geodetic longitudes are used */
     if (lon[offset[leg]] < 0.) lon[offset[leg]] += 360.0;
-    fread((void *)gmt[offset[leg]], (size_t)2, (size_t)3, fp);
+    not_used = fread((void *)gmt[offset[leg]], (size_t)2, (size_t)3, fp);
 
     last_lon = lon[offset[leg]];
     for (rec_no = offset[leg] + 1; rec_no < (offset[leg] + npoints[leg]); rec_no++) {
-      fread((void *)(&ttime[rec_no]), (size_t)4, (size_t)1, fp);
-      fread((void *)(&latitude), (size_t)4, (size_t)1, fp);
-      fread((void *)(&longitude), (size_t)4, (size_t)1, fp);
+      not_used = fread((void *)(&ttime[rec_no]), (size_t)4, (size_t)1, fp);
+      not_used = fread((void *)(&latitude), (size_t)4, (size_t)1, fp);
+      not_used = fread((void *)(&longitude), (size_t)4, (size_t)1, fp);
       lat[rec_no] = (float)(latitude *0.000001);
       lon[rec_no] = (float)(longitude *0.000001);
       if (lon[rec_no] < 0.) lon[rec_no] += 360.0;
-      fread((void *)gmt[rec_no], (size_t)2, (size_t)3, fp);
+      not_used = fread((void *)gmt[rec_no], (size_t)2, (size_t)3, fp);
       for (i = 0; i < 3; i++)
         if (!gmt_flag[i] && gmt[rec_no][i] != NODATA) gmt_flag[i] = TRUE;
       if (fabs(lon[rec_no] - last_lon) > 180.0 ) shift_lon = TRUE;
@@ -531,7 +532,7 @@ int main (int argc, char *argv[])
 		    ym[1][1] = lat[pnt[1]+1];
 		    if (xm[0][1] > xm[1][1]) SWAP(&xm[0][1],&xm[1][1]);
 		    if (ym[0][1] > ym[1][1]) SWAP(&ym[0][1],&ym[1][1]);
-		    delta = abs(pnt[0] - pnt[1]);
+		    delta = GMT_abs(pnt[0] - pnt[1]);
 		    if (over_lap(xm,ym) && delta > 1) {
 		      /* Here we have found a possible crossover. We interpolate to
 		       * find the crossover values for position and time along each
@@ -569,17 +570,17 @@ int main (int argc, char *argv[])
     }
   }
   if (verbose)
-    fprintf(stderr,"x_over: Found %5d cross-overs between %s and %s\n",nxovers,legname[0],legname[1]);
+    fprintf(stderr,"x_over: Found %5ld cross-overs between %s and %s\n",nxovers,legname[0],legname[1]);
   exit (EXIT_SUCCESS);
 }
 
-BOOLEAN find_cross (double *xc, double *yc, double *tc, double *dc, float *hc, double *xvalues, int *pt, float xx[2][2], float yy[2][2])
+GMT_LONG find_cross (double *xc, double *yc, double *tc, double *dc, float *hc, double *xvalues, GMT_LONG *pt, float xx[2][2], float yy[2][2])
 {
   double delx[2], dely[2], grad[2], xc0, t_next;
   double delt[2], dx[2][3];
   double deg_pr_rad = 57.29577951308232087680;
   double ti[6], di[6];
-  int leg, i, j, no_cross, n_int, dtype, i1, i2, n_left, n_right, first, error = 0;
+  GMT_LONG leg, i, j, no_cross, n_int, dtype, i1, i2, n_left, n_right, first, error = 0;
   for (leg = 0; leg < 2; leg++) {	/* Copy pertinent info */
     if (pt[leg] == (npoints[leg_no[leg]] + offset[leg] - 1)) return (FALSE);
     delx[leg] = (double)lon[pt[leg]+1] - (double)lon[pt[leg]];
@@ -616,7 +617,7 @@ BOOLEAN find_cross (double *xc, double *yc, double *tc, double *dc, float *hc, d
   if ((*yc) < yy[0][0] || (*yc) > yy[1][0] || (*yc) < yy[0][1] || (*yc) > yy[1][1]) return (FALSE);
 
   /* if the crossover coincides with one of the input points, we might end up computing this
-   * crossover twice. To avoid duplicates of this kind, allow such crossovers to occus only
+   * crossover twice. To avoid duplicates of this kind, allow such crossovers to occur only
    * at the second point (timewise)
    */
 
@@ -680,11 +681,11 @@ BOOLEAN find_cross (double *xc, double *yc, double *tc, double *dc, float *hc, d
         }
       }
 
-      error = GMT_intpol (ti, di, n_int, 1, &tc[leg], &(dx[leg][dtype]), int_mode);	/* Do the interpolation */
+      error = GMT_intpol (ti, di, n_int, (GMT_LONG)1, &tc[leg], &(dx[leg][dtype]), int_mode);	/* Do the interpolation */
 
       if (error != 0) {	/* Oh shit, what could this mean... */
-        fprintf(stderr,"x_over : Error = %d returned from intpol\n",error);
-        fprintf(stderr,"(pnt[0] = %d, pnt[1] = %d\n", pt[0],pt[1]);
+        fprintf(stderr,"x_over : Error = %ld returned from intpol\n",error);
+        fprintf(stderr,"(pnt[0] = %ld, pnt[1] = %ld\n", pt[0],pt[1]);
         return (FALSE);
       }
     }
@@ -720,7 +721,7 @@ BOOLEAN find_cross (double *xc, double *yc, double *tc, double *dc, float *hc, d
     return (TRUE);
 }
 
-BOOLEAN over_lap (float xx[2][2], float yy[2][2])		/* Checks if the two areas overlap */
+GMT_LONG over_lap (float xx[2][2], float yy[2][2])		/* Checks if the two areas overlap */
 {
   if (xx[1][0] < xx[0][1] || xx[0][0] > xx[1][1]) return (FALSE);
   if (yy[1][0] < yy[0][1] || yy[0][0] > yy[1][1]) return (FALSE);
