@@ -1,12 +1,12 @@
 /*--------------------------------------------------------------------
- *	$Id: xyz2grd.c,v 1.60 2006/10/30 21:18:19 remko Exp $
+ *	$Id: xyz2grd.c 9729 2012-03-21 23:19:37Z pwessel $
  *
- *	Copyright (c) 1991-2006 by P. Wessel and W. H. F. Smith
- *	See COPYING file for copying and redistribution conditions.
+ *	Copyright (c) 1991-2012 by P. Wessel and W. H. F. Smith
+ *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
- *	the Free Software Foundation; version 2 of the License.
+ *	the Free Software Foundation; version 2 or any later version.
  *
  *	This program is distributed in the hope that it will be useful,
  *	but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -39,35 +39,35 @@
 
 struct XYZ2GRD_CTRL {
 	struct A {	/* -A[n|z|u|l] */
-		BOOLEAN active;
+		GMT_LONG active;
 		char mode;
 	} A;
 	struct D {	/* -D<xname>/<yname>/<zname>/<scale>/<offset>/<title>/<remark> */
-		BOOLEAN active;
+		GMT_LONG active;
 		char *information;
 	} D;
 	struct E {	/* -E[<nodata>] */
-		BOOLEAN active;
-		BOOLEAN set;
+		GMT_LONG active;
+		GMT_LONG set;
 		double nodata;
 	} E;
 	struct F {	/* -F */
-		BOOLEAN active;
+		GMT_LONG active;
 	} F;
 	struct G {	/* -G<output_grdfile> */
-		BOOLEAN active;
+		GMT_LONG active;
 		char *file;
 	} G;
 	struct I {	/* -Idx[/dy] */
-		BOOLEAN active;
+		GMT_LONG active;
 		double xinc, yinc;
 	} I;
 	struct N {	/* -N<nodata> */
-		BOOLEAN active;
+		GMT_LONG active;
 		double value;
 	} N;
 	struct S {	/* -S */
-		BOOLEAN active;
+		GMT_LONG active;
 		char *file;
 	} S;
 	struct Z Z;
@@ -75,12 +75,14 @@ struct XYZ2GRD_CTRL {
 
 int main (int argc, char **argv)
 {
-	BOOLEAN error = FALSE, do_grid = FALSE, count = FALSE, average = TRUE;
-	BOOLEAN b_only = FALSE, done = FALSE, nofile = TRUE;
-
-	int i, gmt_ij, ii, jj, z = 2, nm = 0, n_read = 0, n_filled = 0, n_used = 0, fno, n_args, *flag = VNULL;
-	int ij = -1;	/* Will be incremented to 0 or set first time around */
-	int n_empty = 0, n_stuffed = 0, n_bad = 0, n_expected_fields, n_fields, n_confused = 0, n_files = 0, high_low = 0;
+	GMT_LONG error = FALSE, do_grid = FALSE, count = FALSE, average = TRUE;
+	GMT_LONG b_only = FALSE, done = FALSE, nofile = TRUE;
+	
+	GMT_LONG z = 2, fno, n_args, *flag = VNULL;
+	GMT_LONG n_expected_fields, n_fields, n_files = 0, high_low = 0;
+	GMT_LONG i, ii, jj , gmt_ij, nm = 0, n_read = 0, n_filled = 0, n_used = 0;
+	GMT_LONG ij = -1;	/* Will be incremented to 0 or set first time around */
+	GMT_LONG n_empty = 0, n_stuffed = 0, n_bad = 0, n_confused = 0;
 
 	double *in, d_value, wesn[4];
 
@@ -88,17 +90,17 @@ int main (int argc, char **argv)
 
 	char line[BUFSIZ];
 
-	FILE *fp = NULL, *fps = NULL;
+	FILE *fp = NULL;
 
 	struct GRD_HEADER grd;
 	struct GMT_Z_IO io;
-	struct XYZ2GRD_CTRL *Ctrl;
+	struct XYZ2GRD_CTRL *Ctrl = NULL;
 	
-	void *New_Xyz2grd_Ctrl (), Free_Xyz2grd_Ctrl (struct XYZ2GRD_CTRL *C);
+	void *New_xyz2grd_Ctrl (), Free_xyz2grd_Ctrl (struct XYZ2GRD_CTRL *C);
 	
-	argc = GMT_begin (argc, argv);
+	argc = (int)GMT_begin (argc, argv);
 
-	Ctrl = (struct XYZ2GRD_CTRL *)New_Xyz2grd_Ctrl ();	/* Allocate and initialize a new control structure */
+	Ctrl = (struct XYZ2GRD_CTRL *)New_xyz2grd_Ctrl ();	/* Allocate and initialize a new control structure */
 
 	GMT_grd_init (&grd, argc, argv, FALSE);
 
@@ -115,7 +117,7 @@ int main (int argc, char **argv)
 				case ':':
 				case 'f':
 				case '\0':
-					error += GMT_parse_common_options (argv[i], &grd.x_min, &grd.x_max, &grd.y_min, &grd.y_max);
+					error += (GMT_LONG)GMT_parse_common_options (argv[i], &grd.x_min, &grd.x_max, &grd.y_min, &grd.y_max);
 					break;
 
 				case 'A':
@@ -174,7 +176,7 @@ int main (int argc, char **argv)
 					break;
 				case 'Z':
 					Ctrl->Z.active = TRUE;
-					error += GMT_parse_z_io (&argv[i][2], &Ctrl->Z);
+					error += (GMT_LONG)GMT_parse_z_io (&argv[i][2], &Ctrl->Z);
 					break;
 				default:
 					error = TRUE;
@@ -187,15 +189,15 @@ int main (int argc, char **argv)
 	}
 
 	if (argc == 1 || GMT_give_synopsis_and_exit) {
-		fprintf (stderr, "xyz2grd %s - Converting [xy]z data to a GMT grdfile\n\n", GMT_VERSION);
+		fprintf (stderr, "xyz2grd %s - Converting [xy]z data to a GMT grid file\n\n", GMT_VERSION);
 		fprintf (stderr, "usage: xyz2grd [<[xy]zfile(s)>] -G<grdfile> %s %s\n", GMT_I_OPT, GMT_Rgeo_OPT);
 		fprintf (stderr, "\t[-A[n|z|u|l]] [%s] [E[<nodata>]] [-F] [%s]\n", GMT_GRDEDIT, GMT_Ho_OPT);
 		fprintf (stderr, "\t[-N<nodata>] [-S[<zfile]] [-V] [-Z[<flags>]] [%s] [%s] [%s]\n", GMT_t_OPT, GMT_bi_OPT, GMT_f_OPT);
 
 		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
 
-		fprintf (stderr, "\txyzfile(s) is a 1- or 3-column file [or standard input] with [x,y,]z values\n");
-		fprintf (stderr, "\t-G to name the output grdfile.\n");
+		fprintf (stderr, "\txyzfile(s) is a 1- or 3-column file [or standard input] with [x,y,]z values.\n");
+		fprintf (stderr, "\t-G to name the output grid file.\n");
 		GMT_inc_syntax ('I', 0);
 		GMT_explain_option ('R');
 		fprintf (stderr, "\n\tOPTIONS:\n");
@@ -203,50 +205,54 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\t   Append n (-An): Count number of multiple entries per node instead.\n");
 		fprintf (stderr, "\t   Append u (-Au): Keep maximum value if multiple entries per node.\n");
 		fprintf (stderr, "\t   Append l (-Al): Keep minimum value if multiple entries per node.\n");
-		fprintf (stderr, "\t   [Default (no -A option) will compute mean values]\n");
-		fprintf (stderr, "\t-D to enter header information.  Specify '=' to get default value\n");
+		fprintf (stderr, "\t   [Default (no -A option) will compute mean values].\n");
+		fprintf (stderr, "\t-D to enter header information.  Specify '=' to get default value.\n");
 		fprintf (stderr, "\t-E input is an ESRI ArcInfo interchange ASCII grid.  Optionally,\n");
-		fprintf (stderr, "\t   append the nodata value that represents NaNs [obtain from ESRI grid]\n");
+		fprintf (stderr, "\t   append the nodata value that represents nodes that should be NaN\n");
+		fprintf (stderr, "\t   [Normally, a nodata_value declaration may be found in the ESRI grid].\n");
 		fprintf (stderr, "\t   Information for -R , -I, and -F are obtained from the ESRI file.\n");
-		fprintf (stderr, "\t-F will force pixel registration [Default is grid registration]\n");
+		fprintf (stderr, "\t-F will force pixel registration [Default is grid registration].\n");
 		GMT_explain_option ('H');
-		fprintf (stderr, "\t-N set value for nodes without input xyz triplet [Default is NaN]\n");
-		fprintf (stderr, "\t   Z-table entries that equal <nodata> are replaced by NaN\n");
+		fprintf (stderr, "\t-N set value for nodes without input xyz triplet [Default is NaN].\n");
+		fprintf (stderr, "\t   Z-table entries that equal <nodata> are replaced by NaN.\n");
 		fprintf (stderr, "\t-S Requires -Z and will swap the byte-order of the input data and write\n");
-		fprintf (stderr, "\t   the result to <zfile> (or stdout if no file given).  No grdfile created!\n");
+		fprintf (stderr, "\t   the result to <zfile> (or stdout if no file given).  No grid file created!\n");
 		fprintf (stderr, "\t   For this option, only one input file (or stdin) is allowed.\n");
 		GMT_explain_option ('V');
-		fprintf (stderr, "\t-Z sets exact specification of incoming 1-column z-table\n");
-		fprintf (stderr, "\t   If data is in row format, state if first row is at T(op) or B(ottom)\n");
-		fprintf (stderr, "\t     Then, append L or R to indicate starting point in row\n");
-		fprintf (stderr, "\t   If data is in column format, state if first columns is L(left) or R(ight)\n");
-		fprintf (stderr, "\t     Then, append T or B to indicate starting point in column\n");
-		fprintf (stderr, "\t   To skip a header of size <n> bytes, append s<n> [<n> = 0]\n");
-		fprintf (stderr, "\t   To swap byte-order in 2-byte words, append w\n");
-		fprintf (stderr, "\t   Append x if gridline-registered, periodic data in x without repeating column at xmax\n");
-		fprintf (stderr, "\t   Append y if gridline-registered, periodic data in y without repeating row at ymax\n");
+		fprintf (stderr, "\t-Z sets exact specification of incoming 1-column z-table.\n");
+		fprintf (stderr, "\t   If data is in row format, state if first row is at T(op) or B(ottom).\n");
+		fprintf (stderr, "\t     Then, append L or R to indicate starting point in row.\n");
+		fprintf (stderr, "\t   If data is in column format, state if first columns is L(left) or R(ight).\n");
+		fprintf (stderr, "\t     Then, append T or B to indicate starting point in column.\n");
+		fprintf (stderr, "\t   To skip a header of size <n> bytes, append s<n> [<n> = 0].\n");
+		fprintf (stderr, "\t   To swap byte-order in 2-byte words, append w.\n");
+		fprintf (stderr, "\t   Append x if gridline-registered, periodic data in x without repeating column at xmax.\n");
+		fprintf (stderr, "\t   Append y if gridline-registered, periodic data in y without repeating row at ymax.\n");
 		fprintf (stderr, "\t   Specify one of the following data types (all binary except a):\n");
-		fprintf (stderr, "\t     a  Ascii\n");
-		fprintf (stderr, "\t     c  signed 1-byte character\n");
-		fprintf (stderr, "\t     u  unsigned 1-byte character\n");
-		fprintf (stderr, "\t     h  signed short 2-byte integer\n");
-		fprintf (stderr, "\t     H  unsigned short 2-byte integer\n");
-		fprintf (stderr, "\t     i  signed 4-byte integer\n");
-		fprintf (stderr, "\t     I  unsigned 4-byte integer\n");
-		fprintf (stderr, "\t     l  signed long (4- or 8-byte) integer\n");
-		fprintf (stderr, "\t     f  4-byte floating point single precision\n");
-		fprintf (stderr, "\t     d  8-byte floating point double precision\n");
-		fprintf (stderr, "\t   [Default format is scanline orientation in ASCII representation: -ZTLa]\n");
+		fprintf (stderr, "\t     A  Ascii (multiple floating point values per record).\n");
+		fprintf (stderr, "\t     a  Ascii (one value per record).\n");
+		fprintf (stderr, "\t     c  signed 1-byte character.\n");
+		fprintf (stderr, "\t     u  unsigned 1-byte character.\n");
+		fprintf (stderr, "\t     h  signed short 2-byte integer.\n");
+		fprintf (stderr, "\t     H  unsigned short 2-byte integer.\n");
+		fprintf (stderr, "\t     i  signed 4-byte integer.\n");
+		fprintf (stderr, "\t     I  unsigned 4-byte integer.\n");
+		fprintf (stderr, "\t     l  signed long (4- or 8-byte) integer.\n");
+		fprintf (stderr, "\t     f  4-byte floating point single precision.\n");
+		fprintf (stderr, "\t     d  8-byte floating point double precision.\n");
+		fprintf (stderr, "\t   [Default format is scanline orientation in ASCII representation: -ZTLa].\n");
 		fprintf (stderr, "\t   This option assumes all nodes have data values.\n");
 		GMT_explain_option (':');
 		GMT_explain_option ('i');
 		GMT_explain_option ('n');
-		fprintf (stderr, "\t   Default is 3 input columns\n");
+		fprintf (stderr, "\t   Default is 3 input columns.\n");
 		GMT_explain_option ('f');
 		GMT_explain_option ('.');
 		exit (EXIT_FAILURE);
 	}
 
+	GMT_check_lattice (&Ctrl->I.xinc, &Ctrl->I.yinc, &Ctrl->F.active, &Ctrl->I.active);
+
 	if (Ctrl->S.active && !Ctrl->Z.active) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR option -S:  Must also specify -Z\n", GMT_program);
 		error++;
@@ -276,7 +282,7 @@ int main (int argc, char **argv)
 			error++;
 		}
 	}
-	if (!Ctrl->S.active && !Ctrl->G.file) {
+	if (!Ctrl->S.active && !(Ctrl->G.active || Ctrl->G.file)) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR option -G:  Must specify output file\n", GMT_program);
 		error++;
 	}
@@ -293,14 +299,12 @@ int main (int argc, char **argv)
 
 	if (error) exit (EXIT_FAILURE);
 
-	GMT_put_history (argc, argv);	/* Update .gmtcommands4 */
-
 	no_data_f = (float)Ctrl->N.value;
 	
 	if (do_grid) {
 		grd.x_inc = Ctrl->I.xinc;
 		grd.y_inc = Ctrl->I.yinc;
-		grd.node_offset = Ctrl->F.active;
+		grd.node_offset = (int)Ctrl->F.active;
 		GMT_RI_prepare (&grd);	/* Ensure -R -I consistency and set nx, ny */
 	}
 	
@@ -330,7 +334,7 @@ int main (int argc, char **argv)
 
 	if (GMT_io.binary[GMT_IN] && gmtdefs.verbose) {
 		char *type[2] = {"double", "single"};
-		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
+		fprintf (stderr, "%s: Expects %ld-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
 	}
 
 	if (GMT_io.binary[GMT_IN] && GMT_io.in_col_type[2] & GMT_IS_RATIME && GMT_io.single_precision[GMT_IN]) {
@@ -345,20 +349,20 @@ int main (int argc, char **argv)
 	else if (!Ctrl->E.active) {
 		grd.xy_off = 0.5 * grd.node_offset;
 
-		GMT_grd_RI_verify (&grd, 1);
+		GMT_err_fail (GMT_grd_RI_verify (&grd, 1), Ctrl->G.file);
 
 		if (gmtdefs.verbose) fprintf (stderr, "%s: nx = %d  ny = %d\n", GMT_program, grd.nx, grd.ny);
 
-		nm = grd.nx * grd.ny;
+		nm = GMT_get_nm (grd.nx, grd.ny);
 
 		a = (float *) GMT_memory (VNULL, (size_t)nm, sizeof (float), GMT_program);
-		flag = (int *) GMT_memory (VNULL, (size_t)nm, sizeof (int), GMT_program);
+		flag = (GMT_LONG *) GMT_memory (VNULL, (size_t)nm, sizeof (GMT_LONG), GMT_program);
 
-		n_expected_fields = (Ctrl->Z.active) ? 1 : 3;
+		n_expected_fields = (Ctrl->Z.active) ? 1 : Ctrl->A.mode == 'n' ? 2 : 3;
 		z = (Ctrl->Z.active) ? 0 : 2;
 	}
 
-	GMT_set_z_io (&io, &grd);
+	GMT_err_fail (GMT_set_z_io (&io, &grd), Ctrl->G.file);
 
 	GMT_set_xy_domain (wesn, &grd);	/* May include some padding if gridline-registered */
 
@@ -379,16 +383,28 @@ int main (int argc, char **argv)
 			GMT_setmode (GMT_IN);
 #endif
 		}
+#ifdef WIN32
+		else if (Ctrl->E.active && (fp = fopen (argv[fno], GMT_io.r_mode)) == NULL) {
+			fprintf (stderr, "%s: Cannot open file %s\n", GMT_program, argv[fno]);
+			continue;
+		}
 		else if ((fp = GMT_fopen (argv[fno], GMT_io.r_mode)) == NULL) {
 			fprintf (stderr, "%s: Cannot open file %s\n", GMT_program, argv[fno]);
 			continue;
 		}
+#else
+		else if ((fp = GMT_fopen (argv[fno], GMT_io.r_mode)) == NULL) {
+			fprintf (stderr, "%s: Cannot open file %s\n", GMT_program, argv[fno]);
+			continue;
+		}
+#endif
 
 		if (!nofile && gmtdefs.verbose) fprintf (stderr, "%s: Working on file %s\n", GMT_program, argv[fno]);
 
 		if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) GMT_fgets (line, BUFSIZ, fp);
 
 		if (Ctrl->S.active) {	/* Just swap data and exit */
+			FILE *fps = NULL;
 
 			if (gmtdefs.verbose) fprintf (stderr, "%s: Swapping data bytes only\n", GMT_program);
 
@@ -403,7 +419,7 @@ int main (int argc, char **argv)
 				exit (EXIT_FAILURE);
 			}
 
-			if (io.skip) fseek (fp, (long)io.skip, SEEK_CUR);
+			if (io.skip) GMT_fseek (fp, (long)io.skip, SEEK_CUR);
 			while ((io.read_item) (fp, &d_value)) (io.write_item) (fps, d_value);
 
 			if (fp != GMT_stdin) GMT_fclose (fp);
@@ -414,11 +430,11 @@ int main (int argc, char **argv)
 
 		if (Ctrl->Z.active) {	/* Read separately because of all the possible formats */
 			if (Ctrl->N.active && GMT_is_dnan (Ctrl->N.value)) Ctrl->N.active = FALSE;	/* No point testing */
-			if (io.skip) fseek (fp, (long)io.skip, SEEK_CUR);
+			if (io.skip) GMT_fseek (fp, (long)io.skip, SEEK_CUR);
 			while ((io.read_item) (fp, &d_value)) {
 				ij++;
 				if (ij == io.n_expected) {
-					fprintf (stderr, "%s: More than %d records, only %d was expected (aborting)!\n", GMT_program, ij, io.n_expected);
+					fprintf (stderr, "%s: More than %ld records, only %ld was expected (aborting)!\n", GMT_program, ij, io.n_expected);
 					exit (EXIT_FAILURE);
 				}
 				(io.get_gmt_ij) (&io, ij, &gmt_ij);
@@ -429,97 +445,118 @@ int main (int argc, char **argv)
 			}
 			ij++;
 			if (ij != io.n_expected) {	/* Input amount doesn't match expectations */
-				fprintf (stderr, "%s: Found %d records, but %d was expected (aborting)!\n", GMT_program, ij, io.n_expected);
+				fprintf (stderr, "%s: Found %ld records, but %ld was expected (aborting)!\n", GMT_program, ij, io.n_expected);
 				exit (EXIT_FAILURE);
 			}
 
 			GMT_check_z_io (&io, a);	/* This fills in missing periodic row or column */
 		}
 		else if (Ctrl->E.active) {	/* Read an ESRI Arc Interchange grid format in ASCII */
-			int n_left, j;
+			GMT_LONG n_left, j;
 			float value;
 			
-			GMT_fgets (line, BUFSIZ, fp);
+			grd.node_offset = 0;
+			fgets (line, BUFSIZ, fp);
+			GMT_chop (line);					/* Rid the world of CR/LF */
 			if (sscanf (line, "%*s %d", &grd.nx) != 1) {
 				fprintf (stderr, "%s: Error decoding ncols record\n", GMT_program);
 				exit (EXIT_FAILURE);
 			}
-			GMT_fgets (line, BUFSIZ, fp);
+			fgets (line, BUFSIZ, fp);
+			GMT_chop (line);					/* Rid the world of CR/LF */
 			if (sscanf (line, "%*s %d", &grd.ny) != 1) {
 				fprintf (stderr, "%s: Error decoding ncols record\n", GMT_program);
 				exit (EXIT_FAILURE);
 			}
-			GMT_fgets (line, BUFSIZ, fp);
+			fgets (line, BUFSIZ, fp);	GMT_str_tolower (line);
+			GMT_chop (line);					/* Rid the world of CR/LF */
 			if (sscanf (line, "%*s %lf", &grd.x_min) != 1) {
 				fprintf (stderr, "%s: Error decoding xll record\n", GMT_program);
 				exit (EXIT_FAILURE);
 			}
-			GMT_fgets (line, BUFSIZ, fp);
+			if (!strncmp (line, "xllcorner", (size_t)9)) grd.node_offset = 1;	/* Pixel grid */
+			fgets (line, BUFSIZ, fp);	GMT_str_tolower (line);
+			GMT_chop (line);					/* Rid the world of CR/LF */
 			if (sscanf (line, "%*s %lf", &grd.y_min) != 1) {
 				fprintf (stderr, "%s: Error decoding yll record\n", GMT_program);
 				exit (EXIT_FAILURE);
 			}
-			GMT_fgets (line, BUFSIZ, fp);
+			if (!strncmp (line, "yllcorner", (size_t)9)) grd.node_offset = 1;	/* Pixel grid */
+			fgets (line, BUFSIZ, fp);
+			GMT_chop (line);					/* Rid the world of CR/LF */
 			if (sscanf (line, "%*s %lf", &grd.x_inc) != 1) {
 				fprintf (stderr, "%s: Error decoding cellsize record\n", GMT_program);
 				exit (EXIT_FAILURE);
 			}
+			grd.y_inc = grd.x_inc;
+			grd.xy_off = 0.5 * grd.node_offset;
+			grd.x_max = grd.x_min + (grd.nx - 1 + grd.node_offset) * grd.x_inc;
+			grd.y_max = grd.y_min + (grd.ny - 1 + grd.node_offset) * grd.y_inc;
+			
+			GMT_err_fail (GMT_grd_RI_verify (&grd, 1), Ctrl->G.file);
+
+			if (gmtdefs.verbose) fprintf (stderr, "%s: nx = %d  ny = %d\n", GMT_program, grd.nx, grd.ny);
+			nm = n_left = GMT_get_nm (grd.nx, grd.ny);
+
+			a = (float *) GMT_memory (VNULL, (size_t)nm, sizeof (float), GMT_program);
 			if (!Ctrl->E.set) {	/* Only read if not specified in -E */
-				GMT_fgets (line, BUFSIZ, fp);
+				fgets (line, BUFSIZ, fp);
+				GMT_chop (line);					/* Rid the world of CR/LF */
 				if (sscanf (line, "%*s %lf", &Ctrl->E.nodata) != 1) {
 					fprintf (stderr, "%s: Error decoding nodata_value record\n", GMT_program);
 					exit (EXIT_FAILURE);
 				}
 			}
-			grd.y_inc = grd.x_inc;
-			grd.node_offset = 0;
-			if (!strcmp (line, "xllcorner")) grd.node_offset = 1;	/* Pixel grid */
-			grd.xy_off = 0.5 * grd.node_offset;
-			grd.x_max = grd.x_min + (grd.nx - 1 + grd.node_offset) * grd.x_inc;
-			grd.y_max = grd.y_min + (grd.ny - 1 + grd.node_offset) * grd.y_inc;
-			
-			GMT_grd_RI_verify (&grd, 1);
-
-			if (gmtdefs.verbose) fprintf (stderr, "%s: nx = %d  ny = %d\n", GMT_program, grd.nx, grd.ny);
-			nm = n_left = grd.nx * grd.ny;
-
-			a = (float *) GMT_memory (VNULL, (size_t)nm, sizeof (float), GMT_program);
-			/* ESRI grids are scanline oriented (top to bottom) as are GMT grids */
 			i = j = 0;
+			fscanf (fp, "%s", line);	GMT_str_tolower (line);
+			if (!strcmp (line, "nodata_value")) {	/* Found the optional nodata word */
+				fscanf (fp, "%f", &value);
+				if (Ctrl->E.set && !GMT_IS_ZERO (value - Ctrl->E.nodata)) {
+					fprintf (stderr, "%s: Your -E%g overrides the nodata_value of %g found in the ESRI file\n", GMT_program, Ctrl->E.nodata, value);
+				}
+				else
+					Ctrl->E.nodata = value;
+			}
+			else {	/* Instead got the very first data value */
+				value = (float) atof (line);
+				a[0] = ((double)value == Ctrl->E.nodata) ? GMT_f_NaN : value;
+				if (++i == grd.nx) i = 0, j++;
+				n_left--;
+			}
+			/* ESRI grids are scanline oriented (top to bottom) as are GMT grids */
 			while (fscanf (fp, "%f", &value) == 1 && n_left) {
 				ij = j * grd.nx + i;
 				a[ij] = (value == Ctrl->E.nodata) ? GMT_f_NaN : (float) value;
-				i++;
-				if (i == grd.nx) i = 0, j++;
+				if (++i == grd.nx) i = 0, j++;
 				n_left--;
 			}
 			if (n_left) {
-				fprintf (stderr, "%s: Expected %d points, found only %d\n", GMT_program, nm, nm - n_left);
+				fprintf (stderr, "%s: Expected %ld points, found only %ld\n", GMT_program, nm, nm - n_left);
 				exit (EXIT_FAILURE);
 			}
 		}
 		else {	/* Get x, y, z */
-
+			gmtdefs.nan_is_gap = FALSE;	/* Here, all x,y matters */
 			while ((n_fields = GMT_input (fp, &n_expected_fields, &in)) >= 0 && !(GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
 
-				if (GMT_io.status & GMT_IO_MISMATCH) {
-					fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %d\n", GMT_program, n_fields, n_expected_fields, n_read);
-					exit (EXIT_FAILURE);
-				}
 				n_read++;
+				if (GMT_io.status & GMT_IO_MISMATCH) {
+					fprintf (stderr, "%s: Mismatch between actual (%ld) and expected (%ld) fields near line %ld (skipped)\n", GMT_program, n_fields, n_expected_fields, n_read);
+					continue;
+				}
 
-				if (GMT_y_is_outside (in[1],  wesn[2], wesn[3])) continue;	/* Outside y-range */
-				if (GMT_x_is_outside (&in[0], wesn[0], wesn[1])) continue;	/* Outside x-range */
+				if (GMT_y_is_outside (in[GMT_Y],  wesn[2], wesn[3])) continue;	/* Outside y-range */
+				if (GMT_x_is_outside (&in[GMT_X], wesn[0], wesn[1])) continue;	/* Outside x-range */
 
 				/* Ok, we are inside the region - process data */
 
-				ii = GMT_x_to_i (in[0], grd.x_min, grd.x_inc, grd.xy_off, grd.nx);
+				ii = GMT_x_to_i (in[GMT_X], grd.x_min, grd.x_inc, grd.xy_off, grd.nx);
 				if (ii == -1) ii++, n_confused++;
 				if (ii == grd.nx) ii--, n_confused++;
-				jj = GMT_y_to_j (in[1], grd.y_min, grd.y_inc, grd.xy_off, grd.ny);
+				jj = GMT_y_to_j (in[GMT_Y], grd.y_min, grd.y_inc, grd.xy_off, grd.ny);
 				if (jj == -1) jj++, n_confused++;
 				if (jj == grd.ny) jj--, n_confused++;
-				ij = jj * grd.nx + ii;
+				ij = GMT_IJ (jj, ii, grd.nx);
 				if (high_low) {	/* Always come here if looking for extreme values */
 					if (flag[ij]) {	/* Already assigned the first value */
 						if ((high_low == -1 && (in[z] < (double)a[ij])) || (high_low == +1 && (in[z] > (double)a[ij]))) a[ij] = (float)in[z];
@@ -536,12 +573,18 @@ int main (int argc, char **argv)
 				n_used++;
 			}
 		}
+#ifdef WIN32
+		if (fp != GMT_stdin) {
+			(Ctrl->E.active) ? fclose (fp) : GMT_fclose (fp);
+		}
+#else
 		if (fp != GMT_stdin) GMT_fclose (fp);
+#endif
 	}
 
 	if (!(Ctrl->Z.active || Ctrl->E.active)) {	/* xyz data could have resulted in duplicates */
-		if (GMT_io.in_col_type[0] == GMT_IS_LON && (fabs (fabs (grd.x_max - grd.x_min) - 360.0) < GMT_CONV_LIMIT) && !grd.node_offset) {	/* Make sure longitudes got replicated */
-			int j, ij_west, ij_east;
+		if (GMT_io.in_col_type[0] == GMT_IS_LON && GMT_360_RANGE (grd.x_max, grd.x_min) && !grd.node_offset) {	/* Make sure longitudes got replicated */
+			GMT_LONG j, ij_west, ij_east;
 
 			for (j = 0; j < grd.ny; j++) {	/* For each row, look at west and east bin */
 				ij_west = j * grd.nx;
@@ -590,37 +633,34 @@ int main (int argc, char **argv)
 				n_stuffed++;
 			}
 		}
-		GMT_free ((void *)flag);
 		
 		if (gmtdefs.verbose) {
 			sprintf (line, "%s\n", gmtdefs.d_format);
-			fprintf (stderr, "%s:  n_read: %d  n_used: %d  n_filled: %d  n_empty: %d set to ", GMT_program,
+			fprintf (stderr, "%s:  n_read: %ld  n_used: %ld  n_filled: %ld  n_empty: %ld set to ", GMT_program,
 				n_read, n_used, n_filled, n_empty);
 			(GMT_is_dnan (Ctrl->N.value)) ? fprintf (stderr, "NaN\n") : fprintf (stderr, line, Ctrl->N.value);
-			if (n_bad) fprintf (stderr, "%s: %d records unreadable\n", GMT_program, n_bad);
-			if (n_stuffed && !count) fprintf (stderr, "%s: Warning - %d nodes had multiple entries that were processed\n", GMT_program, n_stuffed);
-			if (n_confused) fprintf (stderr, "%s: Warning - %d values gave bad indices: Pixel vs gridline confusion?\n", GMT_program, n_confused);
+			if (n_bad) fprintf (stderr, "%s: %ld records unreadable\n", GMT_program, n_bad);
+			if (n_stuffed && !count) fprintf (stderr, "%s: Warning - %ld nodes had multiple entries that were processed\n", GMT_program, n_stuffed);
+			if (n_confused) fprintf (stderr, "%s: Warning - %ld values gave bad indices: Pixel vs gridline confusion?\n", GMT_program, n_confused);
 		}
 	}
 
-	if (GMT_write_grd (Ctrl->G.file, &grd, a, 0.0, 0.0, 0.0, 0.0, GMT_pad, FALSE)) {
-		fprintf (stderr, "%s: Error writing file %s\n", GMT_program, Ctrl->G.file);
-		exit (EXIT_FAILURE);
-	}
+	GMT_err_fail (GMT_write_grd (Ctrl->G.file, &grd, a, 0.0, 0.0, 0.0, 0.0, GMT_pad, FALSE), Ctrl->G.file);
 
 	GMT_free ((void *)a);
+	if (!Ctrl->E.active) GMT_free ((void *)flag);
 
-	Free_Xyz2grd_Ctrl (Ctrl);	/* Deallocate control structure */
+	Free_xyz2grd_Ctrl (Ctrl);	/* Deallocate control structure */
 
 	GMT_end (argc, argv);
 
 	exit (EXIT_SUCCESS);
 }
 
-void *New_Xyz2grd_Ctrl () {	/* Allocate and initialize a new control structure */
+void *New_xyz2grd_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct XYZ2GRD_CTRL *C;
 	
-	C = (struct XYZ2GRD_CTRL *) GMT_memory (VNULL, 1, sizeof (struct XYZ2GRD_CTRL), "New_Xyz2grd_Ctrl");
+	C = (struct XYZ2GRD_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct XYZ2GRD_CTRL), "New_xyz2grd_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	C->N.value = GMT_d_NaN;
@@ -629,8 +669,9 @@ void *New_Xyz2grd_Ctrl () {	/* Allocate and initialize a new control structure *
 	return ((void *)C);
 }
 
-void Free_Xyz2grd_Ctrl (struct XYZ2GRD_CTRL *C) {	/* Deallocate control structure */
-	if (C->G.file) GMT_free ((void *)C->G.file);	
-	if (C->S.file) GMT_free ((void *)C->S.file);	
+void Free_xyz2grd_Ctrl (struct XYZ2GRD_CTRL *C) {	/* Deallocate control structure */
+	if (C->D.information) free ((void *)C->D.information);	
+	if (C->G.file) free ((void *)C->G.file);	
+	if (C->S.file) free ((void *)C->S.file);	
 	GMT_free ((void *)C);	
 }
